<?xml version="1.0"?>
<!-- 
/*
** Module   : java_templates.tpl
** Abstract : Java AST templates used during Data Model Object generation
**
** Copyright (c) 2005-2018, Golden Code Development Corporation.
**
** _#_ _I_ __Date__ __JPRM__ ____________________________Description____________________________
** 001 ECF 20050816   @22147 Good working version. Provides templates for
**                           the definition of classes, instance
**                           variables, getter and setter methods, and
**                           default constructors for DMO classes. Also
**                           has javadoc comment templates for each of
**                           these constructs.
** 002 ECF 20050902   @22438 Added second (non-default) constructor. This
**                           was needed to support using RecordBuffer as
**                           the DMO superclass.
** 003 ECF 20050924   @22906 Added many templates to support more advanced
**                           DMO generation. DMOs are now split into an
**                           interface and an implementing class. Added
**                           implementation of Persistable interface
**                           methods.
** 004 ECF 20051027   @23205 Changed implementation of property access and
**                           initialization. Added new getter/setter
**                           variants and modified list initialization.
** 005 ECF 20060203   @24340 Added temp table support. New templates for
**                           a temp table interface, class, and a ctor
**                           which accepts a multiplex ID.
** 006 ECF 20060418   @25603 Enhanced support for converting extent
**                           fields. Added templates for array assignment
**                           methods and a method to report extent size.
** 007 ECF 20060807   @28466 All DMO implementation classes now implement
**                           java.io.Serializable. This allows a DMO cache
**                           to be spooled out to disk.
** 008 ECF 20060907   @29315 Added assign_to_idx_getter template. Calls
**                           assign() method on the BaseDataType return
**                           value from an indexed getter method call.
** 009 ECF 20061027   @30769 Changed DMO interface templates. They now
**                           extend the marker interface DataModelObject.
** 010 ECF 20070710   @34425 Optimized DMO setters. Always use version of
**                           wrapper's assign method which accepts 'force'
**                           flag (set to true).
** 011 ECF 20080118   @36872 Fixed composite inner class definition. Must
**                           implement Serializable.
** 012 ECF 20090814   @43626 Added templates for indexed getters/setters.
**                           New variants accept a NumberType for the index
**                           parameter.
** 013 ECF 20130119          Added templates for an inner interface (including
**                           javadoc) within the DMO interface.
** 014 CA  20130209          Emit simple getter for extent fields.
** 015 CA  20131015          Added templates for annotating DMO class and properties with the 
**                           legacy names.
** 016 CA  20131021          Added templates for annotating DMO class with the legacy index
**                           names.
** 017 VMN 20131128          Added primary index support in template for annotating DMO class
**                           with the legacy index name.
** 018 OM  20140117          Added fieldId annotation template token.
** 019 VMN 20140120          Modified legacy_field_annotation.
** 020 VMN 20140201          Added support for custom denormalization of fields with extent.
** 021 VMN 20140422          Added table attribute for LegacyTable.
** 022 VMN 20150205          Templates 'list_prop_scalar_setter_meth' and
**                           list_prop_array_setter_meth
**                           refactored.
** 023 VMN 20150321          Template 'list_prop_scalar_setter_empty_meth' added.
** 024 ECF 20150414          Added and refactored numerous templates to enable downstream,
**                           runtime, bytecode assembly.
** 025 OM  20150824          Added support for super-interfaces for TEMP_TABLES.
** 026 ECF 20160222          Denormalization fixes; implemented generics for composite class
**                           lists.
** 027 ECF 20160321          Added fieldtype annotation to simple_set_from_field.
** 028 SVL 20160502          Added hasValidation annotation for tables with delete validation.
** 029 SVL 20160705          Added templates for legacy index components.
** 030 GES 20171208          Removed force flag from assign().
** 031 GES 20180415          Removed duplicate template.
**     CA  20180510          Allow multiple table annotation to be emitted.
*/
-->
<!--
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
-->

<templates>

   <!-- permanent DMO interface definition -->
   <ast-root name="perm_interface_def" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="KW_INTERFACE" text="${classname}">
         <annotation datatype="java.lang.String" key="extends" value="DataModelObject" />
         <annotation datatype="java.lang.String" key="access" value="public" />
         <annotation datatype="java.lang.String" key="pkgname" value="${pkgname}" />
         <ast id="0" type="CS_INSTANCE_METHODS" text="" />
         <ast id="0" type="CS_INNER_CLASSES" text="" />
      </ast>
   </ast-root>
   
   <!-- permanent top level DMO class definition -->
   <ast-root name="perm_top_class_def" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="KW_CLASS" text="${classname}Impl">
         <annotation datatype="java.lang.String" key="access" value="${access}" />
         <annotation datatype="java.lang.String" key="pkgname" value="${pkgname}" />
         <ast id="0" type="KW_IMPLEMENTS" text="">
            <ast id="0" type="CLASS_REFERENCE" text="Serializable" />
            <ast id="0" type="CLASS_REFERENCE" text="Persistable" />
            <ast id="0" type="CLASS_REFERENCE" text="${classname}" />
         </ast>
         <ast id="0" type="CS_INSTANCE_VARS" text="" />
         <ast id="0" type="CS_CONSTRUCTORS" text="" />
         <ast id="0" type="CS_INSTANCE_METHODS" text="" />
         <ast id="0" type="CS_INNER_CLASSES" text="" />
      </ast>
   </ast-root>
   
   <!-- temporary table interface definition -->
   <ast-root name="temp_interface_def" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="KW_INTERFACE" text="${classname}">
         <annotation datatype="java.lang.String" key="access" value="public" />
         <annotation datatype="java.lang.String" key="pkgname" value="${pkgname}" />
         <annotation datatype="java.lang.String" key="extends" value="Temporary" />
         <ast id="0" type="CS_INSTANCE_METHODS" text="" />
         <ast id="0" type="CS_INNER_CLASSES" text="" />
      </ast>
   </ast-root>
   
   <!-- temporary table super interface definition -->
   <ast-root name="temp_super_interface_def" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="KW_INTERFACE" text="${classname}">
         <annotation datatype="java.lang.String" key="access" value="public" />
         <annotation datatype="java.lang.String" key="pkgname" value="${pkgname}" />
         <annotation datatype="java.lang.String" key="extends" value="${extends}" />
         <ast id="0" type="CS_INNER_CLASSES" text="" />
      </ast>
   </ast-root>
   
   <!-- temporary table top level DMO class definition -->
   <ast-root name="temp_top_class_def" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="KW_CLASS" text="${classname}Impl">
         <annotation datatype="java.lang.String" key="access" value="${access}" />
         <annotation datatype="java.lang.String" key="pkgname" value="${pkgname}" />
         <ast id="0" type="KW_IMPLEMENTS" text="">
            <ast id="0" type="CLASS_REFERENCE" text="Serializable" />
            <ast id="0" type="CLASS_REFERENCE" text="Persistable" />
            <ast id="0" type="CLASS_REFERENCE" text="${classname}" />
         </ast>
         <ast id="0" type="CS_INSTANCE_VARS" text="" />
         <ast id="0" type="CS_CONSTRUCTORS" text="" />
         <ast id="0" type="CS_INSTANCE_METHODS" text="" />
         <ast id="0" type="CS_INNER_CLASSES" text="" />
      </ast>
   </ast-root>
   
   <!-- multiplex ID instance variable javadoc -->
   <ast-root name="javadoc_mplex_inst_var" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="DOC_COMMENT" text="">
         <ast id="0" type="DOC_PARAGRAPH" text="">
            <ast id="0" type="DOC_TEXTRUN" text="Multiplex ID" />
         </ast>
      </ast>
   </ast-root>
   
   <!-- inner interface definition -->
   <ast-root name="inner_interface_def" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="KW_INTERFACE" text="Buf">
         <annotation datatype="java.lang.String" key="extends" value="${ifacename}, Buffer" />
         <!-- there are no additional methods (interface just groups other
              interfaces, but we need at least one child to allow for
              descent/ascent during brew -->
         <ast id="0" type="CS_INSTANCE_METHODS" text="" />
      </ast>
   </ast-root>
   
   <!-- inner super interface definition -->
   <ast-root name="inner_super_interface_def" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="KW_INTERFACE" text="Buf">
         <annotation datatype="java.lang.String" key="extends" value="${ifacename}, Buffer, ${extends}.Buf" />
         <!-- there are no additional methods (interface just groups other
              interfaces, but we need at least one child to allow for
              descent/ascent during brew -->
         <ast id="0" type="CS_INSTANCE_METHODS" text="" />
      </ast>
   </ast-root>
   
   <!-- composite inner class definition -->
   <ast-root name="inner_class_def" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="KW_CLASS" text="${classname}">
         <annotation datatype="java.lang.String" key="access" value="${access}" />
         <ast id="0" type="KW_IMPLEMENTS" text="">
            <ast id="0" type="CLASS_REFERENCE" text="Serializable" />
         </ast>
         <ast id="0" type="CS_INSTANCE_VARS" text="" />
         <ast id="0" type="CS_CONSTRUCTORS" text="" />
         <ast id="0" type="CS_INSTANCE_METHODS" text="" />
      </ast>
   </ast-root>
   
   <!-- array constant declaration with an initializer stub -->
   <ast-root name="array_constant" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="ASSIGN" text="=">
         <ast id="0" type="REFERENCE_DEF" text="${datatype}">
            <annotation datatype="java.lang.String" key="name" value="${varname}" />
            <annotation datatype="java.lang.String" key="access" value="${access}" />
            <annotation datatype="java.lang.Boolean" key="static" value="true" />
            <annotation datatype="java.lang.Boolean" key="final" value="true" />
            <annotation datatype="java.lang.Long" key="dimension" value="${dimension}" />
         </ast>
         <ast id="0" type="INITIALIZER" text="">
            <annotation datatype="java.lang.Boolean" key="peernode" value="true" />
         </ast>
      </ast>
   </ast-root>
   
   <!-- instance variable declaration with an initializer stub -->
   <ast-root name="instance_var_decl" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="REFERENCE_DEF" text="${datatype}">
         <annotation datatype="java.lang.Boolean" key="final" value="${final}" />
         <annotation datatype="java.lang.String" key="name" value="${varname}" />
         <annotation datatype="java.lang.String" key="access" value="${access}" />
      </ast>
   </ast-root>
   
   <!-- annotation with the legacy field -->
   <ast-root name="legacy_field_annotation" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst">
      <ast id="0" type="ANNOTATION" text="LegacyField">
         <annotation datatype="java.lang.Boolean" key="peernode" value="true" />
      </ast>
   </ast-root>
   
   <!-- annotation with the legacy table name -->
   <ast-root name="legacy_table_annotation" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst">
      <ast id="0" type="ANNOTATION" text="LegacyTable">
         <annotation datatype="java.lang.Boolean" key="peernode" value="true" />
      </ast>
   </ast-root>

   <!-- add legacy indexes annotations -->
   <ast-root name="legacy_indexes_annotation" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst">
      <ast id="0" type="ANNOTATION" text="LegacyIndexes">
         <ast id="0" type="INITIALIZER"/>
      </ast>
   </ast-root>
   
   <!-- add legacy index annotation -->
   <ast-root name="legacy_index_annotation" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst">
      <ast id="0" type="ANNOTATION" text="LegacyIndex">
         <ast id="0" type="ASSIGN" text="=">
            <ast id="0" type="REFERENCE" text="name" />
            <ast id="0" type="STRING" text="${legacy_name}" />
         </ast>
         <ast id="0" type="ASSIGN" text="=">
            <ast id="0" type="REFERENCE" text="converted" />
            <ast id="0" type="STRING" text="${conv_name}" />
         </ast>
         <annotation datatype="java.lang.Boolean" key="peernode" value="true" />
      </ast>
   </ast-root>

   <!-- parent for legacy index components -->
   <ast-root name="legacy_index_components" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst">
      <ast id="0" type="ASSIGN" text="=">
         <ast id="0" type="REFERENCE" text="components" />
         <ast id="0" type="INITIALIZER">
            <annotation datatype="java.lang.Boolean" key="peernode" value="true" />
         </ast>
      </ast>
   </ast-root>

   <!-- add legacy index component annotation -->
   <ast-root name="legacy_index_component" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst">
      <ast id="0" type="ANNOTATION" text="LegacyIndexComponent">
         <ast id="0" type="ASSIGN" text="=">
            <ast id="0" type="REFERENCE" text="fieldName" />
            <ast id="0" type="STRING" text="${legacy_name}" />
         </ast>
      </ast>
   </ast-root>
   
   <!-- assignment of an annotation key-value pair with a string value -->
   <ast-root name="annotation_assign_string" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst">
      <ast id="0" type="ASSIGN" text="=">
         <ast id="0" type="REFERENCE" text="${key}" />
         <ast id="0" type="STRING" text="${value}" />
      </ast>
   </ast-root>
   
   <!-- assignment of an annotation key-value pair with a numeric literal value -->
   <ast-root name="annotation_assign_num" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst">
      <ast id="0" type="ASSIGN" text="=">
         <ast id="0" type="REFERENCE" text="${key}" />
         <ast id="0" type="NUM_LITERAL" text="${value}" />
      </ast>
   </ast-root>
   
   <!-- assignment of an annotation key-value pair with a boolean true value -->
   <ast-root name="annotation_assign_true" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst">
      <ast id="0" type="ASSIGN" text="=">
         <ast id="0" type="REFERENCE" text="${key}" />
         <ast id="0" type="BOOL_TRUE" text="true" />
      </ast>
   </ast-root>
   
   <!-- default constructor definition -->
   <ast-root name="default_ctor" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="METHOD_DEF" text="${ctorname}">
         <annotation datatype="java.lang.String" key="asmhint" value="default_ctor" />
         <annotation datatype="java.lang.String" key="access" value="public" />
         <annotation datatype="java.lang.Boolean" key="constructor" value="true" />
         <ast id="0" type="BLOCK" text="">
            <annotation datatype="java.lang.Boolean" key="peernode" value="true" />
         </ast>
      </ast>
   </ast-root>
   
   <!-- multiplex constructor definition (temp tables only) -->
   <ast-root name="multiplex_ctor" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="METHOD_DEF" text="${ctorname}">
         <annotation datatype="java.lang.String" key="asmhint" value="multiplex_ctor" />
         <annotation datatype="java.lang.String" key="access" value="public" />
         <annotation datatype="java.lang.Boolean" key="constructor" value="true" />
         <ast id="0" type="LPARENS" text="">
            <ast id="0" type="REFERENCE_DEF" text="${mplextype}">
               <annotation datatype="java.lang.String" key="name" value="multiplexID" />
            </ast>
         </ast>
         <ast id="0" type="BLOCK" text="">
            <ast id="0" type="STATIC_METHOD_CALL" text="this" />
            <ast id="0" type="ASSIGN" text="">
               <ast id="0" type="${fieldtype}" text="_multiplex" />
               <ast id="0" type="REFERENCE" text="multiplexID" />
            </ast>
         </ast>
      </ast>
   </ast-root>
   
   <!-- skeleton initialize method definition -->
   <ast-root name="initialize_method_def" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="METHOD_DEF" text="initialize">
         <annotation datatype="java.lang.String" key="access" value="private" />
         <ast id="0" type="BLOCK" text="">
            <annotation datatype="java.lang.Boolean" key="peernode" value="true" />
         </ast>
      </ast>
   </ast-root>
   
   <!-- initialize a single, pre-defined instance member variable -->
   <ast-root name="initialize_inst_var" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="ASSIGN" text="=">
         <annotation datatype="java.lang.Boolean" key="peernode" value="true" />
         <ast id="0" type="${fieldtype}" text="${varname}" />
      </ast>
   </ast-root>
   
   <!-- assign a member to null -->
   <ast-root name="assign_reference_null" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="ASSIGN" text="=">
         <ast id="0" type="MEMBER" text="${memname}" />
         <ast id="0" type="NULL_LITERAL" text="" />
      </ast>
   </ast-root>
   
   <!-- list initialization loop for initialize method -->
   <ast-root name="init_list_loop" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="BLOCK_DEF" text="">
         <ast id="0" type="KW_FOR" text="for">
            <ast id="0" type="ASSIGN" text="=">
               <ast id="0" type="REFERENCE_DEF" text="int">
                  <annotation datatype="java.lang.String" key="name" value="i" />
               </ast>
               <ast id="0" type="EXPRESSION" text="">
                  <ast id="0" type="NUM_LITERAL" text="0" />
               </ast>
            </ast>
            <ast id="0" type="EXPRESSION" text="">
               <ast id="0" type="LT" text="&lt;">
                  <ast id="0" type="REFERENCE" text="i" />
                  <ast id="0" type="NUM_LITERAL" text="${extent}" />
               </ast>
            </ast>
            <ast id="0" type="EXPRESSION" text="">
               <ast id="0" type="INCREMENT" text="++">
                  <ast id="0" type="REFERENCE" text="i" />
               </ast>
            </ast>
         </ast>
         <ast id="0" type="BLOCK" text="">
            <annotation datatype="java.lang.Boolean" key="peernode" value="true" />
         </ast>
      </ast>
   </ast-root>
   
   <!-- method call to add an element to a list -->
   <ast-root name="method_call_add" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="METHOD_CALL" text="add">
         <annotation datatype="java.lang.Boolean" key="peernode" value="true" />
         <ast id="0" type="REFERENCE" text="${varname}" />
      </ast>
   </ast-root>
   
   <!-- method call to add a new composite instance to a list -->
   <ast-root name="add_new_composite" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="METHOD_CALL" text="add">
         <ast id="0" type="${fieldtype}" text="${varname}">
            <annotation datatype="java.lang.Long" key="extent" value="${extent}" />
         </ast>
         <ast id="0" type="CONSTRUCTOR" text="${compname}" />
      </ast>
   </ast-root>
   
   <!-- method call to add a null literal to a list -->
   <ast-root name="add_new_null" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="METHOD_CALL" text="add">
         <ast id="0" type="REFERENCE" text="${varname}" />
         <ast id="0" type="NULL_LITERAL" text="" />
      </ast>
   </ast-root>
   
   <!-- instance variable initialization - always initializes to null -->
   <ast-root name="instance_var_init" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="ASSIGN" text="=">
         <annotation datatype="java.lang.Boolean" key="peernode" value="true" />
         <ast id="0" type="REFERENCE_DEF" text="${datatype}">
            <annotation datatype="java.lang.String" key="access" value="${access}" />
            <annotation datatype="java.lang.String" key="name" value="${varname}" />
         </ast>
         <ast id="0" type="NULL_LITERAL" text="" />
      </ast>
   </ast-root>
   
   <!-- define and initialize a List variable -->
   <ast-root name="arraylist_var_def" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="ASSIGN" text="=">
         <ast id="0" type="REFERENCE_DEF" text="List">
            <annotation datatype="java.lang.Boolean" key="final" value="false" />
            <annotation datatype="java.lang.String" key="access" value="${access}" />
            <annotation datatype="java.lang.String" key="name" value="${varname}" />
         </ast>
         <ast id="0" type="CONSTRUCTOR" text="ArrayList">
            <ast id="0" type="NUM_LITERAL" text="${extent}" />
         </ast>
      </ast>
   </ast-root>
   
   <!-- define and initialize a generic List variable -->
   <ast-root name="generic_arraylist_var_def" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="ASSIGN" text="=">
         <ast id="0" type="REFERENCE_DEF" text="List&lt;${compclass}&gt;">
            <annotation datatype="java.lang.Boolean" key="final" value="false" />
            <annotation datatype="java.lang.String" key="access" value="${access}" />
            <annotation datatype="java.lang.String" key="name" value="${varname}" />
         </ast>
         <ast id="0" type="CONSTRUCTOR" text="ArrayList&lt;&gt;">
            <ast id="0" type="NUM_LITERAL" text="${extent}" />
         </ast>
      </ast>
   </ast-root>
   
   <!-- declare a List variable -->
   <ast-root name="arraylist_var_decl" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="REFERENCE_DEF" text="java.util.List">
         <annotation datatype="java.lang.Boolean" key="final" value="false" />
         <annotation datatype="java.lang.String" key="access" value="${access}" />
         <annotation datatype="java.lang.String" key="name" value="${varname}" />
      </ast>
   </ast-root>
   
   <!-- initialize an ArrayList variable initialization -->
   <ast-root name="arraylist_var_init" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="ASSIGN" text="=">
         <ast id="0" type="${fieldtype}" text="${varname}" />
         <ast id="0" type="CONSTRUCTOR" text="java.util.ArrayList">
            <ast id="0" type="NUM_LITERAL" text="${extent}" />
         </ast>
      </ast>
   </ast-root>
   
   <!-- invoke initialize method on object reference -->
   <ast-root name="method_call_initialize" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="METHOD_CALL" text="initialize">
         <ast id="0" type="REFERENCE" text="${varname}" />
      </ast>
   </ast-root>
   
   <!-- invoke assign method on object reference -->
   <ast-root name="method_call_assign" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="METHOD_CALL" text="assign">
         <ast id="0" type="REFERENCE" text="${varname}" />
         <ast id="0" type="REFERENCE" text="this" />
      </ast>
   </ast-root>
   
   <!-- simple return statement - return a variable reference -->
   <ast-root name="simple_ref_return" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="KW_RETURN" text="">
         <ast id="0" type="EXPRESSION" text="">
            <ast id="0" type="REFERENCE" text="${varname}" />
         </ast>
      </ast>
   </ast-root>
   
   <!-- define deepCopy method of Undoable interface -->
   <ast-root name="define_deepCopy" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="METHOD_DEF" text="deepCopy">
         <annotation datatype="java.lang.String" key="asmhint" value="deep_copy" />
         <annotation datatype="java.lang.String" key="access" value="public" />
         <annotation datatype="java.lang.String" key="rettype" value="${datatype}" />
         <ast id="0" type="BLOCK" text="">
            <annotation datatype="java.lang.Boolean" key="peernode" value="true" />
            <ast id="0" type="ASSIGN" text="=">
               <ast id="0" type="REFERENCE_DEF" text="${datatype}">
                  <annotation datatype="java.lang.String" key="name" value="copy" />
               </ast>
               <ast id="0" type="CONSTRUCTOR" text="${classname}Impl" />
            </ast>
         </ast>
      </ast>
   </ast-root>
   
   <!-- define assign method of Undoable interface -->
   <ast-root name="define_assign" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="METHOD_DEF" text="assign">
         <annotation datatype="java.lang.String" key="asmhint" value="assign" />
         <annotation datatype="java.lang.String" key="access" value="public" />
         <annotation datatype="java.lang.String" key="rettype" value="void" />
         <ast id="0" type="LPARENS" text="">
            <ast id="0" type="REFERENCE_DEF" text="${datatype}">
               <annotation datatype="java.lang.String" key="name" value="old" />
            </ast>
         </ast>
         <ast id="0" type="BLOCK" text="">
            <annotation datatype="java.lang.Boolean" key="peernode" value="true" />
            <ast id="0" type="ASSIGN" text="">
               <ast id="0" type="REFERENCE_DEF" text="${classname}Impl">
                  <annotation datatype="java.lang.String" key="name" value="from" />
               </ast>
               <ast id="0" type="CAST" text="${classname}Impl">
                  <ast id="0" type="REFERENCE" text="old" />
               </ast>
            </ast>
         </ast>
      </ast>
   </ast-root>
   
   <!-- initialize composite instance from list item -->
   <ast-root name="init_comp_list_item" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="METHOD_CALL" text="initialize">
         <ast id="0" type="CAST" text="${datatype}">
            <ast id="0" type="METHOD_CALL" text="get">
               <ast id="0" type="REFERENCE" text="${varname}" />
               <ast id="0" type="REFERENCE" text="i" />
            </ast>
         </ast>
      </ast>
   </ast-root>
   
   <!-- invoke List.add() to add an element to a list -->
   <ast-root name="list_add_call" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="METHOD_CALL" text="add">
         <annotation datatype="java.lang.Boolean" key="peernode" value="true" />
         <ast id="0" type="REFERENCE" text="${varname}" />
      </ast>
   </ast-root>
   
   <!-- simple setter using value returned by simple getter -->
   <ast-root name="simple_set_from_get" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="STATIC_METHOD_CALL" text="${settername}">
         <ast id="0" type="METHOD_CALL" text="${gettername}">
            <annotation datatype="java.lang.String" key="fieldtype" value="${fieldtype}" />
            <ast id="0" type="REFERENCE" text="from" />
         </ast>
      </ast>
   </ast-root>
   
   <!-- simple setter using value of a private field -->
   <ast-root name="simple_set_from_field" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="STATIC_METHOD_CALL" text="${settername}">
         <ast id="0" type="MEMBER" text="${fromfield}">
            <annotation datatype="java.lang.String" key="fieldtype" value="${fieldtype}" />
            <ast id="0" type="REFERENCE" text="from" />
         </ast>
      </ast>
   </ast-root>
   
   <!-- indexed setter using value returned by indexed getter -->
   <ast-root name="indexed_set_from_get" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="STATIC_METHOD_CALL" text="${settername}">
         <ast id="0" type="REFERENCE" text="i" />
         <ast id="0" type="METHOD_CALL" text="${gettername}">
            <annotation datatype="java.lang.String" key="fieldtype" value="${fieldtype}" />
            <ast id="0" type="REFERENCE" text="from" />
            <ast id="0" type="REFERENCE" text="i" />
         </ast>
      </ast>
   </ast-root>
   
   <!-- indexed setter using field within a composite element of a list -->
   <ast-root name="indexed_set_from_comp_field" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="STATIC_METHOD_CALL" text="${settername}">
         <ast id="0" type="REFERENCE" text="i" />
         <ast id="0" type="MEMBER" text="${fromfield}">
            <ast id="0" type="METHOD_CALL" text="get" >
               <ast id="0" type="MEMBER" text="${composite}">
                  <ast id="0" type="REFERENCE" text="from" />
               </ast>
               <ast id="0" type="REFERENCE" text="i" />
            </ast>
         </ast>
      </ast>
   </ast-root>

   <!-- simple getter method signature -->
   <ast-root name="simple_getter_sig" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="METHOD_DECL" text="${methodname}">
         <annotation datatype="java.lang.String" key="access" value="public" />
         <annotation datatype="java.lang.String" key="rettype" value="${rettype}" />
      </ast>
   </ast-root>
   
   <!-- simple getter method signature for extent fields -->
   <ast-root name="simple_getter_sig_extent" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="METHOD_DECL" text="${methodname}">
         <annotation datatype="java.lang.String" key="access" value="public" />
         <annotation datatype="java.lang.String" key="rettype" value="${rettype}[]" />
      </ast>
   </ast-root>
   
   <!-- simple setter method signature -->
   <ast-root name="simple_setter_sig" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="METHOD_DECL" text="${methodname}">
         <annotation datatype="java.lang.String" key="access" value="public" />
         <annotation datatype="java.lang.String" key="rettype" value="void" />
         <ast id="0" type="LPARENS" text="">
            <ast id="0" type="REFERENCE_DEF" text="${datatype}">
               <annotation datatype="java.lang.String" key="name" value="${varname}" />
            </ast>
         </ast>
      </ast>
   </ast-root>
   
   <!-- getter method implementation which returns object reference -->
   <ast-root name="direct_getter" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="METHOD_DEF" text="${methodname}">
         <annotation datatype="java.lang.String" key="asmhint" value="direct_getter" />
         <annotation datatype="java.lang.String" key="access" value="public" />
         <annotation datatype="java.lang.String" key="rettype" value="${rettype}" />
         <ast id="0" type="BLOCK" text="">
            <ast id="0" type="KW_RETURN" text="">
               <ast id="0" type="EXPRESSION" text="">
                  <ast id="0" type="${fieldtype}" text="${varname}" />
               </ast>
            </ast>
         </ast>
      </ast>
   </ast-root>
   
   <!-- setter method implementation which resets object reference -->
   <ast-root name="direct_setter" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="METHOD_DEF" text="${methodname}">
         <annotation datatype="java.lang.String" key="asmhint" value="direct_setter" />
         <annotation datatype="java.lang.String" key="access" value="public" />
         <annotation datatype="java.lang.String" key="rettype" value="void" />
         <ast id="0" type="LPARENS" text="">
            <ast id="0" type="REFERENCE_DEF" text="${datatype}">
               <annotation datatype="java.lang.String" key="name" value="${varname}" />
            </ast>
         </ast>
         <ast id="0" type="BLOCK" text="">
            <ast id="0" type="ASSIGN" text="">
               <ast id="0" type="MEMBER" text="${varname}">
                  <ast id="0" type="REFERENCE" text="this" />
               </ast>
               <ast id="0" type="REFERENCE" text="${varname}" />
            </ast>
         </ast>
      </ast>
   </ast-root>
   
   <!-- simple getter method definition -->
   <ast-root name="simple_getter" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="METHOD_DEF" text="${methodname}">
         <annotation datatype="java.lang.String" key="asmhint" value="simple_getter" />
         <annotation datatype="java.lang.String" key="access" value="public" />
         <annotation datatype="java.lang.String" key="rettype" value="${rettype}" />
         <ast id="0" type="BLOCK" text="">
            <ast id="0" type="KW_RETURN" text="">
               <ast id="0" type="LPARENS" text="">
                  <ast id="0" type="EXPRESSION" text="">
                     <ast id="0" type="CONSTRUCTOR" text="${rettype}">
                        <ast id="0" type="${fieldtype}" text="${varname}" />
                     </ast>
                  </ast>
               </ast>
            </ast>
         </ast>
      </ast>
   </ast-root>
   
   <!-- simple getter method definition for extent fields -->
   <ast-root name="simple_getter_extent" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="METHOD_DEF" text="${methodname}">
         <annotation datatype="java.lang.String" key="asmhint" value="simple_getter_extent" />
         <annotation datatype="java.lang.String" key="access" value="public" />
         <annotation datatype="java.lang.String" key="rettype" value="${rettype}[]" />
         <ast id="0" type="BLOCK" text="">
            <ast id="0" type="ASSIGN" text="=">
               <ast id="0" type="REFERENCE_DEF" text="${rettype}[]">
                  <annotation datatype="java.lang.String" key="name" value="_${varname}" />
               </ast>
               <ast id="0" type="CONSTRUCTOR" text="${rettype}">
                  <annotation datatype="java.lang.Long" key="extent" value="${extent}" />
               </ast>
            </ast>
            <ast id="0" type="BLOCK_DEF" text="">
               <ast id="0" type="KW_FOR" text="for">
                  <ast id="0" type="ASSIGN" text="=">
                     <ast id="0" type="REFERENCE_DEF" text="int">
                        <annotation datatype="java.lang.String" key="name" value="i" />
                     </ast>
                     <ast id="0" type="EXPRESSION" text="">
                        <ast id="0" type="NUM_LITERAL" text="0" />
                     </ast>
                  </ast>
                  <ast id="0" type="EXPRESSION" text="">
                     <ast id="0" type="LT" text="&lt;">
                        <ast id="0" type="REFERENCE" text="i" />
                        <ast id="0" type="NUM_LITERAL" text="${extent}" />
                     </ast>
                  </ast>
                  <ast id="0" type="EXPRESSION" text="">
                     <ast id="0" type="INCREMENT" text="++">
                        <ast id="0" type="REFERENCE" text="i" />
                     </ast>
                  </ast>
               </ast>
               <ast id="0" type="BLOCK" text="">
                  <annotation datatype="java.lang.Boolean" key="peernode" value="true" />
                  <ast id="0" type="ASSIGN" text="=">
                     <ast id="0" type="REFERENCE" text="_${varname}">
                        <ast id="0" type="LBRACKET" text="[">
                           <ast id="0" type="REFERENCE" text="i" />
                        </ast>
                     </ast>
                     <ast id="0" type="STATIC_METHOD_CALL" text="${methodname}">
                        <ast id="0" type="REFERENCE" text="i" />
                     </ast>
                  </ast>
               </ast>
            </ast>
            <ast id="0" type="KW_RETURN" text="">
               <ast id="0" type="REFERENCE" text="_${varname}"/>
            </ast>
         </ast>
      </ast>
   </ast-root>
   
   <!-- simple setter method definition -->
   <ast-root name="simple_setter" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="METHOD_DEF" text="${methodname}">
         <annotation datatype="java.lang.String" key="asmhint" value="simple_setter" />
         <annotation datatype="java.lang.String" key="access" value="public" />
         <annotation datatype="java.lang.String" key="rettype" value="void" />
         <ast id="0" type="LPARENS" text="">
            <ast id="0" type="REFERENCE_DEF" text="${datatype}">
               <annotation datatype="java.lang.String" key="name" value="${varname}" />
            </ast>
         </ast>
         <ast id="0" type="BLOCK" text="">
            <ast id="0" type="METHOD_CALL" text="assign">
               <ast id="0" type="MEMBER" text="${varname}">
                  <ast id="0" type="REFERENCE" text="this" />
               </ast>
               <ast id="0" type="REFERENCE" text="${varname}" />
            </ast>
         </ast>
      </ast>
   </ast-root>
   
   <!-- indexed getter method signature -->
   <ast-root name="indexed_getter_sig" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="METHOD_DECL" text="${methodname}">
         <annotation datatype="java.lang.String" key="access" value="public" />
         <annotation datatype="java.lang.String" key="rettype" value="${rettype}" />
         <ast id="0" type="LPARENS" text="">
            <ast id="0" type="REFERENCE_DEF" text="${idxtype}">
               <annotation datatype="java.lang.String" key="name" value="index" />
            </ast>
         </ast>
      </ast>
   </ast-root>
   
   <!-- indexed setter method signature -->
   <ast-root name="indexed_setter_sig" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="METHOD_DECL" text="${methodname}">
         <annotation datatype="java.lang.String" key="access" value="public" />
         <annotation datatype="java.lang.String" key="rettype" value="void" />
         <ast id="0" type="LPARENS" text="">
            <ast id="0" type="REFERENCE_DEF" text="${idxtype}">
               <annotation datatype="java.lang.String" key="name" value="index" />
            </ast>
            <ast id="0" type="REFERENCE_DEF" text="${datatype}">
               <annotation datatype="java.lang.String" key="name" value="element" />
            </ast>
         </ast>
      </ast>
   </ast-root>
   
   <!-- list property size method signature -->
   <ast-root name="list_prop_size_sig" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="METHOD_DECL" text="${methodname}">
         <annotation datatype="java.lang.String" key="access" value="public" />
         <annotation datatype="java.lang.String" key="rettype" value="${rettype}" />
      </ast>
   </ast-root>
   
   <!-- list property size method implementation -->
   <ast-root name="list_prop_size_meth" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="METHOD_DEF" text="${methodname}">
         <annotation datatype="java.lang.String" key="asmhint" value="sizer" />
         <annotation datatype="java.lang.String" key="access" value="public" />
         <annotation datatype="java.lang.String" key="rettype" value="${rettype}" />
         <ast id="0" type="BLOCK" text="">
            <ast id="0" type="KW_RETURN" text="">
               <ast id="0" type="EXPRESSION" text="">
                  <ast id="0" type="LPARENS" text="">
                     <ast id="0" type="CONSTRUCTOR" text="integer">
                        <ast id="0" type="METHOD_CALL" text="size">
                           <ast id="0" type="REFERENCE" text="${varname}" />
                        </ast>
                     </ast>
                  </ast>
               </ast>
            </ast>
         </ast>
      </ast>
   </ast-root>

   <!-- list property size value for method with custom extent -->
   <ast-root name="list_prop_size_meth_custom_extent" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="METHOD_DEF" text="${methodname}">
         <annotation datatype="java.lang.String" key="access" value="public" />
         <annotation datatype="java.lang.String" key="rettype" value="${rettype}" />
         <ast id="0" type="BLOCK" text="">
            <ast id="0" type="KW_RETURN" text="">
               <ast id="0" type="EXPRESSION" text="">
                  <ast id="0" type="LPARENS" text="">
                     <ast id="0" type="CONSTRUCTOR" text="integer">
                        <ast id="0" type="REFERENCE" text="${varname}" />
                     </ast>
                  </ast>
               </ast>
            </ast>
         </ast>
      </ast>
   </ast-root>

   <!-- list property scalar value setter method signature -->
   <ast-root name="list_prop_scalar_setter_sig" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="METHOD_DECL" text="${methodname}">
         <annotation datatype="java.lang.String" key="access" value="public" />
         <annotation datatype="java.lang.String" key="rettype" value="void" />
         <ast id="0" type="LPARENS" text="">
            <ast id="0" type="REFERENCE_DEF" text="${datatype}">
               <annotation datatype="java.lang.String" key="name" value="element" />
            </ast>
         </ast>
      </ast>
   </ast-root>
   
   <!-- list property scalar value setter method implementation -->
   <ast-root name="list_prop_scalar_setter_meth" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="METHOD_DEF" text="${methodname}">
         <annotation datatype="java.lang.String" key="asmhint" value="indexed_setter_from_scalar" />
         <annotation datatype="java.lang.String" key="access" value="public" />
         <annotation datatype="java.lang.String" key="rettype" value="void" />
         <ast id="0" type="LPARENS" text="">
            <ast id="0" type="REFERENCE_DEF" text="${datatype}">
               <annotation datatype="java.lang.String" key="name" value="element" />
            </ast>
         </ast>
         <ast id="0" type="BLOCK" text="">
            <ast id="0" type="ASSIGN" text="">
               <ast id="0" type="REFERENCE_DEF" text="int">
                  <annotation datatype="java.lang.String" key="name" value="len" />
               </ast>
               <ast id="0" type="METHOD_CALL" text="${mname}">
                  <ast id="0" type="REFERENCE" text="${varname}" />
               </ast>
            </ast>
            <ast id="0" type="BLOCK_DEF" text="">
               <ast id="0" type="KW_FOR" text="for">
                  <ast id="0" type="ASSIGN" text="=">
                     <ast id="0" type="REFERENCE_DEF" text="int">
                        <annotation datatype="java.lang.String" key="name" value="i" />
                     </ast>
                     <ast id="0" type="EXPRESSION" text="">
                        <ast id="0" type="NUM_LITERAL" text="0" />
                     </ast>
                  </ast>
                  <ast id="0" type="EXPRESSION" text="">
                     <ast id="0" type="LT" text="">
                        <ast id="0" type="REFERENCE" text="i" />
                        <ast id="0" type="REFERENCE" text="len" />
                     </ast>
                  </ast>
                  <ast id="0" type="EXPRESSION" text="">
                     <ast id="0" type="INCREMENT" text="++">
                        <ast id="0" type="REFERENCE" text="i" />
                     </ast>
                  </ast>
               </ast>
               <ast id="0" type="BLOCK" text="">
                  <ast id="0" type="STATIC_METHOD_CALL" text="${methodname}">
                     <annotation datatype="java.lang.String" key="fieldtype" value="${fieldtype}" />
                     <ast id="0" type="REFERENCE" text="i" />
                     <ast id="0" type="REFERENCE" text="element" />
                  </ast>
               </ast>
            </ast>
         </ast>
      </ast>
   </ast-root>
   
   <!-- simple setter method definition stub for extent fields -->
   <ast-root name="list_prop_scalar_setter_empty_meth" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="METHOD_DEF" text="${methodname}">
         <annotation datatype="java.lang.String" key="access" value="public" />
         <annotation datatype="java.lang.String" key="rettype" value="void" />
         <ast id="0" type="LPARENS" text="">
            <ast id="0" type="REFERENCE_DEF" text="${datatype}">
               <annotation datatype="java.lang.String" key="name" value="element" />
            </ast>
         </ast>
         <ast id="0" type="BLOCK" text="">
            <annotation datatype="java.lang.Boolean" key="peernode" value="true" />
         </ast>
      </ast>
   </ast-root>

   <!-- list property array value setter method signature -->
   <ast-root name="list_prop_array_setter_sig" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="METHOD_DECL" text="${methodname}">
         <annotation datatype="java.lang.String" key="access" value="public" />
         <annotation datatype="java.lang.String" key="rettype" value="void" />
         <ast id="0" type="LPARENS" text="">
            <ast id="0" type="REFERENCE_DEF" text="${datatype}[]">
               <annotation datatype="java.lang.String" key="name" value="elements" />
            </ast>
         </ast>
      </ast>
   </ast-root>
   
   <!-- list property array value setter method implementation -->
   <ast-root name="list_prop_array_setter_meth" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="METHOD_DEF" text="${methodname}">
         <annotation datatype="java.lang.String" key="asmhint" value="indexed_setter_from_array" />
         <annotation datatype="java.lang.String" key="access" value="public" />
         <annotation datatype="java.lang.String" key="rettype" value="void" />
         <ast id="0" type="LPARENS" text="">
            <ast id="0" type="REFERENCE_DEF" text="${datatype}[]">
               <annotation datatype="java.lang.String" key="name" value="elements" />
            </ast>
         </ast>
         <ast id="0" type="BLOCK" text="">
            <ast id="0" type="ASSIGN" text="">
               <ast id="0" type="REFERENCE_DEF" text="int">
                  <annotation datatype="java.lang.String" key="name" value="len" />
               </ast>
               <ast id="0" type="STATIC_METHOD_CALL" text="Math.min">
                  <annotation datatype="java.lang.String" key="class" value="java.lang.Math" />
                  <annotation datatype="java.lang.String" key="method" value="min" />
                  <ast id="0" type="MEMBER" text="length">
                     <ast id="0" type="REFERENCE" text="elements" />
                  </ast>
                  <ast id="0" type="METHOD_CALL" text="${mname}">
                     <ast id="0" type="REFERENCE" text="${varname}" />
                  </ast>
               </ast>
            </ast>
            <ast id="0" type="BLOCK_DEF" text="">
               <ast id="0" type="KW_FOR" text="for">
                  <ast id="0" type="ASSIGN" text="=">
                     <ast id="0" type="REFERENCE_DEF" text="int">
                        <annotation datatype="java.lang.String" key="name" value="i" />
                     </ast>
                     <ast id="0" type="EXPRESSION" text="">
                        <ast id="0" type="NUM_LITERAL" text="0" />
                     </ast>
                  </ast>
                  <ast id="0" type="EXPRESSION" text="">
                     <ast id="0" type="LT" text="">
                        <ast id="0" type="REFERENCE" text="i" />
                        <ast id="0" type="REFERENCE" text="len" />
                     </ast>
                  </ast>
                  <ast id="0" type="EXPRESSION" text="">
                     <ast id="0" type="INCREMENT" text="++">
                        <ast id="0" type="REFERENCE" text="i" />
                     </ast>
                  </ast>
               </ast>
               <ast id="0" type="BLOCK" text="">
                  <ast id="0" type="STATIC_METHOD_CALL" text="${methodname}">
                     <annotation datatype="java.lang.String" key="fieldtype" value="${fieldtype}" />
                     <ast id="0" type="REFERENCE" text="i" />
                     <ast id="0" type="REFERENCE" text="elements">
                        <ast id="0" type="LBRACKET" text="">
                           <ast id="0" type="REFERENCE" text="i" />
                        </ast>
                     </ast>
                  </ast>
               </ast>
            </ast>
         </ast>
      </ast>
   </ast-root>
   
   <!-- get-at-index method call -->
   <ast-root name="get_at_index_call" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="CAST" text="${datatype}">
         <ast id="0" type="METHOD_CALL" text="get">
            <ast id="0" type="REFERENCE" text="${varname}" />
            <ast id="0" type="REFERENCE" text="index" />
         </ast>
      </ast>
   </ast-root>
   
   <!-- set-at-index method call -->
   <ast-root name="set_at_index_call" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="METHOD_CALL" text="set">
         <ast id="0" type="REFERENCE" text="${varname}" />
         <ast id="0" type="REFERENCE" text="index" />
         <ast id="0" type="REFERENCE" text="element" />
      </ast>
   </ast-root>
   
   <!-- call indexed getter and assign to result -->
   <ast-root name="assign_to_idx_getter" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="METHOD_CALL" text="assign">
         <ast id="0" type="STATIC_METHOD_CALL" text="${methodname}">
            <ast id="0" type="REFERENCE" text="index" />
         </ast>
         <ast id="0" type="REFERENCE" text="element" />
      </ast>
   </ast-root>
   
   <!-- indexed composite property getter method definition for a raw typed list -->
   <ast-root name="composite_getter" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="METHOD_DEF" text="${methodname}">
         <annotation datatype="java.lang.String" key="asmhint" value="composite_getter" />
         <annotation datatype="java.lang.String" key="access" value="public" />
         <annotation datatype="java.lang.String" key="rettype" value="${rettype}" />
         <ast id="0" type="LPARENS" text="">
            <ast id="0" type="REFERENCE_DEF" text="int">
               <annotation datatype="java.lang.String" key="name" value="index" />
            </ast>
         </ast>
         <ast id="0" type="BLOCK" text="">
            <ast id="0" type="KW_RETURN" text="">
               <ast id="0" type="EXPRESSION" text="">
                  <ast id="0" type="METHOD_CALL" text="${methodname}">
                     <ast id="0" type="CAST" text="${comptype}">
                        <ast id="0" type="METHOD_CALL" text="get">
                           <ast id="0" type="${fieldtype}" text="${compname}" />
                           <ast id="0" type="REFERENCE" text="index" />
                        </ast>
                     </ast>
                  </ast>
               </ast>
            </ast>
         </ast>
      </ast>
   </ast-root>
   
   <!-- indexed composite property getter method definition for a generically typed list -->
   <ast-root name="generic_composite_getter"
             terse="true"
             ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="METHOD_DEF" text="${methodname}">
         <annotation datatype="java.lang.String" key="asmhint" value="composite_getter" />
         <annotation datatype="java.lang.String" key="access" value="public" />
         <annotation datatype="java.lang.String" key="rettype" value="${rettype}" />
         <ast id="0" type="LPARENS" text="">
            <ast id="0" type="REFERENCE_DEF" text="int">
               <annotation datatype="java.lang.String" key="name" value="index" />
            </ast>
         </ast>
         <ast id="0" type="BLOCK" text="">
            <ast id="0" type="KW_RETURN" text="">
               <ast id="0" type="EXPRESSION" text="">
                  <ast id="0" type="METHOD_CALL" text="${methodname}">
                     <ast id="0" type="METHOD_CALL" text="get">
                        <ast id="0" type="${fieldtype}" text="${compname}" />
                        <ast id="0" type="REFERENCE" text="index" />
                     </ast>
                  </ast>
               </ast>
            </ast>
         </ast>
      </ast>
   </ast-root>
   
   <!-- indexed composite property setter method definition for a raw typed list -->
   <ast-root name="composite_setter" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="METHOD_DEF" text="${methodname}">
         <annotation datatype="java.lang.String" key="asmhint" value="composite_setter" />
         <annotation datatype="java.lang.String" key="access" value="public" />
         <annotation datatype="java.lang.String" key="rettype" value="void" />
         <ast id="0" type="LPARENS" text="">
            <ast id="0" type="REFERENCE_DEF" text="int">
               <annotation datatype="java.lang.String" key="name" value="index" />
            </ast>
            <ast id="0" type="REFERENCE_DEF" text="${datatype}">
               <annotation datatype="java.lang.String" key="name" value="element" />
            </ast>
         </ast>
         <ast id="0" type="BLOCK" text="">
            <ast id="0" type="EXPRESSION" text="">
               <ast id="0" type="METHOD_CALL" text="${methodname}">
                  <ast id="0" type="CAST" text="${comptype}">
                     <ast id="0" type="METHOD_CALL" text="get">
                        <ast id="0" type="${fieldtype}" text="${compname}" />
                        <ast id="0" type="REFERENCE" text="index" />
                     </ast>
                  </ast>
                  <ast id="0" type="REFERENCE" text="element" />
               </ast>
            </ast>
         </ast>
      </ast>
   </ast-root>
   
   <!-- indexed composite property setter method definition for a generically typed list -->
   <ast-root name="generic_composite_setter"
             terse="true"
             ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="METHOD_DEF" text="${methodname}">
         <annotation datatype="java.lang.String" key="asmhint" value="composite_setter" />
         <annotation datatype="java.lang.String" key="access" value="public" />
         <annotation datatype="java.lang.String" key="rettype" value="void" />
         <ast id="0" type="LPARENS" text="">
            <ast id="0" type="REFERENCE_DEF" text="int">
               <annotation datatype="java.lang.String" key="name" value="index" />
            </ast>
            <ast id="0" type="REFERENCE_DEF" text="${datatype}">
               <annotation datatype="java.lang.String" key="name" value="element" />
            </ast>
         </ast>
         <ast id="0" type="BLOCK" text="">
            <ast id="0" type="EXPRESSION" text="">
               <ast id="0" type="METHOD_CALL" text="${methodname}">
                  <ast id="0" type="METHOD_CALL" text="get">
                     <ast id="0" type="${fieldtype}" text="${compname}" />
                     <ast id="0" type="REFERENCE" text="index" />
                  </ast>
                  <ast id="0" type="REFERENCE" text="element" />
               </ast>
            </ast>
         </ast>
      </ast>
   </ast-root>
   
   <!-- indexed property getter method definition for wrapper index
        (delegates to primitve index variant of same method) -->
   <ast-root name="delegate_wrapper_index_getter" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="METHOD_DEF" text="${methodname}">
         <annotation datatype="java.lang.String" key="asmhint" value="delegate_index_getter" />
         <annotation datatype="java.lang.String" key="access" value="public" />
         <annotation datatype="java.lang.String" key="rettype" value="${rettype}" />
         <ast id="0" type="LPARENS" text="">
            <ast id="0" type="REFERENCE_DEF" text="${indextype}">
               <annotation datatype="java.lang.String" key="name" value="index" />
            </ast>
         </ast>
         <ast id="0" type="BLOCK" text="">
            <ast id="0" type="KW_RETURN" text="">
               <ast id="0" type="EXPRESSION" text="">
                  <ast id="0" type="STATIC_METHOD_CALL" text="${methodname}">
                     <ast id="0" type="METHOD_CALL" text="intValue">
                        <ast id="0" type="REFERENCE" text="index" />
                     </ast>
                  </ast>
               </ast>
            </ast>
         </ast>
      </ast>
   </ast-root>
   
   <!-- indexed property setter method definition for wrapper index
        (delgates to primitve index variant of same method) -->
   <ast-root name="delegate_wrapper_index_setter" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="METHOD_DEF" text="${methodname}">
         <annotation datatype="java.lang.String" key="asmhint" value="delegate_index_setter" />
         <annotation datatype="java.lang.String" key="access" value="public" />
         <annotation datatype="java.lang.String" key="rettype" value="void" />
         <ast id="0" type="LPARENS" text="">
            <ast id="0" type="REFERENCE_DEF" text="${indextype}">
               <annotation datatype="java.lang.String" key="name" value="index" />
            </ast>
            <ast id="0" type="REFERENCE_DEF" text="${datatype}">
               <annotation datatype="java.lang.String" key="name" value="element" />
            </ast>
         </ast>
         <ast id="0" type="BLOCK" text="">
            <ast id="0" type="EXPRESSION" text="">
               <ast id="0" type="STATIC_METHOD_CALL" text="${methodname}">
                  <ast id="0" type="METHOD_CALL" text="intValue">
                     <ast id="0" type="REFERENCE" text="index" />
                  </ast>
                  <ast id="0" type="REFERENCE" text="element" />
               </ast>
            </ast>
         </ast>
      </ast>
   </ast-root>
   
   <!-- default constructor javadoc comment -->
   <ast-root name="javadoc_def_ctor" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="DOC_COMMENT" text="">
         <ast id="0" type="DOC_PARAGRAPH" text="">
            <ast id="0" type="DOC_TEXTRUN" text="Default constructor." />
         </ast>
      </ast>
   </ast-root>
   
   <!-- multiplex constructor javadoc comment (temp tables only) -->
   <ast-root name="javadoc_multiplex_ctor" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="DOC_COMMENT" text="">
         <ast id="0" type="DOC_PARAGRAPH" text="">
            <ast id="0" type="DOC_TEXTRUN" text="Constructor which accepts a multiplex ID." />
         </ast>
         <ast id="0" type="DOCTAG_PARAM" text="multiplexID">
            <ast id="0" type="DOC_TEXTRUN" text="Multiplex ID." />
         </ast>
      </ast>
   </ast-root>
   
   <!-- initialize method javadoc comment -->
   <ast-root name="javadoc_initialize" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="DOC_COMMENT" text="">
         <ast id="0" type="DOC_PARAGRAPH" text="">
            <ast id="0" type="DOC_TEXTRUN" text="Initialize instance member variables to default starting values." />
         </ast>
      </ast>
   </ast-root>
   
   <!-- class-level javadoc comment for a permanent DMO -->
   <ast-root name="javadoc_class" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="DOC_COMMENT" text="">
         <annotation datatype="java.lang.Boolean" key="peernode" value="true" />
         <ast id="0" type="DOC_PARAGRAPH" text="">
            <ast id="0" type="DOC_TEXTRUN" text="Data Model Object corresponding with table " />
            <ast id="0" type="DOC_HTML" text="code">
               <ast id="0" type="DOC_TEXTRUN" text="${dbname}.${tablename}" />
            </ast>
            <ast id="0" type="DOC_TEXTRUN" text=" (previously " />
            <ast id="0" type="DOC_HTML" text="code">
               <ast id="0" type="DOC_TEXTRUN" text="${dbname}.${historical}" />
            </ast>
            <ast id="0" type="DOC_TEXTRUN" text=")." />
         </ast>
      </ast>
   </ast-root>
   
   <!-- class-level javadoc comment for a temp table DMO -->
   <ast-root name="javadoc_class_tmp_tab" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="DOC_COMMENT" text="">
         <annotation datatype="java.lang.Boolean" key="peernode" value="true" />
         <ast id="0" type="DOC_PARAGRAPH" text="">
            <ast id="0" type="DOC_TEXTRUN" text="Data Model Object corresponding with a temporary table." />
         </ast>
      </ast>
   </ast-root>
   
   <!-- inner class-level javadoc comment -->
   <ast-root name="javadoc_inner_class" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="DOC_COMMENT" text="">
         <ast id="0" type="DOC_PARAGRAPH" text="">
            <ast id="0" type="DOC_TEXTRUN" text="Inner class which composites properties of like extent. All properties migrated from the legacy table " />
            <ast id="0" type="DOC_HTML" text="code">
               <ast id="0" type="DOC_TEXTRUN" text="${dbname}.${historical}" />
            </ast>
            <ast id="0" type="DOC_TEXTRUN" text=" of extent ${extent} are data members of this class." />
         </ast>
      </ast>
   </ast-root>
   
   <!-- inner interface javadoc comment -->
   <ast-root name="javadoc_inner_iface" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="DOC_COMMENT" text="">
         <ast id="0" type="DOC_PARAGRAPH" text="">
            <ast id="0" type="DOC_TEXTRUN" text="DMO interface for use by converted business logic." />
         </ast>
      </ast>
   </ast-root>
   
   <!-- arbitrary javadoc comment paragraph -->
   <ast-root name="javadoc_paragraph" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="DOC_PARAGRAPH" text="">
         <ast id="0" type="DOC_TEXTRUN" text="${content}" />
      </ast>
   </ast-root>
   
   <!-- deepCopy method javadoc comment -->
   <ast-root name="javadoc_deepCopy" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="DOC_COMMENT" text="">
         <ast id="0" type="DOC_PARAGRAPH" text="">
            <ast id="0" type="DOC_TEXTRUN" text="Make a deep copy of the current DMO instance and return it." />
         </ast>
         <ast id="0" type="DOCTAG_RETURN" text="return">
            <ast id="0" type="DOC_TEXTRUN" text="A new instance of this class whose data is identical to that of this instance." />
         </ast>
      </ast>
   </ast-root>
   
   <!-- assign method javadoc comment -->
   <ast-root name="javadoc_assign" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="DOC_COMMENT" text="">
         <ast id="0" type="DOC_PARAGRAPH" text="">
            <ast id="0" type="DOC_TEXTRUN" text="Modify the contained data of the instance to copy the instance data from the given, backup instance." />
         </ast>
         <ast id="0" type="DOCTAG_PARAM" text="old">
            <ast id="0" type="DOC_TEXTRUN" text="The backup instance from which to copy data." />
         </ast>
      </ast>
   </ast-root>
   
   <!-- uniqueConstraints method javadoc comment -->
   <ast-root name="javadoc_uniqueConstraints" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="DOC_COMMENT" text="">
         <ast id="0" type="DOC_PARAGRAPH" text="">
            <ast id="0" type="DOC_TEXTRUN" text="Retrieve the combinations of property names which form this database object's unique constraints, if any." />
         </ast>
         <ast id="0" type="DOC_PARAGRAPH" text="">
            <ast id="0" type="DOC_TEXTRUN" text="This method does not conform with the Java-bean naming convention for an accessor method, to ensure it never conflicts with a data member instance variable." />
         </ast>
         <ast id="0" type="DOCTAG_RETURN" text="return">
            <ast id="0" type="DOC_TEXTRUN" text="A two-dimensional array of property names, or " />
            <ast id="0" type="DOC_HTML" text="code">
               <ast id="0" type="DOC_TEXTRUN" text="null" />
            </ast>
            <ast id="0" type="DOC_TEXTRUN" text=" if this object has no unique constraints defined. The inner array lists, in order, the names of the properties which form the constraint." />
         </ast>
      </ast>
   </ast-root>
   
   <!-- data member getter method javadoc comment -->
   <ast-root name="javadoc_getter" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="DOC_COMMENT" text="">
         <annotation datatype="java.lang.Boolean" key="peernode" value="true" />
         <ast id="0" type="DOC_PARAGRAPH" text="">
            <ast id="0" type="DOC_TEXTRUN" text="Getter: ${fulldesc}" />
         </ast>
         <ast id="0" type="DOCTAG_RETURN" text="return">
            <ast id="0" type="DOC_TEXTRUN" text="${briefdesc}" />
         </ast>
      </ast>
   </ast-root>
   
   <!-- data member indexed getter method javadoc comment -->
   <ast-root name="javadoc_idx_getter" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="DOC_COMMENT" text="">
         <ast id="0" type="DOC_PARAGRAPH" text="">
            <ast id="0" type="DOC_TEXTRUN" text="Indexed Getter: ${fulldesc}" />
         </ast>
         <ast id="0" type="DOCTAG_PARAM" text="index">
            <ast id="0" type="DOC_TEXTRUN" text="Index of element (where " />
            <ast id="0" type="DOC_HTML" text="code">
               <ast id="0" type="DOC_TEXTRUN" text="0 &amp;lt;= " />
               <ast id="0" type="DOC_HTML" text="em">
                  <ast id="0" type="DOC_TEXTRUN" text="index" />
               </ast>
               <ast id="0" type="DOC_TEXTRUN" text=" &amp;lt; ${extent}" />
            </ast>
            <ast id="0" type="DOC_TEXTRUN" text=")." />
         </ast>
         <ast id="0" type="DOCTAG_RETURN" text="return">
            <ast id="0" type="DOC_TEXTRUN" text="${briefdesc}" />
         </ast>
      </ast>
   </ast-root>
   
   <!-- data member setter method javadoc comment -->
   <ast-root name="javadoc_setter" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="DOC_COMMENT" text="">
         <annotation datatype="java.lang.Boolean" key="peernode" value="true" />
         <ast id="0" type="DOC_PARAGRAPH" text="">
            <ast id="0" type="DOC_TEXTRUN" text="${descprefix}Setter: ${fulldesc}" />
         </ast>
         <ast id="0" type="DOCTAG_PARAM" text="${paramname}">
            <ast id="0" type="DOC_TEXTRUN" text="${briefdesc}" />
         </ast>
      </ast>
   </ast-root>
   
   <!-- data member indexed setter method javadoc comment -->
   <ast-root name="javadoc_idx_setter" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="DOC_COMMENT" text="">
         <annotation datatype="java.lang.Boolean" key="peernode" value="true" />
         <ast id="0" type="DOC_PARAGRAPH" text="">
            <ast id="0" type="DOC_TEXTRUN" text="${descprefix}Setter: ${fulldesc}" />
         </ast>
         <ast id="0" type="DOCTAG_PARAM" text="index">
            <ast id="0" type="DOC_TEXTRUN" text="Index of element (where " />
            <ast id="0" type="DOC_HTML" text="code">
               <ast id="0" type="DOC_TEXTRUN" text="0 &amp;lt;= " />
               <ast id="0" type="DOC_HTML" text="em">
                  <ast id="0" type="DOC_TEXTRUN" text="index" />
               </ast>
               <ast id="0" type="DOC_TEXTRUN" text=" &amp;lt; ${extent}" />
            </ast>
            <ast id="0" type="DOC_TEXTRUN" text=")." />
         </ast>
         <ast id="0" type="DOCTAG_PARAM" text="${paramname}">
            <ast id="0" type="DOC_TEXTRUN" text="${briefdesc}" />
         </ast>
      </ast>
   </ast-root>
   
   <!-- data member instance variable javadoc comment -->
   <ast-root name="javadoc_var" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="DOC_COMMENT" text="">
         <ast id="0" type="DOC_PARAGRAPH" text="">
            <ast id="0" type="DOC_TEXTRUN" text="${content}" />
         </ast>
      </ast>
   </ast-root>
   
   <!-- list instance variable javadoc comment -->
   <ast-root name="javadoc_list_var" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="DOC_COMMENT" text="">
         <ast id="0" type="DOC_PARAGRAPH" text="">
            <annotation datatype="java.lang.Boolean" key="peernode" value="true" />
            <ast id="0" type="DOC_TEXTRUN" text="${description}; ${extent} elements of type " />
         </ast>
      </ast>
   </ast-root>
   
   <!-- data member list property size method javadoc comment -->
   <ast-root name="javadoc_list_prop_size" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="DOC_COMMENT" text="">
         <ast id="0" type="DOC_PARAGRAPH" text="">
            <ast id="0" type="DOC_TEXTRUN" text="Report number of list elements for ${fulldesc}" />
         </ast>
         <ast id="0" type="DOCTAG_RETURN" text="return">
            <ast id="0" type="DOC_TEXTRUN" text="Number of list elements for ${briefdesc}" />
         </ast>
      </ast>
   </ast-root>
   
   <!-- data member list property scalar setter method javadoc comment -->
   <ast-root name="javadoc_list_prop_scalar_setter" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="DOC_COMMENT" text="">
         <ast id="0" type="DOC_PARAGRAPH" text="">
            <ast id="0" type="DOC_TEXTRUN" text="Set all elements of the target list property to the given scalar value.  Modifies the contents of:  ${fulldesc}" />
         </ast>
         <ast id="0" type="DOCTAG_PARAM" text="element">
            <ast id="0" type="DOC_TEXTRUN" text="Scalar value to which to set all elements of:  ${briefdesc}" />
         </ast>
      </ast>
   </ast-root>
   
   <!-- data member list property array setter method javadoc comment -->
   <ast-root name="javadoc_list_prop_array_setter" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="DOC_COMMENT" text="">
         <ast id="0" type="DOC_PARAGRAPH" text="">
            <ast id="0" type="DOC_TEXTRUN" text="Set as many elements of the target list property as are provided in the given array of scalar values.  Modifies the contents of:  ${fulldesc}" />
         </ast>
         <ast id="0" type="DOC_PARAGRAPH" text="">
            <ast id="0" type="DOC_TEXTRUN" text="If the given array is larger than the target list, any additional elements in the given array are ignored.  If the target array is smaller than the target list, only those elements whose array indices match are updated." />
         </ast>
         <ast id="0" type="DOCTAG_PARAM" text="elements">
            <ast id="0" type="DOC_TEXTRUN" text="Array of scalar values to which to set the corresponding elements of:  ${briefdesc}" />
         </ast>
      </ast>
   </ast-root>
   
   <!-- indexed_getter definition-->
   <ast-root name="indexed_getter" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="METHOD_DEF" text="${methodname}">
         <annotation datatype="java.lang.String" key="asmhint" value="indexed_getter" />
         <annotation datatype="java.lang.String" key="access" value="public" />
         <annotation datatype="java.lang.String" key="rettype" value="${rettype}" />
         <ast id="0" type="LPARENS" text="">
            <ast id="0" type="REFERENCE_DEF" text="${datatype}">
               <annotation datatype="java.lang.String" key="name" value="${indexname}" />
            </ast>
         </ast>
         <ast id="0" type="BLOCK" text="">
            <annotation datatype="java.lang.Boolean" key="peernode" value="true" />
         </ast>
      </ast>
   </ast-root>
   
   <!-- indexed_setter definition-->
   <ast-root name="indexed_setter" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="METHOD_DEF" text="${methodname}">
         <annotation datatype="java.lang.String" key="asmhint" value="indexed_setter" />
         <annotation datatype="java.lang.String" key="access" value="public" />
         <annotation datatype="java.lang.String" key="rettype" value="void" />
         <ast id="0" type="LPARENS" text="">
            <ast id="0" type="REFERENCE_DEF" text="${idxtype}">
               <annotation datatype="java.lang.String" key="name" value="${indexname}" />
            </ast>
            <ast id="0" type="REFERENCE_DEF" text="${datatype}">
               <annotation datatype="java.lang.String" key="name" value="${varname}" />
            </ast>
         </ast>
         <ast id="0" type="BLOCK" text="">
            <annotation datatype="java.lang.Boolean" key="peernode" value="true" />
         </ast>
      </ast>
   </ast-root>
   
   <!-- throw an exception -->
   <ast-root name="throw_exception" terse="true" ast_class="com.goldencode.p2j.uast.JavaAst" >
      <ast id="0" type="KW_THROW" text="throw">
         <annotation datatype="java.lang.Boolean" key="peernode" value="true" />
         <ast id="0" type="CONSTRUCTOR" text="${except}">
         </ast>
      </ast>
   </ast-root>
</templates>