/*
** Module   : TaggedNameHelper.java
** Abstract : TaggedName utility methods.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 HC  20170612 Initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.admin;

import com.goldencode.p2j.util.*;

/**
 * Utility methods for the {@link TaggedName} class.
 */
public class TaggedNameHelper
{
   /**
    * Constructor method.
    *
    * @param    name
    *           Name.
    * @param    tag
    *           Tag.
    *
    * @return   A {@link TaggedName} instance.
    */
   public static TaggedName create(BaseDataType name, BaseDataType tag)
   {
      return new TaggedName(new String[] {prettyPrint(name), prettyPrint(tag)});
   }

   /**
    * Constructor method from array of {@link Object} instances.
    *
    * @param    values
    *
    * @return   A {@link TaggedName} instance.
    */
   public static TaggedName create(Object[] values)
   {
      String[] data = new String[values.length];

      for (int i = 0; i < values.length; i++)
      {
         if (values[i] instanceof BaseDataType)
            data[i] = prettyPrint((BaseDataType) values[i]);
         else
            data[i] = values[i].toString();
      }

      return new TaggedName(data);
   }

   /**
    * Pretty-print the given 4GL value. At this time, for {@link character} or {@link int64} types,
    * the {@link int64#longValue()} or {@link character#getValue()} methods are invoked. Else, the
    * {@link BaseDataType#toString()} is used to obtain the value.
    *
    * @param    value
    *           The 4GL value
    *
    * @return   the value as a string
    */
   public static String prettyPrint(BaseDataType value)
   {
      if (value == null || value.isUnknown())
      {
         return "?";
      }

      String s = "";

      if (value instanceof character)
      {
         s = ((character) value).getValue();
      }
      else if (value instanceof int64)
      {
         s = Long.toString(((int64) value).longValue());
      }
      else
      {
         s = value.toString();
      }

      return s;
   }

   /**
    * Works in the same way as {@link #prettyPrint(BaseDataType)}, but returns
    * an empty string for <code>null</code> and unknown values.
    *
    * @param    value
    *           The 4GL value
    *
    * @return   the value as a string
    */
   public static String prettyPrintNoUnknows(BaseDataType value)
   {
      return (value == null || value.isUnknown()) ? "" : prettyPrint(value);
   }
}
