/*
** Module   : FilterSessionsView.java
** Abstract : FilterSessionsView is a modal view that provides UI to filter sessions by parameters.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ----------------Description----------------------
** 001 SBI 20170601 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/


package com.goldencode.p2j.admin.client.application.home.console.sessions;

import java.util.logging.Level;

import org.gwtbootstrap3.client.ui.Button;
import org.gwtbootstrap3.client.ui.Modal;
import org.gwtbootstrap3.client.ui.TextBox;

import com.goldencode.p2j.admin.client.widget.dialog.ModalViewWithUiHandlers;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.uibinder.client.UiBinder;
import com.google.gwt.uibinder.client.UiField;
import com.google.gwt.uibinder.client.UiHandler;
import com.google.gwt.user.client.ui.Widget;
import com.google.inject.Inject;


/**
 * FilterSessionsView is a modal view that provides UI to filter sessions by parameters.
 */
public class FilterSessionsView
extends ModalViewWithUiHandlers<SessionUIHandler>
{
   /**
    * GWT UI creator
    */
   interface Binder
   extends UiBinder<Widget, FilterSessionsView>
   {}
   
   /** The filter text box */
   @UiField
   TextBox filterSelector;

   /** The Filter button */
   @UiField
   Button filterAction;
   
   /** The Cancel button */
   @UiField
   Button cancelAction;
   
   /** The underlined modal dialog */
   @UiField
   Modal modal;
   
   /**
    * Constructs this dialog used by by MPV gwtplatform of ArcBees Inc.
    * 
    * @param    binder
    *           The injected GWT UI creator
    */
   @Inject
   public FilterSessionsView(FilterSessionsView.Binder binder)
   {
      initWidget(binder.createAndBindUi(this));
      addShowHandler(() -> {
         filterSelector.setText(getUiHandlers().getCurrentFilter());
      });
   }

   /**
    * The Filter button click handler.
    * 
    * @param    e
    *           Click event
    */
   @UiHandler("filterAction")
   void onFilterAction(ClickEvent e)
   {
      getUiHandlers().setDataSelectionFilter(filterSelector.getValue());
      logger.log(Level.INFO, "onFilterAction");
   }
   
   /**
    * The Cancel button click handler.
    * 
    * @param    e
    *           Click event
    */
   @UiHandler("cancelAction")
   void onCancelAction(ClickEvent e)
   {
      logger.log(Level.INFO, "onCancelAction");
   }

   /**
    * Provides an access to the target modal dialog.
    * 
    * @return   The underlined modal dialog
    */
   @Override
   protected Modal getModal()
   {
      return modal;
   }
}
