/*
** Module   : MultipleSelectWithValidation.java
** Abstract : Multi selection list box with validation.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 HC  20170612 Initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
**
** Additional terms under GNU Affero GPL version 3 section 7:
**
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
**
**   0. Attribution Requirement.
**
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
**
**   1. No License To Use Trademarks.
**
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
**
**   2. No Misrepresentation of Affiliation.
**
**     You may not represent yourself as Golden Code Development Corporation or FWD.
**
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
**
**   3. No Misrepresentation of Source or Origin.
**
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/
package com.goldencode.p2j.admin.client.widget;

import com.google.gwt.editor.client.*;
import com.google.web.bindery.event.shared.*;
import org.gwtbootstrap3.client.ui.base.mixin.*;
import org.gwtbootstrap3.client.ui.form.validator.*;
import org.gwtbootstrap3.extras.select.client.ui.*;

import java.util.*;

/**
 * The class implements multi selection list box widget with blank validation. "Blank" is
 * interpreted as no value selected.
 */
public class MultipleSelectWithValidation
extends MultipleSelect
implements HasBlankValidator<List<String>>,
           HasValidators<List<String>>,
           Editor<List<String>>
{
   /** The error handling implementation */
   private final ErrorHandlerMixin<List<String>> errorHandlerMixin =
      new ErrorHandlerMixin<>(this);

   /** The validation logic */
   private final BlankValidatorMixin<MultipleSelectWithValidation, List<String>> validatorMixin =
      new BlankValidatorMixin<>(this, errorHandlerMixin.getErrorHandler());

   /**
    * Allow-blank flag getter.
    *
    * @return  see above
    */
   @Override
   public boolean getAllowBlank()
   {
      return validatorMixin.getAllowBlank();
   }

   /**
    * The allow-blank flag setter.
    *
    * @param   allowBlank
    *         the new allow blank flag
    */
   @Override
   public void setAllowBlank(boolean allowBlank)
   {
      validatorMixin.setAllowBlank(allowBlank);
   }

   /**
    * Adds the validator.
    *
    * @param   validator
    *          the validator
    */
   @Override
   public void addValidator(Validator<List<String>> validator)
   {
      validatorMixin.addValidator(validator);
   }

   /**
    * Gets the validate on blur.
    *
    * @return  the validate on blur
    */
   @Override
   public boolean getValidateOnBlur()
   {
      return validatorMixin.getValidateOnBlur();
   }

   /**
    * Removes the validator.
    *
    * @param   validator
    *          the validator
    *
    * @return  <code>true</code>, if successful
    */
   @Override
   public boolean removeValidator(Validator<List<String>> validator)
   {
      return validatorMixin.removeValidator(validator);
   }

   /**
    * Reset the form element to blank and clear error messages.
    */
   @Override
   public void reset()
   {
      validatorMixin.reset();
   }

   /**
    * Sets the validate on blur.
    *
    * @param   validateOnBlur
    *          the new validate on blur
    */
   @Override
   public void setValidateOnBlur(boolean validateOnBlur)
   {
      validatorMixin.setValidateOnBlur(true);
   }

   /**
    * The validators used to validate this object.
    *
    * @param   validators
    *          the new validators
    */
   @Override
   public void setValidators(Validator<List<String>>[] validators)
   {
      validatorMixin.setValidators(validators);
   }

   /**
    * Validate the field's value using the supplied validators.
    *
    * @return  <code>true</code>, if valid
    */
   @Override
   public boolean validate()
   {
      return validatorMixin.validate();
   }

   /**
    * Validate the field's value using the supplied validators.
    *
    * @param   show
    *          the error to the user.
    *
    * @return  <code>true</code>, if valid
    */
   @Override
   public boolean validate(boolean show)
   {
      return validatorMixin.validate(show);
   }

   /**
    * Adds a validation changed handler.
    *
    * @param   handler
    *          the handler
    * @return  the handler registration
    */
   @Override
   public HandlerRegistration
   addValidationChangedHandler(ValidationChangedEvent.ValidationChangedHandler handler)
   {
      return validatorMixin.addValidationChangedHandler(handler);
   }
}
