/*
** Module   : ModalViewWithUiHandlers.java
** Abstract : ModalViewWithUiHandlers represents a base view for modal dialogs having a common
**            parent view and controller.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ----------------Description----------------------
** 001 SBI 20170601 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.admin.client.widget.dialog;

import java.util.logging.Logger;

import org.gwtbootstrap3.client.shared.event.*;
import org.gwtbootstrap3.client.ui.*;

import com.google.web.bindery.event.shared.*;
import com.gwtplatform.mvp.client.*;

/**
 * ModalViewWithUiHandlers represents a base view for modal dialogs having a common parent view
 * and controller.
 *
 * @param    <C>
 *           A common handler class implemented by its parent controller.
 */
public abstract class ModalViewWithUiHandlers<C extends UiHandlers>
extends ViewWithUiHandlers<C>
{
   /** Class logger */
   protected Logger logger = Logger.getLogger(getClass().getSimpleName());

   /** Registration handler associated with the attached show modal dialog event handler */
   private HandlerRegistration showHandler;
   
   /** Registration handler associated with the attached hide modal dialog event handler */
   private HandlerRegistration hideHandler;
   
   /**
    * Provides an access to the target modal dialog.
    * 
    * @return   The underlined modal dialog
    */
   protected abstract Modal getModal();
   
   /**
    * Add command executed at the moment this modal dialog gets ModalShowEvent.
    * 
    * @param    command
    *           Runnable to be executed by the attached modal show event handler.
    */
   public void addShowHandler(Runnable command)
   {
      if (showHandler != null)
      {
         showHandler.removeHandler();
      }
      
      showHandler = getModal().addShowHandler(new ModalShowHandler()
      {
         
         @Override
         public void onShow(ModalShowEvent evt)
         {
            command.run();
         }
      });
   }
   
   /**
    * Add command executed at the moment this modal dialog gets ModalHideEvent.
    * 
    * @param    command
    *           Runnable to be executed by the attached modal hide event handler.
    */
   public void addHideHandler(Runnable command)
   {
      if (hideHandler != null)
      {
         hideHandler.removeHandler();
      }
      
      hideHandler = getModal().addHideHandler(new ModalHideHandler()
      {
         
         @Override
         public void onHide(ModalHideEvent evt)
         {
            command.run();
         }
      });
   }
}
