/*
** Module   :AttributeType.java
** Abstract :List of attribute type constants.
**
** Copyright (c) 2005-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- -T- --JPRM-- ----------------Description-----------------
** 001 SIY 20050217 ADD  @20013  Created initial version.
** 002 SIY 20050317 ADD  @20457  Added toString() method.
** 003 SIY 20050323 CHG  @20466  Fixed formatting and comments.
** 004 SIY 20050418 CHG  @21005  Added fromString() method, fixed formatting.  
** 005 SIY 20050514 CHG  @21186  More detailed documentation.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.directory;

/**
 * Attribute type constants and helper methods to convert them to/from string.
 * 
 * @author  SIY
 * @version 1.0
 */
public final class AttributeType
{
   /** Attribute type is int */
   public static final int ATTR_INTEGER     = 1;
   
   /** Attribute type is boolean */
   public static final int ATTR_BOOLEAN     = 2;
   
   /** Attribute type is String */
   public static final int ATTR_STRING      = 3;
   
   /** Attribute type is double */
   public static final int ATTR_DOUBLE      = 4;
   
   /** Attribute type is array of bytes */
   public static final int ATTR_BYTEARRAY   = 5;
   
   /** Attribute type is bit filed */
   public static final int ATTR_BITFIELD    = 6;
   
   /** Attribute type is bit selector */
   public static final int ATTR_BITSELECTOR = 7;
   
   /** Attribute type is Date */
   public static final int ATTR_DATE        = 8;
   
   /** Attribute type is Time */
   public static final int ATTR_TIME        = 9;

   /**
    * Static strings which must match defined types.
    */
   private static String[] types =
      {
         "unknown" , "INTEGER"    , "BOOLEAN"  ,
         "STRING"  , "DOUBLE"     , "BYTEARRAY", 
         "BITFIELD", "BITSELECTOR", "DATE"     ,
         "TIME"
      };

   /**
    * Convert type code into humen-readable text string.
    * 
    * @param   type
    *          Type code for which string representation is requested.
    * @return  Text string representation of the type.
    */
   public static String toString(int type)
   {
      return types[type];
   }

   /**
    * Convert type name into type value.
    * 
    * @param   type
    *          A string which represents type name.
    * @return  Type code for specified type name.
    */
   public static int fromString(String type)
   {
      String tmp = type.trim().toUpperCase();
       
      for (int i = 1; i < types.length; i++)
      {
         if (tmp.equals(types[i]))
            return i;
      }
      
      return 0;
   }
}