/*
** Module   : DirectoryManager.java
** Abstract : handles initialization and access to the provider for the
**            directory API
**
** Copyright (c) 2008-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- -T- --JPRM-- ----------------Description-----------------
** 001 GES 20080409 NEW   @37908 Created initial version which supports access
**                               from both the server and client.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.directory;

import com.goldencode.p2j.net.*;

/**
 * This interface defines the primary API for directory access. This API is
 * designed with simplicity in mind.  It is currently incomplete, but it
 * does demonstrate a significantly easier mechanism for directory access.
 */
public class DirectoryManager
{
   /** The current directory instance. */
   private static Directory dir = null;
   
   /**
    * Initialize the current directory API provider and export the directory
    * API via that instance.
    */
   public static synchronized void initServer()
   {
      dir = new DirectoryServer();
      RemoteObject.registerNetworkServer(Directory.class, dir);
   }
   
   /**
    * Initialize the current directory API provider as a remote object for
    * the given session.
    */
   public static synchronized void initRemote(Session session)
   {
      dir = (Directory) RemoteObject.obtainNetworkInstance(Directory.class,
                                                           session);
   }
   
   /**
    * Access the instance of the current directory API provider. In the case
    * where this class has not yet initialized, a <code>NullDirectory</code>
    * will be returned.  This makes such pre-initialization access safe but
    * it is not likely to be a correct result.
    *
    * @return   The current directory API provider.
    */
   public static synchronized Directory getInstance()
   {
      if (dir == null)
      {
         dir = new NullDirectory();
      }
      
      return dir;
   }
}