/*
** Module   : EntryPointResource.java
** Abstract : plugin to manage access to application entry points
**
** Copyright (c) 2014-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ----------------------------Description-----------------------------------
** 001 GES 20141229 Created the first version.
** 002 GES 20160222 Moved most of the implementation into a parent class so it can be shared.
** 003 CA  20161123 Fixed package for getRightsEditorName().
** 004 HC  20170612 Changes related to implementation of new GWT-based Admin client.
*/ 
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.main;

import com.goldencode.p2j.security.*;

/**
 * Implements the application entry point resource. Instances of this resource control access 
 * to application external procedures which can be invoked through user-specified procedure
 * names.
 * <p>
 * Although the user will usually specify the entry point names as legacy 4GL procedure names,
 * any such name will be converted to the corresponding Java converted class name before any
 * resource checking will occur.  This means that resource names MUST be specified as the Java
 * class name in dot-separated package name format.  For a legacy 4GL name of 
 * <code>some-converted-program-name.p</code> that exists in the "pkgroot" of the project, this
 * is the correct form:
 * <p>
 * <pre>
 * com.goldencode.testcases.SomeConvertedProgramName
 * </pre>
 * <p>
 * It is valid to implement inexact (regex) matches as well.  This would allow all possible
 * matches:
 * <p>
 * <pre>
 * .*
 * </pre>
 */
public class EntryPointResource
extends StringConditionResource
{
   /**
    * Default constructor. 
    */
   public EntryPointResource()
   {
   }

   /**
    * Returns the plugin resource type name as a string.
    *
    * @return   The plugin resource type name.
    */
   public String getTypeName()
   {
      return "entrypoint";
   }

   /**
    * Instantiates a plugin's class that implements the <code>Rights</code> interface, using
    * the array of objects representing a set of access rights fields.
    *
    * @param    rights
    *           The objects needed to create an access rights instance.
    *           
    * @return   The newly created rights instance.
    */
   public Rights getRightsInstance(Object[] rights)
   {
      return new EntryPointRights((String)rights[0]);
   }

   /**
    * Checks whether a given string is a valid resource name for this resource type.
    *
    * @param    resource
    *           A string naming a resource.
    *
    * @return   <code>true</code> if the name is syntactically correct. 
    */
   public boolean isInstanceNameValid(String resource)
   {
      if (!super.isInstanceNameValid(resource))
         return false;

      // TODO: this should check if the name could be a valid class name, but it can also
      //       specify a regex, which makes this check tricky
      
      return true;
   }
}