/*
** Module   : WebClientLauncher.java
** Abstract : low-level API to spawn a web client
**
** Copyright (c) 2016-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 GES 20160121 Created initial version.
** 002 SBI 20171023 Changed to provide the required request parameters.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.main;

import com.goldencode.p2j.util.*;

/**
 * Provide a low-level API to spawn a web client.
 */
public interface WebClientLauncher
{
   /**
    * Spawn a new web client process, returning the redirect URL if successful.
    *  
    * @param    user
    *           The OS userid to use for the new process.  
    * @param    pw
    *           The OS password to authenticate the user.
    * @param    gui
    *           This specifies the UI mode (<code>true</code> for GUI, <code>false</code> for
    *           ChUI).
    * @param    options
    *           List of caller-specified options to be passed on the client command line or
    *           <code>null</code> if no values are specified.
    * @param    referrer
    *           The referrer string to provide the web client. If the client gets disconnected
    *           it will be redirected back to this URL. 
    * @param    requestParameters
    *           Holds additional parameters if they are provided. It is expected that the first
    *           requestParameters[0] holds the forwarded host of "X-Forwarded-Host" HTTP header,
    *           requestParameters[1] holds the corresponding forwarded protocol that is provided
    *           by "X-Forwarded-Proto" value and requestParameters[2] holds the client IP address.
    * 
    * @return   The redirect URL for the new web client on success.  If a failure occurred, the
    *           returned string will be prefixed by {@link WebHandler#ERROR_PREFIX} and followed
    *           by descriptive error text.  
    */
   public String spawn(String user,
                       String pw,
                       boolean gui,
                       String[] options,
                       String referrer,
                       String[] requestParameters);
}