/*
** Module   : DataSource.java
** Abstract : Describes an external source from which data is read into a buffer.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 ECF 20170820 Created initial version.
** 002 ECF 20171008 Implemented file as an output resource.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.persist;

import java.io.*;
import com.goldencode.p2j.util.*;

/**
 * Object which describes an external source from which data is read into a temp-table buffer
 * or data set member buffer.
 */
public class DataSource
{
   /** Normalized input stream */
   private InputStream stream = null;
   
   /**
    * Constructor.
    * 
    * @param   source
    *          Object from which to read data.
    */
   public DataSource(character source)
   {
      initializeRemote(source.toStringMessage());
   }
   
   /**
    * Constructor.
    * 
    * @param   source
    *          Object from which to read data.
    */
   public DataSource(String source)
   {
      initializeRemote(source);
   }
   
   /**
    * Constructor.
    * 
    * @param   source
    *          Pointer to area of memory from which to read data.
    */
   public DataSource(memptr source)
   {
      UnimplementedFeature.missing("DataSource c'tor");
   }
   
   /**
    * Constructor.
    * 
    * @param   source
    *          Handle of object from which to read data.
    */
   public DataSource(handle source)
   {
      UnimplementedFeature.missing("DataSource c'tor");
   }
   
   /**
    * Constructor.
    * 
    * @param   source
    *          Object from which to read data.
    */
   public DataSource(longchar source)
   {
      UnimplementedFeature.missing("DataSource c'tor");
   }
   
   /**
    * Get the input stream representing the source resource from which data will be read.
    * This may be a remote or a local resource.
    * 
    * @return  Input stream.
    */
   public InputStream getInputStream()
   {
      return stream;
   }
   
   /**
    * Close the input stream underlying this data source.
    * 
    * @throws  IOException
    *          if there is an error closing the stream.
    */
   void close()
   throws IOException
   {
      stream.close();
   }
   
   /**
    * Initialize the remote input stream which backs this data source.
    * 
    * @param   filename
    *          Client-side file name.
    */
   private void initializeRemote(String filename)
   {
      stream = new InputStreamWrapper(StreamFactory.openFileStream(filename, false, false));
   }
}
