/*
** Module   : QueryConstants.java
** Abstract : Constants used by query support code as well as client code
**
** Copyright (c) 2005-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- -T- --JPRM-- ----------------Description-----------------
** 001 ECF 20051013 ADD   @23257 Created initial version. Constants used by
**                               query support code as well as client code.
** 002 ECF 20051114 CHG   @23383 Added retrieval types and RETRIEVE_MODES
**                               table. Added HQL_* constants. Removed JOIN_*
**                               constants.
** 003 ECF 20051216 CHG   @23742 Modified RETRIEVE_MODES. Changed FIRST/LAST
**                               navigation types to map to NEXT and PREVIOUS
**                               iteration types differently, to correct
**                               CompoundQuery defect.
** 004 ECF 20090303 CHG   @41598 Removed HQL_* constants. No longer necessary,
**                               due to changes in HQLHelper.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.persist;

/**
 * Constants used in association with queries.
 */
public interface QueryConstants
{
   /** Placeholder indicating no retrieval type is set */
   public static final int NONE = -1;
   
   /** Retrieve first record in a result set */
   public static final int FIRST = 0;
   
   /** Retrieve last record in a result set */
   public static final int LAST = 1;
   
   /** Retrieve next record in a result set */
   public static final int NEXT = 2;
   
   /** Retrieve previous record in a result set */
   public static final int PREVIOUS = 3;
   
   /** Retrieve a unique record in a result set */
   public static final int UNIQUE = 4;
   
   /** Reload the current record in a result set */
   public static final int CURRENT = 5;
   
   /** Table of master query result navigation to component query retrieval */
   public static final int[][] RETRIEVE_MODES =
   {
     // FIRST       LAST        NEXT        PREVIOUS    UNIQUE
      { FIRST   ,   LAST    ,   NEXT    ,   PREVIOUS,   UNIQUE  }, // FIRST
      { FIRST   ,   LAST    ,   PREVIOUS,   NEXT    ,   UNIQUE  }, // LAST
      { FIRST   ,   LAST    ,   NEXT    ,   PREVIOUS,   UNIQUE  }, // NEXT
      { FIRST   ,   LAST    ,   PREVIOUS,   NEXT    ,   UNIQUE  }, // PREVIOUS
      { NONE    ,   NONE    ,   NONE    ,   NONE    ,   NONE    }, // UNIQUE
      { CURRENT ,   CURRENT ,   CURRENT ,   CURRENT ,   CURRENT }, // CURRENT
   };
}