/*
** Module   : IncludeHint.java
** Abstract : Collection of data that constitutes a hint record for includes.
**
** Copyright (c) 2004-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- -T- --JPRM-- ----------------Description-----------------
** 001 NVS 20050511 ADD   @21153 Created. Initial implementation.
** 002 NVS 20050519 CHG   @21240 Include filenames are now normalized - they
**                               are relative to the project home (P2J_HOME)
**                               if they are internal to the project.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.preproc;

import com.goldencode.p2j.cfg.Configuration;

/**
 * Keeps information about an include file and how it maps to the preprocessor
 * output.
 * <p>
 * @author NVS
 * @version 1.0.0
 */
public class IncludeHint
{
   /** Include file name */
   private String filename = null;

   /** line number on the output for the first character produced */
   private int startLine = 0;

   /** column number on the output for the first character produced */
   private int startCol = 0;

   /** line number on the output for the last character produced */
   private int endLine = 0;

   /** column number on the output for the last character produced + 1 */
   private int endCol = 0;

   /**
    * Default constructor.
    */
   public IncludeHint(String filename)
   {
      this.filename = Configuration.normalizeFilename(filename);
   }

   /**
    * Gets include file name.
    *
    * @return  include file name
    */
   public String getFilename()
   {
      return filename;
   }

   /**
    * Sets starting line number.
    *
    * @param   line
    *          starting line number
    */
   public void setStartLine(int line)
   {
      startLine = line;
   }

   /**
    * Gets starting line number.
    *
    * @return starting line number
    */
   public int getStartLine()
   {
      return startLine;
   }

   /**
    * Sets starting column number.
    *
    * @param   column
    *          starting column number
    */
   public void setStartColumn(int column)
   {
      startCol = column;
   }

   /**
    * Gets starting column number.
    *
    * @return  starting column number
    */
   public int getStartColumn()
   {
      return startCol;
   }

   /**
    * Sets ending line number.
    *
    * @param   line
    *          endingline number
    */
   public void setEndLine(int line)
   {
      endLine = line;
   }

   /**
    * Gets ending line number.
    *
    * @return ending line number
    */
   public int getEndLine()
   {
      return endLine;
   }

   /**
    * Sets ending column number.
    *
    * @param   column
    *          ending column number
    */
   public void setEndColumn(int column)
   {
      endCol = column;
   }

   /**
    * Gets ending column number.
    *
    * @return  ending column number
    */
   public int getEndColumn()
   {
      return endCol;
   }
}