/*
** Module   : CallGraphReportLine.java
** Abstract : POJO with the details of a callgraph report line.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 CA  20170612 First version.
*/

/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.report.server;

/**
 * Encapsulates the details of a line for a callgraph report.
 */
public class CallGraphReportLine
{
   /** The AST token type (as string). */
   private String type;
   
   /** A description of this line. */
   private String name;
   
   /** Number of occurences/calls for this line. */
   private int callNum;

   /** The callgraph vertex ID associated with this line. */
   private String vertexId;
   
   /**
    * Get the AST token type.
    * 
    * @return   The {@link #type token type}
    */
   public String getType()
   {
      return type;
   }

   /**
    * Set the {@link #type AST token type}.
    * 
    * @param    type
    *           The AST token type.
    */
   public void setType(String type)
   {
      this.type = type;
   }

   /**
    * Get the description of this line.
    * 
    * @return   The {@link #name}
    */
   public String getName()
   {
      return name;
   }

   /**
    * Set the name/description for this line.
    * 
    * @param    name
    *           The name/description for this line.
    */
   public void setName(String name)
   {
      this.name = name;
   }

   /**
    * Get the number of calls for this line.
    * 
    * @return   The {@link #callNum}
    */
   public int getCallNum()
   {
      return callNum;
   }

   /**
    * Set the value for the {@link #callNum} field.
    * 
    * @param    callNum
    *           The number of calls/references for this line.
    */
   public void setCallNum(int callNum)
   {
      this.callNum = callNum;
   }

   /**
    * Get the callgraph vertex ID.
    * 
    * @return   The {@link #vertexId}
    */
   public String getVertexId()
   {
      return vertexId;
   }

   /**
    * Set the callgraph vertex ID.
    * 
    * @param    vertexId
    *           The vertex ID.
    */
   public void setVertexId(String vertexId)
   {
      this.vertexId = vertexId;
   }
}
