/*
** Module   : CallGraphSnippet.java
** Abstract : Encodes a displayable section of the call graph.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 GES 20170530 First version.
*/

/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.report.server;

import java.util.*;

/**
 * Encodes a displayable section of the call graph..
 */
public class CallGraphSnippet
{
   /** List of the nodes in the graph snippet. */ 
   private List<CallGraphNode> nodes = null;
   
   /** List of the links in the graph snippet. */ 
   private List<CallGraphLink> links = null;
   
   /** Map of the AST node details for each AST_NODE in the graph snippet. */ 
   private Map<String, CallSite> asts = null;   
   
   /** The list of root program files used to build the graph snippet. */
   private String[] rootList = null;
   
   /**
    * Create an instance.
    *
    * @param    nodes
    *           List of the nodes in the graph snippet.
    * @param    links
    *           List of the links in the graph snippet.
    * @param    asts
    *           Map of the AST node details for each AST_NODE in the graph snippet.
    */
   public CallGraphSnippet(List<CallGraphNode>   nodes,
                           List<CallGraphLink>   links,
                           Map<String, CallSite> asts,
                           String[]              rootList)
   {
      this.nodes = nodes;
      this.links = links;
      this.asts  = asts;
      this.rootList = rootList;
   }
   
   /**
    * Obtain the list of the nodes in the graph snippet.
    *
    * @return   The list of nodes.
    */
   public List<CallGraphNode> getNodes()
   {
      return nodes;
   }

   /**
    * Set the list of the nodes in the graph snippet.
    *
    * @param    nodes
    *           The new list of nodes.
    */
   public void setNodes(List<CallGraphNode> nodes)
   {
      this.nodes = nodes;
   }

   /**
    * Obtain the list of the links in the graph snippet.
    *
    * @return   The list of links.
    */
   public List<CallGraphLink> getLinks()
   {
      return links;
   }

   /**
    * Set the list of the links in the graph snippet.
    *
    * @param    links
    *           The new list of links.
    */
   public void setLinks(List<CallGraphLink> links)
   {
      this.links = links;
   }
   
   /**
    * Obtain the map of the AST details for the graph snippet.
    *
    * @return   Map of the AST node details for each AST_NODE in the graph snippet.
    */
   public Map<String, CallSite> getAsts()
   {
      return asts;
   }

   /**
    * Set the map of the AST details for the graph snippet.
    *
    * @param    asts
    *           Map of the AST node details for each AST_NODE in the graph snippet.
    */
   public void setAsts(Map<String, CallSite> asts)
   {
      this.asts = asts;
   }

   /**
    * Obtain the list of root program files used to generate the graph snippet.
    *
    * @return   List of root program files used to generate the graph snippet.
    */
   public String[] getRootList()
   {
      return rootList;
   }

   /**
    * Set the list of root program files used to generate the graph snippet.
    *
    * @param    rootList
    *           The root program files used to generate the graph snippet.
    */
  public void setRootList(String[] rootList)
   {
      this.rootList = rootList;
   }
}