/*
** Module   : CustomReport.java
** Abstract : Column metadata and row data for a custom report.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 ECF 20170520 First version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.report.server;

import java.util.*;

/**
 * Column metadata (column headings, format strings, sorting specification) and row data for a
 * custom report. Each row is a map of column index strings to numeric or text data. This format
 * is designed for easy consumption by the web client. The column metadata provides context for
 * the client to present the report.
 */
public class CustomReport
{
   /** List of column definition data for the custom report */
   private List<CustomColumn> colDefs = null;
   
   /** List of row data (each a map of column index strings to numeric or text data) */
   private List<Map<String, Object>> rows = null;
   
   /**
    * Default constructor
    */
   public CustomReport()
   {
   }
   
   /**
    * Getter for column metadata.
    * 
    * @return  List of metadata for each column, from right to left.
    */
   public List<CustomColumn> getColDefs()
   {
      return colDefs;
   }
   
   /**
    * Setter for column metadata.
    * 
    * @param   colDefs
    *          List of metadata for each column, from right to left.
    */
   public void setColDefs(List<CustomColumn> colDefs)
   {
      this.colDefs = colDefs;
   }
   
   /**
    * Getter for all of the report's row data.
    * 
    * @return  List of row data, each a map of column index strings to numeric or text values.
    */
   public List<Map<String, Object>> getRows()
   {
      return rows;
   }
   
   /**
    * Setter for all of the report's row data.
    * 
    * @param   rows
    *          List of row data, each a map of column index strings to numeric or text values.
    */
   public void setRows(List<Map<String, Object>> rows)
   {
      this.rows = rows;
   }
}
