/*
** Module   : FileFilter.java
** Abstract : File filter specification.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 SBI 20170609 First version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.report.server;

/**
 * A file filter specification which defines a single file or group of files to be included in
 * reporting results. Files which do not match the specification are excluded. Multiple instances
 * of this class define a particular {@link FilterProfile filter profile}.
 * <p>
 * A specification is either the name of an individual file, or a path ending in an asterisk.
 * The latter represents a wildcard filter which matches all files whose paths begin with the
 * portion of the specification before the asterisk. The former matches at most one file.
 */
public class FileFilter
{
   /** Flag indicating whether this filter is a wildcard vs. a single file specification */
   private boolean wild;
   
   /** File specification which defines the filter */
   private String spec;
   
   /**
    * Default constructor.
    */
   public FileFilter()
   {
   }
   
   /**
    * Getter for wildcard flag.
    * 
    * @return  {@code True} if specification is a wildcard; {@code false} if a single file.
    */
   public boolean isWild()
   {
      return wild;
   }
   
   /**
    * Setter for wildcard flag.
    * 
    * @param   wild
    *          {@code True} if specification is a wildcard; {@code false} if a single file.
    */
   public void setWild(boolean wild)
   {
      this.wild = wild;
   }
   
   /**
    * Getter for file specification.
    * 
    * @return  Wildcard or single file path specification which defines the filter.
    */
   public String getSpec()
   {
      return spec;
   }
   
   /**
    * Setter for file specification.
    * 
    * @param   spec
    *          Wildcard or single file path specification which defines the filter.
    */
   public void setSpec(String spec)
   {
      this.spec = spec;
   }
}
