/*
** Module   : OverviewRow.java
** Abstract : Overview report row data bean.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 ECF 20170505 First version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.report.server;

/**
 * Bean which encapsulates a row of data for an overview report. This includes the ID of the
 * report, its title, the total number of matches and the number of files in which those matches
 * were found, the category tag within which this row is grouped, a flag indicating whether gap
 * analysis data is included, and the condition expression used by the report.
 */
public final class OverviewRow
{
   /** Report ID */
   private int id;
   
   /** Report title */
   private String title;
   
   /** Number of matches found for the report's condition expression */
   private int matches;
   
   /** Number of files in which matches were found */
   private int files;
   
   /** Tag used to group this report with others */
   private String tag;
   
   /** Flag indicating whether report includes gap analysis support information */
   private boolean support;
   
   /** Condition expression which defines the report's matches */
   private String condition;
   
   /**
    * Default constructor.
    */
   public OverviewRow()
   {
   }
   
   /**
    * Getter for report ID.
    * 
    * @return  Report ID.
    */
   public int getId()
   {
      return id;
   }
   
   /**
    * Setter for report ID.
    * 
    * @param   id
    *          Report ID.
    */
   public void setId(int id)
   {
      this.id = id;
   }
   
   /**
    * Getter for report title.
    * 
    * @return  Report title.
    */
   public String getTitle()
   {
      return title;
   }
   
   /**
    * Setter for report title.
    * 
    * @param   title
    *          Report title.
    */
   public void setTitle(String title)
   {
      this.title = title;
   }
   
   /**
    * Getter for total number of condition matches.
    * 
    * @return  Total matches.
    */
   public int getMatches()
   {
      return matches;
   }
   
   /**
    * Setter for total number of condition matches.
    * 
    * @param   matches
    *          Total matches.
    */
   public void setMatches(int matches)
   {
      this.matches = matches;
   }
   
   /**
    * Getter for total number of files in which the report's condition was matched at least once.
    * 
    * @return  Total files containing matches.
    */
   public int getFiles()
   {
      return files;
   }
   
   /**
    * Setter for total number of files in which the report's condition was matched at least once.
    * 
    * @param   files
    *          Total files containing matches.
    */
   public void setFiles(int files)
   {
      this.files = files;
   }
   
   /**
    * Getter for tag text representing the category of reports within which this row is grouped.
    * 
    * @return  Tag text.
    */
   public String getTag()
   {
      return tag;
   }
   
   /**
    * Setter for tag text representing the category of reports within which this row is grouped.
    * 
    * @param   tag
    *          Tag text.
    */
   public void setTag(String tag)
   {
      this.tag = tag;
   }
   
   /**
    * Getter for flag indicating whether this report includes gap analysis information.
    * 
    * @return  {@code True} if the report includes gap analysis information, else {@code false}.
    */
   public boolean isSupport()
   {
      return support;
   }
   
   /**
    * Setter for flag indicating whether this report includes gap analysis information.
    * 
    * @param   support
    *          {@code True} if the report includes gap analysis information, else {@code false}.
    */
   public void setSupport(boolean support)
   {
      this.support = support;
   }
   
   /**
    * Getter for the report's condition expression.
    * 
    * @return  Condition expression.
    */
   public String getCondition()
   {
      return condition;
   }
   
   /**
    * Setter for the report's condition expression.
    * 
    * @param   condition
    *          Condition expression.
    */
   public void setCondition(String condition)
   {
      this.condition = condition;
   }
}
