/*
** Module   : FwdReport.java
** Abstract : Interface for a report implemented as 4GL syntax extension.
**
** Copyright (c) 2017-2018, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 SVL 20171030 Created initial version.
** 002 SVL 20180209 Added legacy annotations and new export functions.
** 003 HC  20180410 Renamed ReportOutputFormat to MediaType.
** 004 EVL 20180423 Renamed Report to FwdReport.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
**
** Additional terms under GNU Affero GPL version 3 section 7:
**
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
**
**   0. Attribution Requirement.
**
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
**
**   1. No License To Use Trademarks.
**
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
**
**   2. No Misrepresentation of Affiliation.
**
**     You may not represent yourself as Golden Code Development Corporation or FWD.
**
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
**
**   3. No Misrepresentation of Source or Origin.
**
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.reporting;

import com.goldencode.p2j.persist.QueryWrapper;
import com.goldencode.p2j.util.*;

/**
 * Interface for a report implemented as 4GL syntax extension.
 */
@LegacyResource(resource = LegacyResource.REPORT)
public interface FwdReport
extends WrappedResource
{
   /**
    * Set the parameter used by the report.
    *
    * @param  param
    *         Name of the parameter.
    * @param  value
    *         Value of the parameter.
    *
    * @return <code>true</code> on success.
    */
   @LegacyMethod(name = "SET-REPORT-PARAM")
   public logical setReportParameter(String param, BaseDataType value);

   /**
    * Set the parameter used by the report.
    *
    * @param  param
    *         Name of the parameter.
    * @param  value
    *         Value of the parameter.
    *
    * @return <code>true</code> on success.
    */
   @LegacyMethod(name = "SET-REPORT-PARAM")
   public logical setReportParameter(String param, String value);

   /**
    * Set the parameter used by the report.
    *
    * @param  param
    *         Name of the parameter.
    * @param  value
    *         Value of the parameter.
    *
    * @return <code>true</code> on success.
    */
   @LegacyMethod(name = "SET-REPORT-PARAM")
   public logical setReportParameter(character param, BaseDataType value);

   /**
    * Run report end export result to the PDF file.
    *
    * @param  fileName
    *         Name of the output PDF file.
    *
    * @return <code>true</code> on success.
    */
   @LegacyMethod(name = "EXPORT-REPORT-PDF")
   public logical exportReportPdf(character fileName);

   /**
    * Run report end export result to the PDF file.
    *
    * @param  fileName
    *         Name of the output PDF file.
    *
    * @return <code>true</code> on success.
    */
   @LegacyMethod(name = "EXPORT-REPORT-PDF")
   public logical exportReportPdf(String fileName);

   /**
    * Run report end export result to the XLS file.
    *
    * @param  fileName
    *         Name of the output XLS file.
    *
    * @return <code>true</code> on success.
    */
   @LegacyMethod(name = "EXPORT-REPORT-XLS")
   public logical exportReportXls(character fileName);

   /**
    * Run report end export result to the XLS file.
    *
    * @param  fileName
    *         Name of the output XLS file.
    *
    * @return <code>true</code> on success.
    */
   @LegacyMethod(name = "EXPORT-REPORT-XLS")
   public logical exportReportXls(String fileName);

   /**
    * Run report end export result to the XLSX file.
    *
    * @param  fileName
    *         Name of the output XLSX file.
    *
    * @return <code>true</code> on success.
    */
   @LegacyMethod(name = "EXPORT-REPORT-XLSX")
   public logical exportReportXlsx(character fileName);

   /**
    * Run report end export result to the XLSX file.
    *
    * @param  fileName
    *         Name of the output XLSX file.
    *
    * @return <code>true</code> on success.
    */
   @LegacyMethod(name = "EXPORT-REPORT-XLSX")
   public logical exportReportXlsx(String fileName);

   /**
    * Run report end export result to the CSV file.
    *
    * @param  fileName
    *         Name of the output CSV file.
    *
    * @return <code>true</code> on success.
    */
   @LegacyMethod(name = "EXPORT-REPORT-CSV")
   public logical exportReportCsv(character fileName);

   /**
    * Run report end export result to the CSV file.
    *
    * @param  fileName
    *         Name of the output CSV file.
    *
    * @return <code>true</code> on success.
    */
   @LegacyMethod(name = "EXPORT-REPORT-CSV")
   public logical exportReportCsv(String fileName);

   /**
    * Run report end export result using the specified output format.
    *
    * @param  fileName
    *         Name of the output file.
    * @param  outputFormat
    *         Output format.
    *
    * @return <code>true</code> on success.
    */
   public logical exportReport(character fileName, MediaType outputFormat);

   /**
    * Set report design file.
    *
    * @param designFileName
    *        Name of the report design file.
    */
   @LegacyAttribute(name = "REPORT-DESIGN", setter = true)
   public void setReportDesign(character designFileName);

   /**
    * Set report design file.
    *
    * @param designFileName
    *        Name of the report design file.
    */
   @LegacyAttribute(name = "REPORT-DESIGN", setter = true)
   public void setReportDesign(String designFileName);

   /**
    * Get the name of the report design file.
    *
    * @return the name of the report design file.
    */
   @LegacyAttribute(name = "REPORT-DESIGN")
   public character getReportDesign();

   /**
    * Set the data source used by this report.
    *
    * @param dataSource
    *        Handle to the data source.
    */
   @LegacyAttribute(name = "REPORT-DATA-SOURCE", setter = true)
   public void setReportDataSource(handle dataSource);

   /**
    * Set the query used as the data source by this report.
    *
    * @param dataSource
    *        Handle to the query used as the data source by this report.
    */
   @LegacyAttribute(name = "REPORT-DATA-SOURCE", setter = true)
   public void setReportDataSource(QueryWrapper dataSource);

   /**
    * Get the data source used by this report.
    *
    * @return handle to the data source used by this report.
    */
   @LegacyAttribute(name = "REPORT-DATA-SOURCE")
   public handle getReportDataSource();
}
