/*
** Module   : XprObjRectangle.java
** Abstract : Class to represent rectangle graphics object feature for XPR content elements.
**
** Copyright (c) 2018, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 EVL 20180510 Created initial version.
** 002 EVL 20180528 Line width is float in Jasperreports.
** 003 EVL 20180601 Minor fixes for constant and javadoc comment.
** 004 EVL 20180626 Minor javadoc typo fix.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
**
** Additional terms under GNU Affero GPL version 3 section 7:
**
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
**
**   0. Attribution Requirement.
**
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
**
**   1. No License To Use Trademarks.
**
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
**
**   2. No Misrepresentation of Affiliation.
**
**     You may not represent yourself as Golden Code Development Corporation or FWD.
**
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
**
**   3. No Misrepresentation of Source or Origin.
**
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.reporting;

/**
 * The single rectangle object inside XPR file.
 */
public class XprObjRectangle
extends XprObjGraphics
{
   /** Rounded rectangles radius constant. */
   public static final double RADIUS_SCALE = 0.25; 
   
   /** The rectangle line width. */
   private float rectLineWidth = 1.0f;
   
   /** Flag indicating rounded rectangle */
   private boolean isRounded = false;
   
   /**
    * Create a new rectangle XPR object.
    *
    * @param    rectColor
    *           Rectangle color.
    * @param    rowStart
    *           The starting row value for rectangle.
    * @param    colStart
    *           The starting column value for rectangle.
    * @param    rowEnd
    *           The ending row value for rectangle.
    * @param    colEnd
    *           The ending column value for rectangle.
    * @param    width
    *           The line width.
    * @param    rounded
    *           The rectangle is rounded when <code>TRUE</code>.
    */
   public XprObjRectangle(int rectColor, double rowStart, double colStart, double rowEnd,
                          double colEnd, int width, boolean rounded)
   {
      this(XprObjBase.TYPE_RECTANGLE, rectColor, rowStart, colStart, rowEnd, colEnd,
           width, rounded);
   }

   /**
    * Create a new rectangle XPR object for fugure that has inheritance from this rectangle.
    *
    * @param    objType
    *           Another rectangle object type.
    * @param    rectColor
    *           Rectangle color.
    * @param    rowStart
    *           The starting row value for rectangle.
    * @param    colStart
    *           The starting column value for rectangle.
    * @param    rowEnd
    *           The ending row value for rectangle.
    * @param    colEnd
    *           The ending column value for rectangle.
    * @param    width
    *           The line width.
    * @param    rounded
    *           The rectangle is rounded when <code>TRUE</code>.
    */
   public XprObjRectangle(int objType, int rectColor, double rowStart, double colStart,
                          double rowEnd, double colEnd, int width, boolean rounded)
   {
      super(objType, rectColor, rowStart, colStart, rowEnd, colEnd);
      this.rectLineWidth = (float)width * (float)Xpr2PdfWorker.DPI_DEFAULT /
                           (float)XprHelper.getPdfDPI();
      this.isRounded = rounded;
   }

   /**
    * Returns the rectangle line width in points.  Not the same as in pixels.
    *
    * @return   The line thickness in points.
    */
   public float getRectLineWidth()
   {
      return rectLineWidth;
   }

   /**
    * Returns the rounded flag for rectangle.
    *
    * @return   <code>TRUE</code> if rectangle is rounded, <code>FALSE</code> otherwise.
    */
   public boolean isRounded()
   {
      return isRounded;
   }
}