/*
** Module   : XprObjText.java
** Abstract : Class to represent text object feature for XPR content elements.
**
** Copyright (c) 2018, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 EVL 20180510 Created initial version.
** 002 EVL 20180525 Changed approach for text storing.  Only single line of text is now possible.
** 003 EVL 20180527 Code optimization and clean ups.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
**
** Additional terms under GNU Affero GPL version 3 section 7:
**
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
**
**   0. Attribution Requirement.
**
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
**
**   1. No License To Use Trademarks.
**
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
**
**   2. No Misrepresentation of Affiliation.
**
**     You may not represent yourself as Golden Code Development Corporation or FWD.
**
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
**
**   3. No Misrepresentation of Source or Origin.
**
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.reporting;

import java.util.*;

/**
 * The text object inside XPR file.
 */
public class XprObjText
extends XprObjBase
{
   /** Text buffer in a form line by line. */
   private String textBuff = null;
   
   /** Font used with current text. */
   private String fontName = null;
   
   /** Font size. */
   private int fontSize = -1;
   
   /** Bold flag. */
   private boolean bold = false;
   
   /** Italic flag. */
   private boolean italic = false;
   
   /**
    * Create a new text XPR object.
    *
    * @param    fntName
    *           The font name to be used with text object.
    * @param    fntSize
    *           The font size to be used with text object.
    * @param    isBold
    *           Whether the text to be bold or not.
    * @param    isItalic
    *           Whether the text to be italic or not.
    * @param    fgColor
    *           Text color.
    * @param    row
    *           The row value for text.
    * @param    column
    *           The column value for text.
    * @param    text
    *           The value for text.
    */
   public XprObjText(String fntName, int fntSize, boolean isBold, boolean isItalic, int fgColor,
                     double row, double column, String text)
   {
      super(XprObjBase.TYPE_TEXT, fgColor, row, column);
      fontName = fntName;
      fontSize = fntSize;
      bold = isBold;
      italic = isItalic;
      textBuff = text;
   }
   
   /**
    * Adding new text line to the object.
    *
    * @param    newLine
    *           New text line to add.
    */
   public void addText(String newLine)
   {
      textBuff = newLine;
   }
   
   /**
    * Appending new text line to the object's las line.
    *
    * @param    newText
    *           New text line to append.
    */
   public void appendText(String newText)
   {
      textBuff = textBuff.concat(newText);
   }
   
   /**
    * Returns the font name for text object.
    *
    * @return   The font name string.
    */
   public String getFontName()
   {
      return fontName;
   }

   /**
    * Returns the font size for text object.
    *
    * @return   The font size as integer value.
    */
   public int getFontSize()
   {
      return fontSize;
   }

   /**
    * Returns the font bold attribute.
    *
    * @return   <code>TRUE</code> if font is bold.
    */
   public boolean isBold()
   {
      return bold;
   }
   
   /**
    * Returns the font italic attribute.
    *
    * @return   <code>TRUE</code> if font is italic.
    */
   public boolean isItalic()
   {
      return italic;
   }
   
   /**
    * Returns the first line of the text buffer for the given object.
    *
    * @return   Text value of this object.
    */
   public String getValue()
   {
      return textBuff;
   }
}