/*
** Module   : XprPageOptions.java
** Abstract : Class to represent base object feature for XPR content elements.
**
** Copyright (c) 2018, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 EVL 20180626 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
**
** Additional terms under GNU Affero GPL version 3 section 7:
**
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
**
**   0. Attribution Requirement.
**
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
**
**   1. No License To Use Trademarks.
**
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
**
**   2. No Misrepresentation of Affiliation.
**
**     You may not represent yourself as Golden Code Development Corporation or FWD.
**
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
**
**   3. No Misrepresentation of Source or Origin.
**
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.reporting;

/**
 * The class representing single page options set/get facility for the cases we need different
 * option values for every new report page.
 */
public class XprPageOptions
{
   /** Constant to define portrait orientation. */
   public static final int PORTRAIT = 0;
   
   /** Constant to define landscape orientation. */
   public static final int LANDSCAPE = 1;
   
   /** Page width in pixels. */
   private int pageWidthPix = -1;
   
   /** Page height in pixels. */
   private int pageHeightPix = -1;
   
   /**
    * Create a new XPR page options object.
    * 
    * @param    width
    *           The page width in pixels.
    * @param    height
    *           The page height in pixels.
    * @param    pageOri
    *           The current page orientation.
    */
   public XprPageOptions(int width, int height, int pageOri)
   {
      pageWidthPix = width;
      pageHeightPix = height;
      // refresh page orientation
      setPageOrientation(pageOri);
   }

   /**
    * Returns the current page width in pixels.
    *
    * @return   The current page width.
    */
   public int getWidthPix()
   {
      return pageWidthPix;
   }
   
   /**
    * Returns the current page height in pixels.
    *
    * @return   The current page width.
    */
   public int getHeightPix()
   {
      return pageHeightPix;
   }
   
   /**
    * Changes the new page orientation.
    *
    * @param   pageNewOri
    *          The new orientation to apply, either PORTRAIT or LANDSCAPE.
    */
   public void setPageOrientation(int pageNewOri)
   {
      // change orientation to portait or to landscape
      if ((pageNewOri == PORTRAIT && pageWidthPix > pageHeightPix) ||
          (pageNewOri == LANDSCAPE && pageWidthPix < pageHeightPix))
      {
         // need one temporary var to swap values
         int tmpVar = pageWidthPix;
         pageWidthPix = pageHeightPix;
         pageHeightPix = tmpVar;
      }
   }
}