/*
** Module   : ColorTableResource.java
** Abstract : Define all APIs available via the COLOR-TABLE system handle.
**
** Copyright (c) 2014-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 MAG 20140812 Created initial version.
** 002 EVL 20160224 Javadoc fixes to make compatible with Oracle Java 8 for Solaris 10.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui;

import com.goldencode.p2j.util.*;

/**
 * Exposes the following methods or attributes from the COLOR-TABLE handle:
 * <table summary="">
 * <tr>
 *    <th>COLOR-TABLE attribute or method</th>
 *    <th>API</th>
 * </tr>
 * <tr>
 *    <td>GET-RED-VALUE</td>
 *    <td>{@link #getRedValue}</td>
 * </tr>
 * <tr>
 *    <td>SET-RED-VALUE</td>
 *    <td>{@link #setRedValue}</td>
 * </tr>
 * <tr>
 *    <td>GET-GREEN-VALUE</td>
 *    <td>{@link #getGreenValue}</td>
 * </tr>
 * <tr>
 *    <td>SET-GREEN-VALUE</td>
 *    <td>{@link #setGreenValue}</td>
 * </tr>
 * <tr>
 *    <td>GET-BLUE-VALUE</td>
 *    <td>{@link #getBlueValue}</td>
 * </tr>
 * <tr>
 *    <td>SET-BLUE-VALUE</td>
 *    <td>{@link #setBlueValue}</td>
 * </tr>
 * <tr>
 *    <td>GET-RGB-VALUE</td>
 *    <td>{@link #getRgbValue}</td>
 * </tr>
 * <tr>
 *    <td>SET-RGB-VALUE</td>
 *    <td>{@link #setRgbValue}</td>
 * </tr>
 * <tr>
 *    <td>GET-DYNAMIC</td>
 *    <td>{@link #getDynamic}</td>
 * </tr>
 * <tr>
 *    <td>SET-DYNAMIC</td>
 *    <td>{@link #setDynamic}</td>
 * </tr>
 * </table>
 * This interface is used as a returned type by the 
 * {@link handle#unwrapColorTable} API, which, depending on the 
 * handle's state, it either returns an {@link ColorTable#asHandle()} 
 * instance or a dynamic proxy using {@link handle#invalidAttrAccessProxy} 
 * <p>
 * IMPORTANT: The index of an entry in color table is 0 based.
 */
@LegacyResource(resource = LegacyResource.PSEUDO_WIDGET)
public interface ColorTableResource
extends CommonHandle,
        InstantiatingProcedure,
        NumEntries
{
   /**
    * Returns the red component of an entry in the color table.
    * 
    * @param    index
    *           An integer expression that specifies an entry in the color table.
    *           
    * @return   See above.
    */
   @LegacyMethod(name = "GET-RED-VALUE")
   public integer getRedValue(int64 index);
   
   /**
    * Returns the red component of an entry in the color table.
    * 
    * @param    index
    *           An integer expression that specifies an entry in the color table.
    *           
    * @return   See above.
    */
   @LegacyMethod(name = "GET-RED-VALUE")
   public integer getRedValue(long index);

   /**
    * Specifies the red component of an entry in the color table.
    * 
    * @param    index
    *           An integer expression that specifies an entry in the color table.
    * @param    redValue
    *           An integer expression that specifies the red RGB component of an entry in the
    *           color table. The value must be in the range 0 to 255.
    *           
    * @return   If the operation is successful, the method returns TRUE.
    */
   @LegacyMethod(name = "SET-RED-VALUE")
   public logical setRedValue(int64 index, int64 redValue);
   
   /**
    * Specifies the red component of an entry in the color table.
    * 
    * @param    index
    *           An integer expression that specifies an entry in the color table.
    * @param    redValue
    *           An integer expression that specifies the red RGB component of an entry in the
    *           color table. The value must be in the range 0 to 255.
    *           
    * @return   If the operation is successful, the method returns TRUE.
    */
   @LegacyMethod(name = "SET-RED-VALUE")
   public logical setRedValue(int64 index, long redValue);

   /**
    * Specifies the red component of an entry in the color table.
    * 
    * @param    index
    *           An integer expression that specifies an entry in the color table.
    * @param    redValue
    *           An integer expression that specifies the red RGB component of an entry in the
    *           color table. The value must be in the range 0 to 255.
    *           
    * @return   If the operation is successful, the method returns TRUE.
    */
   @LegacyMethod(name = "SET-RED-VALUE")
   public logical setRedValue(long index, int64 redValue);
   
   /**
    * Specifies the red component of an entry in the color table.
    * 
    * @param    index
    *           An integer expression that specifies an entry in the color table.
    * @param    redValue
    *           An integer expression that specifies the red RGB component of an entry in the
    *           color table. The value must be in the range 0 to 255.
    *           
    * @return   If the operation is successful, the method returns TRUE.
    */
   @LegacyMethod(name = "SET-RED-VALUE")
   public logical setRedValue(long index, long redValue);

   /**
    * Returns the green component of an entry in the color table.
    * 
    * @param    index
    *           An integer expression that specifies an entry in the color table.
    *           
    * @return   See above.
    */
   @LegacyMethod(name = "GET-GREEN-VALUE")
   public integer getGreenValue(int64 index);
   
   /**
    * Returns the green component of an entry in the color table.
    * 
    * @param    index
    *           An integer expression that specifies an entry in the color table.
    *           
    * @return   See above.
    */
   @LegacyMethod(name = "GET-GREEN-VALUE")
   public integer getGreenValue(long index);

   /**
    * Specifies the green component of an entry in the color table.
    * 
    * @param    index
    *           An integer expression that specifies an entry in the color table.
    * @param    greenValue
    *           An integer expression that specifies the green RGB component of an entry in the
    *           color table. The value must be in the range 0 to 255.
    *           
    * @return   If the operation is successful, the method returns TRUE.
    */
   @LegacyMethod(name = "SET-GREEN-VALUE")
   public logical setGreenValue(int64 index, int64 greenValue);

   /**
    * Specifies the green component of an entry in the color table.
    * 
    * @param    index
    *           An integer expression that specifies an entry in the color table.
    * @param    greenValue
    *           An integer expression that specifies the green RGB component of an entry in the
    *           color table. The value must be in the range 0 to 255.
    *           
    * @return   If the operation is successful, the method returns TRUE.
    */
   @LegacyMethod(name = "SET-GREEN-VALUE")
   public logical setGreenValue(int64 index, long greenValue);

   /**
    * Specifies the green component of an entry in the color table.
    * 
    * @param    index
    *           An integer expression that specifies an entry in the color table.
    * @param    greenValue
    *           An integer expression that specifies the green RGB component of an entry in the
    *           color table. The value must be in the range 0 to 255.
    *           
    * @return   If the operation is successful, the method returns TRUE.
    */
   @LegacyMethod(name = "SET-GREEN-VALUE")
   public logical setGreenValue(long index, int64 greenValue);

   /**
    * Specifies the green component of an entry in the color table.
    * 
    * @param    index
    *           An integer expression that specifies an entry in the color table.
    * @param    greenValue
    *           An integer expression that specifies the green RGB component of an entry in the
    *           color table. The value must be in the range 0 to 255.
    *           
    * @return   If the operation is successful, the method returns TRUE.
    */
   @LegacyMethod(name = "SET-GREEN-VALUE")
   public logical setGreenValue(long index, long greenValue);
   
   /**
    * Returns the blue component of an entry in the color table.
    * 
    * @param    index
    *           An integer expression that specifies an entry in the color table.
    *           
    * @return   See above.
    */
   @LegacyMethod(name = "GET-BLUE-VALUE")
   public integer getBlueValue(int64 index);
   
   /**
    * Returns the blue component of an entry in the color table.
    * 
    * @param    index
    *           An integer expression that specifies an entry in the color table.
    *           
    * @return   See above.
    */
   @LegacyMethod(name = "GET-BLUE-VALUE")
   public integer getBlueValue(long index);

   /**
    * Specifies the blue component of an entry in the color table.
    * 
    * @param    index
    *           An integer expression that specifies an entry in the color table.
    * @param    blueValue
    *           An integer expression that specifies the blue RGB component of an entry in the
    *           color table. The value must be in the range 0 to 255.
    *           
    * @return   If the operation is successful, the method returns TRUE.
    */
   @LegacyMethod(name = "SET-BLUE-VALUE")
   public logical setBlueValue(int64 index, int64 blueValue);

   /**
    * Specifies the blue component of an entry in the color table.
    * 
    * @param    index
    *           An integer expression that specifies an entry in the color table.
    * @param    blueValue
    *           An integer expression that specifies the blue RGB component of an entry in the
    *           color table. The value must be in the range 0 to 255.
    *           
    * @return   If the operation is successful, the method returns TRUE.
    */
   @LegacyMethod(name = "SET-BLUE-VALUE")
   public logical setBlueValue(int64 index, long blueValue);

   /**
    * Specifies the blue component of an entry in the color table.
    * 
    * @param    index
    *           An integer expression that specifies an entry in the color table.
    * @param    blueValue
    *           An integer expression that specifies the blue RGB component of an entry in the
    *           color table. The value must be in the range 0 to 255.
    *           
    * @return   If the operation is successful, the method returns TRUE.
    */
   @LegacyMethod(name = "SET-BLUE-VALUE")
   public logical setBlueValue(long index, int64 blueValue);

   /**
    * Specifies the blue component of an entry in the color table.
    * 
    * @param    index
    *           An integer expression that specifies an entry in the color table.
    * @param    blueValue
    *           An integer expression that specifies the blue RGB component of an entry in the
    *           color table. The value must be in the range 0 to 255.
    *           
    * @return   If the operation is successful, the method returns TRUE.
    */
   @LegacyMethod(name = "SET-BLUE-VALUE")
   public logical setBlueValue(long index, long blueValue);

   /**
    * Returns an integer that represents a combination of the red, green, and blue 
    * values of an entry in the color table.
    * 
    * @param    index
    *           An integer expression that specifies an entry in the color table.
    *           
    * @return   See above.
    */
   @LegacyMethod(name = "GET-RGB-VALUE")
   public integer getRgbValue(int64 index);
   
   /**
    * Returns an integer that represents a combination of the red, green, and blue 
    * values of an entry in the color table.
    * 
    * @param    index
    *           An integer expression that specifies an entry in the color table.
    *           
    * @return   See above.
    */
   @LegacyMethod(name = "GET-RGB-VALUE")
   public integer getRgbValue(long index);

   /**
    * Specifies a combination of the red, green, and blue values of an entry in the color table.
    * 
    * @param    index
    *           An integer expression that specifies an entry in the color table.
    * @param    rgbValue
    *           An integer expression that specifies the RGB component of an entry in the color
    *           table. You can obtain this value from the color property of an ActiveX control, 
    *           by using the RGB-VALUE function, or by using the GET-RGB-VALUE() method.    
    *           
    * @return   If the operation is successful, the method returns TRUE.
    */
   @LegacyMethod(name = "SET-RGB-VALUE")
   public logical setRgbValue(int64 index, int64 rgbValue);

   /**
    * Specifies a combination of the red, green, and blue values of an entry in the color table.
    * 
    * @param    index
    *           An integer expression that specifies an entry in the color table.
    * @param    rgbValue
    *           An integer expression that specifies the RGB component of an entry in the color
    *           table. You can obtain this value from the color property of an ActiveX control, 
    *           by using the RGB-VALUE function, or by using the GET-RGB-VALUE() method.    
    *           
    * @return   If the operation is successful, the method returns TRUE.
    */
   @LegacyMethod(name = "SET-RGB-VALUE")
   public logical setRgbValue(int64 index, long rgbValue);

   /**
    * Specifies a combination of the red, green, and blue values of an entry in the color table.
    * 
    * @param    index
    *           An integer expression that specifies an entry in the color table.
    * @param    rgbValue
    *           An integer expression that specifies the RGB component of an entry in the color
    *           table. You can obtain this value from the color property of an ActiveX control, 
    *           by using the RGB-VALUE function, or by using the GET-RGB-VALUE() method.    
    *           
    * @return   If the operation is successful, the method returns TRUE.
    */
   @LegacyMethod(name = "SET-RGB-VALUE")
   public logical setRgbValue(long index, int64 rgbValue);

   /**
    * Specifies a combination of the red, green, and blue values of an entry in the color table.
    * 
    * @param    index
    *           An integer expression that specifies an entry in the color table.
    * @param    rgbValue
    *           An integer expression that specifies the RGB component of an entry in the color
    *           table. You can obtain this value from the color property of an ActiveX control, 
    *           by using the RGB-VALUE function, or by using the GET-RGB-VALUE() method.    
    *           
    * @return   If the operation is successful, the method returns TRUE.
    */
   @LegacyMethod(name = "SET-RGB-VALUE")
   public logical setRgbValue(long index, long rgbValue);

   /**
    * Returns TRUE if the entry in the color table is a dynamic color.
    *  
    * @param    index
    *           An integer expression that specifies an entry in the color table.
    *           
    * @return   See above.
    */
   @LegacyMethod(name = "GET-DYNAMIC")
   public logical getDynamic(int64 index);
   
   /**
    * Returns TRUE if the entry in the color table is a dynamic color.
    * 
    * @param    index
    *           An integer expression that specifies an entry in the color table.
    *           
    * @return   See above.
    */
   @LegacyMethod(name = "GET-DYNAMIC")
   public logical getDynamic(long index);

   /**
    * Sets a color entry to a dynamic or static color.
    * 
    * @param    index
    *           An integer expression that specifies an entry in the color table.
    * @param    dynamic
    *           Logical expression that specifies the dynamic status of an entry in the color
    *           table. If dynamic is TRUE sets the entry to a dynamic color and sets the entry
    *           to a static color if dynamic is FALSE.    
    *                      
    * @return   If the operation is successful, the method returns TRUE.
    */
   @LegacyMethod(name = "SET-DYNAMIC")
   public logical setDynamic(int64 index, logical dynamic);
   
   /**
    * Sets a color entry to a dynamic or static color.
    * 
    * @param    index
    *           An integer expression that specifies an entry in the color table.
    * @param    dynamic
    *           Logical expression that specifies the dynamic status of an entry in the color
    *           table. If dynamic is TRUE sets the entry to a dynamic color and sets the entry
    *           to a static color if dynamic is FALSE.    
    *                      
    * @return   If the operation is successful, the method returns TRUE.
    */
   @LegacyMethod(name = "SET-DYNAMIC")
   public logical setDynamic(int64 index, boolean dynamic);

   /**
    * Sets a color entry to a dynamic or static color.
    * 
    * @param    index
    *           An integer expression that specifies an entry in the color table.
    * @param    dynamic
    *           Logical expression that specifies the dynamic status of an entry in the color
    *           table. If dynamic is TRUE sets the entry to a dynamic color and sets the entry
    *           to a static color if dynamic is FALSE.    
    *                      
    * @return   If the operation is successful, the method returns TRUE.
    */
   @LegacyMethod(name = "SET-DYNAMIC")
   public logical setDynamic(long index, logical dynamic);

   /**
    * Sets a color entry to a dynamic or static color.
    * 
    * @param    index
    *           An integer expression that specifies an entry in the color table.
    * @param    dynamic
    *           Logical expression that specifies the dynamic status of an entry in the color
    *           table. If dynamic is TRUE sets the entry to a dynamic color and sets the entry
    *           to a static color if dynamic is FALSE.    
    *                      
    * @return   If the operation is successful, the method returns TRUE.
    */
   @LegacyMethod(name = "SET-DYNAMIC")
   public logical setDynamic(long index, boolean dynamic);
}