/*
** Module   : ControlEntity.java
** Abstract : server side control widget parent class
**
** Copyright (c) 2005-2018, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ----------------Description-----------------
** 001 NVS 20051026   @23486 Created initial version.
** 002 GES 20060114   @23908 Removed totally useless screen-value attr.
** 003 SIY 20060202   @24235 Added support for TO option.
** 004 SIY 20060203   @24247 Added force-label flag.
** 005 GES 20060204   @24262 Added getType() method to obtain the data
**                           type the widget represents.  Also converted
**                           the instance data to primitives instead of
**                           wrappers.
** 006 SIY 20060209   @24435 Implemented tracking of widget data type.
**                           Fixed method name. Methods from derived 
**                           classes moved here.
** 007 SIY 20060426   @25913 Changed signature of getNextTabItem.
** 008 SIY 20060515   @26150 Moved COLUMN-LABEL attribute from 
**                           GenericWidget.
** 009 SIY 20060720   @28148 Added support for INDEX attribute.
** 010 GES 20060803   @28370 Some cleanup and refactoring.
** 011 NVS 20060918   @29643 Added setAttrSpace() method.
** 012 SIY 20061102   @30936 Added alignment mode constants and method.
** 013 NVS 20070426   @33243 For the first time, implemented 
**                           getNextTabItem() method, which was a stub.
** 014 GES 20080320   @37596 Changed getType() to getDataClass() to avoid
**                           method name conflicts with the TYPE attr.
** 015 SIY 20090604   @42611 Forcing labels resets NO-LABEL flag.
** 016 SIY 20101122          Added generic in one method return type in order
**                           to make it compatible with other changes.
** 017 SIY 20111101          Updated imports.
** 018 CA  20130222          Added NPE protection for cases when a widget is used before is 
**                           attached to a frame (after a CREATE WIDGET statement is used).
** 019 CA  20131115          Changes related to properly instantiate dynamic resources.
** 020 HC  20140911          GUI coordinate support - data types refactoring.
** 021 CA  20140926          Added shared widget configuration support, to allow GUI/ChUI
**                           concrete implementation for the same widget ID.  Refactored the
**                           widget configuration classes: all fields were made public; these
**                           classes need to be as dumb as possible, all logic related to
**                           setting/getting a certain field should be at the widget or at the
**                           caller. Refs #2254
** 022 HC  20141207          Implemented runtime support for widget coordinate attributes.
** 023 GES 20141124          Added modified attr.  Moved setColon() to BaseEntity since customer
**                           code shows it can be used widely for other widgets (undocumented
**                           feature).
** 024 GES 20150104          Modifications for more correct modified attribute support.
** 025 CA  20150105          Changes for SIDE-LABEL-HANDLE runtime support.
** 026 CA  20150123          Fixed handling of side-labels in shared frames.
** 027 CA  20150219          Added support for DATA-TYPE attribute.
** 028 CA  20150222          Fixed label's COLUMN and WIDTH-CHARS attributes (in ChUI).
**                           When shared frames are created, the client-side label needs to be 
**                           reset.
** 029 IAS 20150224          added setAutoresize(boolean).
** 030 HC  20150323          Removed business logic from config classes.
** 031 CA  20150722          Removed resetLabel, as layout was changed to be performed only once.
** 032 IAS 20150801          Added support for the INDEX attribute.
** 033 IAS 20150926          Added pushScreenDefinition to setAutoResize.
** 034 OM  20150924          setAlign() moved to BaseEntity. Fixed NPE in case of the late
**                           FRAME assignment
** 035 CA  20151016          Fixed NPE when LABEL attr for dyn widget is set before FRAME.
**                           Fixed auto-resize for FILL-IN widgets.
** 036 IAS 20151020          Added check to avoid NPE in case of the late FRAME attribute
**                           assignment for dynamic widgets.
**     IAS 20151029          VALIDATE() method implementation
** 037 CA  20160303          Made attribute setters be a no-op if the value doesn't change.
** 038 GES 20160616          Reworked validation processing to eliminate unnecessary processing.
** 039 OM  20161207          Dropped dead code from shared frame changes.
** 040 EVL 20170529          Adding Droppable interface to handle file list dropping.
** 041 CA  20170825          Added setter for NO-ATTR-SPACE attribute.
** 042 SBI 20161101          Changed the return value of getNextTabItem() to be a handle.
** 043 SVL 20171025          Fixed widget name in error message. Use messages instead of modal
**                           dialogs for validation errors.
** 044 CA  20180130          Replaced pushScreenDefinition() with pushWidgetAttr() for cases when
**                           business logic is assigning a widget attribute.
** 045 ECF 20180403          Removed UnimplementedFeature.todo calls in is/setModified.
** 046 HC  20180628          Added side-labels to the widget chain.
** 047 CA  20180723          The side-label must know its associated widget (to be able to access
**                           or set the SCREEN-VALUE attribute).
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui;

import com.goldencode.p2j.util.*;

import java.util.*;

import com.goldencode.p2j.ui.client.format.*;

/**
 * Represents widget functionality that is associated with all widgets that
 * represent data OR which provide a control feature (like a button).
 */
public class ControlEntity<T extends ControlConfig>
extends BaseEntity<T>
implements IndexedInterface,
           AutoResizableWidget,
           Droppable,
           WidgetTree
{
   /** Alignment mode - align widget left */
   public static final int ALIGN_LEFT  = 1;

   /** Alignment mode - align widget right */
   public static final int ALIGN_RIGHT = 2;
   
   /** Alignment mode - align widget to label colon */
   public static final int ALIGN_COLON = 3;
   
   /** Type of the variable in the widget. */
   private Class<?> dataTypeClass = null;
   
   /**
    * Reference to the side-label widget (as returned by the SIDE-LABEL-HANDLE).  When
    * <code>null</code>, it means no side-label exists (thus SIDE-LABEL-HANDLE is unknown).
    */
   private LiteralWidget sideLabel = null;
   
   /**
    * Constructor for use by sub-classes.
    *
    * @param    dynamic
    *           Flag indicating if this is a static or dynamic resource.
    * @param    ctrl
    *           Configuration container.
    */
   public ControlEntity(boolean dynamic, T ctrl)
   {
      super(dynamic, ctrl);
   }
   
   /**
    * Accesses the data type which the widget represents. 
    *
    * @return   The class of the value that the widget represents or
    *           <code>null</code> if this widget does not represent any
    *           data (e.g. the widget is a button).
    */
   public Class<?> getDataClass()
   {
      if (dataTypeClass == null)
      {
         BaseDataType var = DisplayFormat.instanceOfType(config.dataType);
         
         if (var == null)
         {
            throw new RuntimeException("Attempt to set invalid data type " +  config.dataType);
         }
         
         dataTypeClass = var.getClass();
      }

      return dataTypeClass;
   }

   /**
    * Sets the column label.
    * 
    * @param   clabel
    *          column label text
    */
   public void setColumnLabel(character clabel)
   {
      setColumnLabel(clabel.isUnknown() ? "" : clabel.toStringMessage());
   }

   /**
    * Sets the column label.
    * 
    * @param   clabel
    *          column label text
    */
   public void setColumnLabel(String clabel)
   {
      if (Utils.equals(config.clabel, clabel))
      {
         return;
      }

      config.clabel = clabel;
      pushWidgetAttr("clabel", config.clabel);
   }
   
   /**
    * Force displaying of label.
    *
    * @param    force
    *           New value for the flag.
    */
   public void setForceLabel(logical force)
   {
      setForceLabel(force.booleanValue());
   }

   /**
    * Force displaying of label.
    *
    * @param    force
    *           New value for the flag.
    */
   public void setForceLabel(final boolean force)
   {
      if (force)
      {
         config.labels = true;
      }
      
      config.forceLabel = force;

      // this is a frame definition option, can't be used via widget handle
   }
   
   /**
    * Getter for the NEXT-TAB-ITEM read-only attribute.
    *
    * @return   The value of the NEXT-TAB-ITEM attribute.
    */
   public handle getNextTabItem()
   {
      return LogicalTerminal.nextTabItem(getFrame(), this);
   }
   
   /**
    * Getter for the PREV-TAB-ITEM read-only attribute.
    *
    * @return   The value of the PREV-TAB-ITEM attribute.
    */
   public handle getPreviousTabItem()
   {
      return LogicalTerminal.previousTabItem(getFrame(), this);
   }
   
   /**
    * Sets the rightmost column of the widget.
    * 
    * @param   pos
    *          The 1-based column number. This is converted to a 0-based
    *          column number for the internal runtime.
    */
   public void setTo(NumberType pos)
   {
      setTo(pos.intValue());
   }

   /**
    * Sets the rightmost column of the widget.
    * 
    * @param   pos
    *          The 1-based column number. This is converted to a 0-based
    *          column number for the internal runtime.
    */
   public void setTo(double pos)
   {
      double newCol = pos - 1;
      ConfigHelper.setTo(config, newCol);
      pushWidgetAttr(new String[] {"initColumn", "align"},
                     new Object[] {config.initColumn, config.align});
   }
   
   /**
    * Gets the column number to which colon between label and widget should be
    * aligned. Note that Progress does not have corresponding attribute so
    * calls to this method are unlikely.
    * 
    * @return  The 1-based column of the desired colon position. This is
    *          converted from a 0-based column number for the internal runtime.
    */
   public decimal getColon()
   {
      double col = ConfigHelper.getColon(config) + 1;
      return new decimal(col);
   }

   /**
    * Sets new value of DATA-TYPE attribute.
    *
    * @param dataType
    *        new value of DATA-TYPE attribute
    */
   @Override
   public void setDataType(character dataType)
   {
      if (dataType == null || dataType.isUnknown())
      {
         String txt = 
           "**Attribute DATA-TYPE for the FILL-IN widget has an invalid value of UNKNOWN. (4056)";
         ErrorManager.recordOrShowError(4056, txt, false, false, false);
      }
      
      setDataType(dataType.toStringMessage());
   }
   
   /**
    * Set type of the bound variable.
    * 
    * @param   dataType
    *          New data type name.
    */
   @Override
   public void setDataType(String dataType)
   {
      if (Utils.equals(config.dataType, dataType))
      {
         return;
      }

      config.dataType = dataType;
      
      // resolve the class now.
      getDataClass();

      pushWidgetAttr("dataType", config.dataType);
   }

   /**
    * Queries the DATA-TYPE attribute of this widget.
    *
    * @return A character value that represents the data type of the field
    *         associated with the widget.
    */
   @Override
   public character getDataType()
   {
      return new character(config.dataType);
   }

   /**
    * Queries if the SCREEN-VALUE attribute for this widget was changed.
    * 
    * @return  If <code>true</code>, the widget's screen value was changed.
    */
   public logical isModified()
   {
      return new logical(config.modified);
   }
   
   /**
    * Defines if the SCREEN-VALUE attribute for this widget has changed.
    * 
    * @param   modified
    *          <code>true</code> to mark the screen value as changed.
    */
   public void setModified(boolean modified)
   {
      if (config.modified == modified)
      {
         return;
      }
      
      ConfigHelper.setModified(config, modified);
      pushWidgetAttr(new String[] {"modified", "state"},
                     new Object[] {config.modified, config.state});
   }
   
   /**
    * Defines if the SCREEN-VALUE attribute for this widget has changed.
    * 
    * @param   modified
    *          <code>true</code> to mark the screen value as changed.
    */
   public void setModified(logical modified)
   {
      if (modified == null || modified.isUnknown())
      {
         UnimplementedFeature.todo("MODIFIED attribute unknown behavior is not known.");
         return;
      }
      
      setModified(modified.booleanValue()); 
   }
   
   /**
    * Set new value to NO-LABELS flag.
    * 
    * @param   noLabels
    *          New value of the flag.
    */
   public void setNoLabels(boolean noLabels)
   {
      if (config.labels == !noLabels)
      {
         return;
      }
      
      config.labels = !noLabels;
      pushWidgetAttr("labels", config.labels);
   }
   
   /**
    * Sets the value of the ATTR-SPACE attribute.
    * 
    * @param   attrSpace
    *          The new boolean value for the ATTR-SPACE attribute.
    */
   public void setAttrSpace(boolean attrSpace)
   {
      config.attrSpace = attrSpace;
      pushWidgetAttr("attrSpace", config.attrSpace);
   }
   
   /**
    * Sets the value of the NO-ATTR-SPACE attribute.
    * 
    * @param   noAttrSpace
    *          The new boolean value for the NO-ATTR-SPACE attribute.
    */
   public void setNoAttrSpace(boolean noAttrSpace)
   {
      setAttrSpace(!noAttrSpace);
   }

   /**
    * Blocks attempts to get the TITLE attribute as it is undefined.
    * 
    * @return Always throws RuntimeException. 
    */
   public character getTitle() 
   {
      throw new RuntimeException("TITLE attribute is undefined");
   }

   /**
    * Blocks attempts to set the TITLE attribute as it is undefined.
    * 
    * @param    t
    *           Unused.
    */
   public void setTitle(character t) 
   {
      throw new RuntimeException("TITLE attribute is undefined");
   }

   /**
    * Blocks attempts to set the TITLE attribute as it is undefined.
    * 
    * @param    t
    *           Unused.
    */
   public void setTitle(String t)
   {
      throw new RuntimeException("TITLE attribute is undefined");
   }

   /**
    * Set INDEX attribute value.
    * 
    * @param    index
    *           New value for the attribute.
    */
   public void setIndex(int index)
   {
      this.config.index = index;
      pushWidgetAttr("index", config.index);
   }

   /**
    * Set INDEX attribute value.
    * 
    * @param    index
    *           New value for the attribute.
    */
   public void setIndex(NumberType index)
   {
      if (index.isUnknown())
         return;
      
      setIndex(index.intValue());
   }

   /**
    * Get current value of the INDEX attribute.
    * 
    * @return   The current value of the attribute.
    */
   @Override
   public integer getIndex()
   {
      return new integer(config.index);
   }
   
   /**
    * Get current value of the AUTO-RESIZE attribute.
    * 
    * @return   The current value of the attribute.
    */
   public logical isAutoResize()
   {
      return new logical(config.autoResize);
   }
   
   /**
    * Set AUTO-RESIZE attribute value.
    * 
    * @param    autoResize
    *           New value for the attribute.
    */
   public void setAutoResize(logical autoResize)
   {
      if (autoResize.isUnknown())
      {
         ErrorManager.recordOrShowError(4083, String.format(
               "Unable to assign UNKNOWN value to attribute AUTO-RESIZE on %s %s.", type(),
                  widgetName()), false);
         return;
      }
      setAutoResize(autoResize.booleanValue());
   }

   /**
    * Set AUTO-RESIZE attribute value.
    * 
    * @param    autoResize
    *           New value for the attribute.
    */
   public void setAutoResize(boolean autoResize)
   {
      if (config.autoResize == autoResize)
      {
         return;
      }

      config.autoResize = autoResize;
      pushWidgetAttr("autoResize", config.autoResize);
   }


   /**
    * Set AUTO-RESIZE to false;
    * 
    */
   public void resetAutoResize()
   {
      config.autoResize = false;
   }
   
   /**
    * Obtains the text widget that is used as the side label for this widget. This is the
    * equivalent of the SIDE-LABEL-HANDLE attribute of COMBO-BOX, EDITOR, FILL-IN, RADIO-SET,
    * SELECTION-LIST, SLIDER and TEXT widgets.
    *
    * @return   The handle to the text widget that is the side label of this widget.
    */
   public handle getSideLabelHandle()
   {
      return sideLabel == null ? new handle() : new handle(sideLabel);
   }

   /**
    * Gets DROP-TARGET attribute.
    * 
    * @return  DROP-TARGET current value.
    */
   @Override
   public logical isDropTarget()
   {
      return new logical(config.dropTarget);
   }
   
   /**
    * Sets DROP-TARGET attribute.
    * 
    * @param   flag
    *          New value for the attribute.
    */
   @Override
   public void setDropTarget(boolean flag)
   {
      if (config.dropTarget != flag)
      {
         config.dropTarget = flag;
         pushWidgetAttr("dropTarget", config.dropTarget);
      }
   }

   /**
    * Accesses the first widget in the group of related widgets.
    *
    * @return   An instance of {@link GenericWidget}, may be {@code this} object.
    */
   @Override
   public GenericWidget firstChild()
   {
      return sideLabel == null ? this : sideLabel;
   }

   /**
    * Accesses the last widget in the group of related widgets.
    *
    * @return   An instance of {@link GenericWidget}, may be {@code this} object.
    */
   @Override
   public GenericWidget lastChild()
   {
      return this;
   }

   /**
    * Obtains the previous sibling widget in the current field group.
    *
    * @return   The previous sibling or an unknown handle if this only widget or if this is the
    *           first widget.
    */
   @Override
   public handle getPrevSibling()
   {
      return sideLabel != null ? new handle(sideLabel) : super.getPrevSibling();
   }

   /**
    * Sets the value for the FONT attribute.
    * Only valid width values must be passed in the font argument.
    * 
    * @param    font
    *           Valid font value.
    */
   @Override
   protected void changeFontWorker(int font)
   {
      super.changeFontWorker(font);

      if (config.autoResize && frame != null && !frame.isInsideSetup())
      {
         config.fontChanged = true;
         // let the width compute automatically
         config.fixedWidth = false;
      }
   }
   
   /**
    * The method that actually performs assignment of the HEIGHT-CHARS attribute.  
    * 
    * @param    height
    *           The attribute value.
    */
   @Override
   protected void setHeightCharsWorker(double height)
   {
      config.autoResize = false;
      super.setHeightCharsWorker(height);
   }

   /**
    * The method that actually performs assignment of the WIDTH-CHARS attribute.  
    * 
    * @param    width
    *           The attribute value.
    */
   @Override
   protected void setWidthCharsWorker(double width)
   {
      config.autoResize = false;
      super.setWidthCharsWorker(width);
   }

   /**
    * The method that actually performs assignment of the WIDTH=PIXELS attribute.  
    * 
    * @param    widthPixels
    *           The attribute value.
    */
   @Override
   protected void setWidthPixelsWorker(int widthPixels)
   {
      config.autoResize = false;
      super.setWidthPixelsWorker(widthPixels);
   }

   /**
    * The method that actually performs assignment of the HEIGHT-PIXELS attribute.  
    * 
    * @param    heightPixels
    *           The attribute value.
    */
   @Override
   protected void setHeightPixelsWorker(int heightPixels)
   {
      config.autoResize = false;
      super.setHeightPixelsWorker(heightPixels);
   }
   
   /**
    * Collect the size-related attributes which can be assigned by this widget.
    * 
    * @param    names
    *           The collection where to add the field names.
    * @param    vals
    *           The collection where to add the field values.
    */
   @Override
   protected void getSizeAttrs(List<String> names, List<Object> vals)
   {
      super.getSizeAttrs(names, vals);

      names.add("autoResize");
      vals.add(config.autoResize);
   }

   /**
    * Check if a side-label can be attached.
    * <p>
    * This is possible only if the widget accepts a side-label the
    * ({@link ControlConfig#possibleSideLabel} flag is <code>true</code>) and also the frame is
    * configured with the SIDE-LABELS option.
    * 
    * @return   <code>true</code> if the SIDE-LABEL-HANDLE needs to return a valid handle.
    */
   protected boolean hasSideLabelHandle()
   {
      FrameConfig fconfig = frame.getFrameWidget().config;
      
      // options which affect this:
      // 1. frame
      // SIDE-LABELS
      // NO-LABELS
      // 2. widget
      // explicit label (forceLabel flag)
      // no label (label field to null)
      // NO-LABELS
      
      return config.possibleSideLabel && 
             fconfig.sideLabels       &&
             ((fconfig.labels && config.labels) ||
              (!fconfig.labels && config.labels && config.forceLabel)); 
   }
   
   /**
    * Process the side-label for this widget.
    * <p>
    * If the widget accepts a side-label, it will construct it if {@link #sideLabel} is not 
    * already set.
    * 
    * @param    explicit
    *           <code>true</code> if this label is explicitly set via the LABEL attribute.
    * 
    * @return   The ID for the side-label widget (as reported by {@link ControlConfig#sideLabelId})
    *           or <code>-1</code> if none exists.
    */
   @Override
   protected int processSideLabel(final boolean explicit)
   {
      if (frame != null && !frame.isInsideSetup() && hasSideLabelHandle() && config.label != null)
      {
         // do this only once. nothing is initialized here.  
         // all info will be coming from the client-side

         if (sideLabel == null)
         {
            if (config.sideLabelId == -1)
            {
               sideLabel = LiteralWidget.createSideLabel(this);
               sideLabel.config.widthChars = -1;
               sideLabel.config.align = ControlEntity.ALIGN_RIGHT;
               sideLabel.setFrame(ControlEntity.this.getFrame());
   
               config.sideLabelId = WidgetId.nextID().asInt();
               sideLabel.config.id = new WidgetId(config.sideLabelId);
   
               LogicalTerminal.registerWidget(sideLabel, config.sideLabelId);
               ConfigManager.getInstance().addWidgetConfig(sideLabel.config);
            }
            else
            {
               // someone else has created it (i.e. a shared frame), retrieve it from the registry
               sideLabel = (LiteralWidget) LogicalTerminal.getWidgetForId(config.sideLabelId);
            }
         }
         
         if (explicit)
         {
            sideLabel.config.align = ControlEntity.ALIGN_LEFT;
         }
         
         // we are just updating the screen value, we don't need to evaluate header expressions
         // or other dynamic content
         frame.getFrameBufferRaw().putScreenValue(config.sideLabelId, new character(config.label));
      }
      
      return config.sideLabelId;
   }
   
   /**
    * Processes validation rules (if they exist), error checking and any other constraint
    * processing that needs to be applied.  This is used to check if a proposed update to an
    * object will pass validation.
    *
    * @param    enabledOnly
    *           If <code>true</code>, VALIDATE will
    *           check enabled fields only. Otherwise, the VALIDATE method validates all fields,
    *           whether enabled or not.
    *
    * @return   {@code true} if the validation is successful.
    */
   protected logical validateFields(boolean enabledOnly)
   {
      if (enabledOnly && !config.enabled)
      {
         return new logical(true);
      }

      String validationError = null;
      
      GenericFrame frame = getFrame();
      
      if (frame == null)
      {
         return new logical(true);
      }

      // we are just trying to validate the widget, we don't need to update dynamic headers/title
      ScreenBuffer sb = frame.getFrameBufferRaw();
      
      if (sb == null)
      {
         return new logical(true);
      }
      
      int id = getId();
      
      // get the proposed widget value
      BaseDataType val = (BaseDataType) sb.getWidgetValue(id);

      if (val == null)
      {
         val = (BaseDataType) frame.getter(id, getDataClass(), true, false);
      }
      
      // validate the change
      validationError = validate(val, true);

      // create and return the validation result instance
      if (validationError == null)
      {
         return new logical(true);
      }
      else if (config.enabled)
      {
         ErrorManager.displayError(validationError, true);
      }
      
      return new logical(false);
   }
}