/*
** Module   : Droppable.java
** Abstract : File names list drop into widgets definition. 
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 EVL 20170526 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui;

import com.goldencode.p2j.util.*;

/**
 * Methods and attributes for widgets supporting file manes list dropping. 
 */
public interface Droppable
{
   /**
    * Gets DROP-TARGET attribute.
    * 
    * @return  DROP-TARGET current value.
    */
   @LegacyAttribute(name = "DROP-TARGET")
   public logical isDropTarget();
   
   /**
    * Sets DROP-TARGET attribute.
    * 
    * @param   flag
    *          New value for the attribute.
    */
   @LegacyAttribute(name = "DROP-TARGET", setter = true)
   public void setDropTarget(boolean flag);
   
   /**
    * Sets DROP-TARGET attribute.
    * 
    * @param   flag
    *          New value for the attribute.
    */
   @LegacyAttribute(name = "DROP-TARGET", setter = true)
   default public void setDropTarget(logical flag)
   {
      if (flag != null && !flag.isUnknown())
      {
         setDropTarget(flag.booleanValue());
      }
   }

   /**
    * Terminates file dropping operation and clean up resources.
    * 
    * @return  <code>TRUE</code> in success, <code>FALSE</code> otherwise.
    */
   @LegacyMethod(name = "END-FILE-DROP")
   default public logical endFileDrop()
   {
      // TODO: stub only for now to get compiled
      return new logical(LogicalTerminal.getClient().endFileDrop());
   }
   
   /**
    * Gets the number of the dropped files.
    * 
    * @return  The integer number for dropped file list size.
    */
   @LegacyAttribute(name = "NUM-DROPPED-FILES")
   default public integer getNumDroppedFiles()
   {
      return new integer(LogicalTerminal.getClient().getNumDroppedFiles());
   }
   
   /**
    * Gets the name of the dropped file from internal 1-based list list.
    * 
    * @param   index
    *          1-based number of the file name to get.
    *          
    * @return  The character name of the file with given index.
    */
   @LegacyMethod(name = "GET-DROPPED-FILE")
   default public character getDroppedFile(int index)
   {
      return new character(LogicalTerminal.getClient().getDroppedFile(index));
   }
   
   /**
    * Gets the name of the dropped file from internal 1-based list list.
    * 
    * @param   index
    *          1-based number of the file name to get.
    *          
    * @return  The character name of the file with given index.
    */
   @LegacyMethod(name = "GET-DROPPED-FILE")
   default public character getDroppedFile(NumberType index)
   {
      if (index != null && !index.isUnknown())
      {
         return getDroppedFile(index.intValue());
      }
      else
      {
         return new character();
      }
   }
   
   // methods helping to access to the class instance
}