/*
** Module   : SelectableText.java
** Abstract : Define SELECTION-TEXT and related methods/attributes available via a handle 
**            instance for the EDITOR/COMBO-BOX/FILL-IN widgets.
**
** Copyright (c) 2014-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 IAS 20150604 Created initial version.
** 002 OM  20170505 Added declaration for EDIT-CLEAR.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui;

import com.goldencode.p2j.util.*;

public interface SelectableText
{
   /**
    * Get SELECTION-TEXT attribute.
    *
    * @return  SELECTION-TEXT value   
    */
   @LegacyAttribute(name = "SELECTION-TEXT")
   public character getSelectionText();

   /**
    * Get SELECTION-END attribute.
    *
    * @return  SELECTION-END value   
    */
   @LegacyAttribute(name = "SELECTION-END")
   public integer getSelectionEnd();

   /**
    * Get SELECTION-START attribute.
    *
    * @return  SELECTION-START value   
    */
   @LegacyAttribute(name = "SELECTION-START")
   public integer getSelectionStart();

   /**
    * Selects text in a widget between start and end points.
    * 
    * @param    start
    *           Starting offset.
    * @param    end
    *           Ending offset.
    *
    * @return   <code>true</code> if the operation succeeded.
    */
   @LegacyMethod(name = "SET-SELECTION")
   public logical setSelection(int64 start, int64 end);
   
   /**
    * Selects text in a widget between start and end points.
    * 
    * @param    start
    *           Starting offset.
    * @param    end
    *           Ending offset.
    *
    * @return   <code>true</code> if the operation succeeded.
    */
   @LegacyMethod(name = "SET-SELECTION")
   public logical setSelection(int64 start, int end);
   
   /**
    * Selects text in a widget between start and end points.
    * 
    * @param    start
    *           Starting offset.
    * @param    end
    *           Ending offset.
    *
    * @return   <code>true</code> if the operation succeeded.
    */
   @LegacyMethod(name = "SET-SELECTION")
   public logical setSelection(int start, int64 end);
   
   /**
    * Selects text in a widget between start and end points.
    * 
    * @param    start
    *           Starting offset.
    * @param    end
    *           Ending offset.
    *
    * @return   <code>true</code> if the operation succeeded.
    */
   @LegacyMethod(name = "SET-SELECTION")
   public logical setSelection(int start, int end);
   
   /**
    * Checks is text selection is action.
    * 
    * @return  <code>true</code> if text selection is active.
    */
   @LegacyAttribute(name = "TEXT-SELECTED")
   public logical isTextSelected();

   /**
    * Clears text selection
    */
   @LegacyMethod(name = "CLEAR-SELECTION")
   public logical clearSelection();
   
   /**
    * Removes selected text.
    */
   @LegacyMethod(name = "EDIT-CLEAR")
   public logical removeSelection();
}
