/*
** Module   : FixedSizeContainer.java
** Abstract : base class for browse, editor, fillin and labels
**
** Copyright (c) 2005-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ----------------------------Description-----------------------------
** 001 SVG 20050609   @21446 Initial version.
** 002 SIY 20050830   @22509 Renamed.
** 003 SIY 20050907   @22644 Fixed use of @inheritDoc tag in javadoc
** 004 SIY 20060128   @24120 Added meaningful toString() method.
** 005 SIY 20060209   @24458 Minor cleanups, provided relative coordinates instead of absolute in
**                           toString() method.
** 006 SIY 20100619          Switching to own implementation of character UI.
** 007 SIY 20100804          Changes in imports, some refactorings.
** 008 SIY 20100915          Renamed from P2JComponent, cleaned up redundant methods and
**                           declarations.
** 009 SIY 20111101          Moved from CHUI package. Minor cleanups/refactorings in preparation
**                           for GUI client.
** 010 GES 20140604          Documentation cleanup.
** 011 CA  20140805          Added generics to force the abstract widget classes to not use
**                           implementation-specific code: the abstract widgets will have access
**                           only to the base APIs provided by the OutputManager.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui.client;

import com.goldencode.p2j.ui.client.widget.*;

/**
 * Common base class for browse, editor, fillin and labels.
 */
public class FixedSizeContainer<O extends OutputManager<?>>
extends AbstractContainer<O>
{
   /**
    * Returns the same as <code>getSize()</code>.
    * 
    * @return  Minimum size.
    */
   @Override
   public Dimension minimumSize()
   {
      return dimension();
   }

   /**
    * Request focus.
    */
   @Override
   public void requestFocus()
   {
      super.requestFocus();
      drawCaret();
   }
}