/*
** Module   : SelectionListBody.java
** Abstract : selection-list implementation
**
** Copyright (c) 2005-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ----------------------------Description------------------------------
** 001 EVL 20050203   @19739 Creation based on CHARVA JList class.
**                           Necessary modification to get compiled
** 002 EVL 20050429   @20944 Adding label creation for the widget
** 003 EVL 20050503   @21073 Changing constructor signature to reflect
**                           the possible options for creation widget
** 004 EVL 20050503   @21074 Adding the implementation of the item
**                           manipulation methods
** 005 EVL 20050522   @21258 Change the name of the VALIDATE() method to
**                           validateThis() from runValidation() to unify
**                           the approach with the other containers
** 006 EVL 20050602   @21393 Changing the inner variable names to sync
**                           with the rest of the project
** 007 EVL 20050602   @21394 Adding the Progress specific methods to work
**                           with Progress SelectionList attributes
** 008 EVL 20050604   @21407 Fixes for javadoc comments syntax issues
** 009 EVL 20050607   @21435 Syntax for comments checking
** 010 EVL 20050607   @21438 Removing the visible member. It is completely
**                           duplicated in CHARVA's Component superclass
**                           so we do not need to double this work. Also
**                           the methods to work with the HIDDEN attribute
**                           are modified to reflect this change
** 011 EVL 20050723   @21805 Properly handling of the change the
**                           LIST_ITEMS attribute. Default delimiter is 
**                           comma.
** 012 EVL 20050822   @22199 Using the P2JFrame/Container interfaces to
**                           detect the class.
** 013 EVL 20050826   @22273 Setting up instance to a wrapped ScrollPane.
** 014 SIY 20050831   @22513 Refactoring (see @@22394-22409).
** 015 SIY 20050907   @22648 Fixed use of @inheritDoc tag in javadoc.
** 016 SIY 20050911   @22671 Updated initialisation.
** 017 EVL 20050912   @22677 Adding Validateable interface to this widget.
** 018 EVL 20050913   @22697 Adding help string handling when gaining 
**                           focus.
** 019 EVL 20050916   @22739 Returned value class is changed to be the
**                           BaseDataType.
** 020 EVL 20050920   @22761 Adding the implementation of the new method
**                           isModified() for the Validateable interface.
** 021 EVL 20050921   @22767 Adding method to be able to undo the widget's
**                           value.
** 022 SIY 20050920   @22849 Added support for SENSITIVE and SELECTED
**                           attributes. Switched to ColorTable colors.
** 023 SIY 20051129   @23563 Moved item list member to config.
** 024 GES 20060114   @24210 Implemented DataContainer interface and the
**                           core methods.
** 025 SIY 20060131   @24211 Use ThinClient special method instead of
**                           direct calls to Window.message().
** 026 SIY 20060209   @24455 Class renamed to SelectionListBody, many 
**                           other changes.
** 027 SIY 20060210   @24520 Implemented multiple selection case of 
**                           setValue().
** 028 SIY 20060222   @24643 Added necessary methods to support changes in
**                           DataContainer interface.
** 029 NVS 20060227   @24871 Key actions KA_* are used instead of virtual
**                           keys VK_* for all editing operations.
** 030 NVS 20060313   @25071 Fixed drawing of uninitialized selection list
** 031 SIY 20060515   @26158 Encapsulated access to help field.
** 032 GES 20060803   @28415 Major cleanup and refactoring.
** 033 NVS 20061204   @31560 Switched to using a private widgetFocused
**                           flag instead of config.selected
** 034 NVS 20061220   @31724 getValue() method is rewritten to return the
**                           value of the selected item, not the label.
** 035 NVS 20061221   @31732 Fixed drawing of list items (were wider by 2
**                           columns). Added removal of the highlight on
**                           GO event.
** 036 NVS 20070426   @33256 Enabled state is checked instead of SENSITIVE
**                           attribute.
** 037 SIY 20080320   @37605 Fixed handling of the SCREEN-VALUE and data
**                           value.
** 038 SIY 20080411   @37939 Fixed drawing. Cleanups and refactoring.
** 039 SIY 20080414   @37971 Fixed minor regression caused by #38. 
**                           Refactored and cleaned up draw() method.
** 040 NVS 20080625   @38927 Fixed drawing when disabled.
** 041 SIY 20100619          Switching to own implementation of character UI.
** 042 SIY 20100804          Changes in imports, some refactorings.
** 043 SIY 20101122          Switching from KeyEvent to KeyInput.
** 044 GES 20110420          Fixed querying the value in multiple selection
**                           case.
** 045 SIY 20110112          Changes related to new color handling scheme.
** 046 SIY 20111101          Updated imports, minor cleanups/refactorings.
** 047 SIY 20111202          Refactored and moved from CHUI package.
** 048 SVL 20120821          Changed c'tor signature in order to match the one
**                           used by ChuiWidgetFactory. Added widget()
**                           function.
** 049 CA  20140805          Added generics to force the abstract widget classes to not use
**                           implementation-specific code: the abstract widgets will have access
**                           only to the base APIs provided by the OutputManager.
** 050 CA  20140926          Added shared widget configuration support, to allow GUI/ChUI
**                           concrete implementation for the same widget ID.  Refactored the
**                           widget configuration classes: all fields were made public; these
**                           classes need to be as dumb as possible, all logic related to
**                           setting/getting a certain field should be at the widget or at the
**                           caller. Refs #2254
** 051 CA  20150212          Allow events to be captured and processed by the widgets without
**                           server-side or main event-loop notification.  This fixes problems in
**                           the EDITING and WAIT-FOR event loops, for i.e. COMBO-BOX and EDITOR
**                           widgets.
** 052 IAS 20150214          Fix behavior to be compatible with 4GL
** 053 IAS 20150305          Hide/restore cursor on focus gain/lost
** 054 HC  20150323          Removed business logic from config classes.
** 055 EVL 20150821          Seperating UI specific code. Adding SCROLL-TO-ITEM() method support.
** 056 HC  20150831          Scrolling support overhaul.
** 057 HC  20151117          Fixed broken ChUI SELECTION-LIST scrolling.
** 058 OM  20170320          Signature of ThinClient.setHelp() change.
** 059 CA  20171127          Fixed a NPE in refreshItems().
** 060 SBI 20180706          Fixed NPE in setSingleValue if config.items is null.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui.client;

import java.util.*;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.chui.ThinClient;
import com.goldencode.p2j.ui.client.event.*;
import com.goldencode.p2j.ui.client.event.listener.*;
import com.goldencode.p2j.ui.client.format.*;
import com.goldencode.p2j.ui.client.model.*;
import com.goldencode.p2j.ui.client.widget.*;
import com.goldencode.p2j.util.*;

/**
 * A component that allows the user to select one or more objects from a list.
 * <p>
 * This class does not provide scrolling capability. Instances of this class
 * are normally inserted into a <code>ScrollPane</code> to provide scrolling.
 */
public abstract class SelectionListBody<O extends OutputManager<?>>
extends DefaultList<String, O>
implements WidgetWithConfig<SelectionListConfig>,
           FocusListener,
           SelectionListener,
           DataContainer
{
   /** Component attribute container. */
   protected SelectionListConfig config = null;
   
   /** Widget in focus flag */
   protected boolean widgetFocused = false;

   /** Default initial value. */
   private BaseDataType initValue = new character();

   /** Track user interaction with widget. */
   private boolean selected = false;
   
   /** Current items' list */ 
   protected ControlSetItem[] items;
   
   /** Cursor visibility before  this wdget gained focus*/ 
   protected boolean cursorVisible;
   
   /** The curent top row in the list to draw. */
   protected int rowTop;
   

   /**
    * Special constructor used to restore component using given configuration.
    * 
    * @param   id
    *          The widget's ID.
    * @param   cfg
    *          Configuration data.
    */
   public SelectionListBody(WidgetId id, SelectionListConfig cfg)
   {
      super(new ListModel<String>());
      config = cfg;

      addFocusListener(this);
      addSelectionListener(this);
   }
   
   /**
    * Replacement API for the logic which should exist in the widget's c'tor, but the logic 
    * can't be executed as it depends on the widget being registered into the official registry.
    * <p>
    * This API is called after the widget is created and added to the registry.
    * 
    * @param    id
    *           The widget's ID.
    * @param    cfg
    *           The config used to initialize this widget.
    */
   @Override
   public void initialize(WidgetId id, SelectionListConfig cfg)
   {
      // no-op
   }

   /**
    * Requesting the focus to the <code>SelectionListBody</code>.
    */
   @Override
   public void requestFocus()
   {
      if (!isEnabled())
         return;
      super.requestFocus();
   }
   
   /**
    * Draws this component. Overrides draw() in JList.
    */
   @Override
   public abstract void draw();

   /**
    * Get item text at specified index.
    * 
    * @param   i
    *          The item index.
    *          
    * @return  The item text.
    */
   protected String getItemText(int i)
   {
      return model().get(i).toString();
   }
   
   /**
    * Key events handler for the SelectionListBody widget. The special processing
    * for SelectionListBody is the SPACE key is the same as ENTER key. Also by
    * moving the list cursor the user changes selected item for single
    * selection mode list.
    * 
    * @param   keyEvent
    *          The key event to be handled.
    */
   @Override
   public void processKeyEvent(KeyInput keyEvent)
   {
      // First call all KeyListener objects that may have been registered
      // for this component.
      int key = keyEvent.actionCode();
      
      // watch GO event to remove the highlight
      if (key == Keyboard.KA_GO)
      {
         widgetFocused = false;
         draw();
      }
      else if (key == Keyboard.SE_ENTRY)
      {
         // force setting of SCREEN-VALUE on entry
         ConfigHelper.setModified(config, true);
      }

      if (selectionMode() == ListSelectionModel.SINGLE_SELECTION
          && keyEvent.keyCode() == Key.VK_ENTER)
         return;

      super.processKeyEvent(keyEvent);
   }
   
   /**
    * Process key event.
    * 
    * @param key
    *          key to process
    */
   @Override
   public void processKey(int key)
   {
      switch (key)
      {
         case Key.VK_CURSOR_DOWN :
            changeRow(1);
            break;

         case Key.VK_CURSOR_UP :
            changeRow(-1);
            break;

         case Key.VK_PAGE_DOWN :
         case Key.VK_END :
         case Key.VK_HOME :
            break;

         case Key.VK_PAGE_UP :
            if (currentRow() == 0)
            {
               changeRow(0);
            }
            break;

         case 32:
            selectItem();
            EventManager.postEvent(new KeyInput(this,
                  EventType.KEY_PRESSED,
                  Keyboard.SE_VALUE_CHANGED,
                  true));
            break;

         case Key.VK_ENTER :
            selectItem();
            EventManager.postEvent(new KeyInput(this,
                  EventType.KEY_PRESSED,
                  Keyboard.KA_RETURN,
                  true));
            break;
      }
   }

   /**
    * Returns the character value of the entry with specified index from the
    * SelectionListBody.
    * 
    * @param   index
    *          The index of the entry which value should be returned.
    * 
    * @return  The string value of the entry with given index.
    */
   public String entry(int index)
   {
      return model().get(index);
   }
   
   /**
    * Checks the selection state of the entry with specified index in
    * SelectionListBody.
    * 
    * @param   index
    *          The index of the entry to be checked inside the list.
    * 
    * @return  <code>true</code> if operation was successful and
    *          <code>false</code> otherwise.
    */
   public boolean isSelected(int index)
   {
      return selected(index);
   }

   /**
    * Scrolls the list to have the specified item at the top of the list.
    * 
    * @param   item
    *          The string entry which must be visible after scrolling the
    *          list.
    * 
    * @return  <code>true</code> if operation was successful and
    *          <code>false</code> otherwise.
    */
   public boolean scrollToItem(String item)
   {
      scrollToItem(model().indexOf(item));
      return true;
   }
   
   /**
    * Scrolls the list to have the specified item at the top of the list.
    * 
    * @param   index
    *          The index of the entry which must be visible after scrolling
    *          the list.
    * 
    * @return  <code>true</code> if operation was successful and
    *          <code>false</code> otherwise.
    */
   public boolean scrollToItem(int index)
   {
      ensureRowVisible(index);
      return true;
   }
   
   /**
    * Getting the currently selected item.
    * Since the list is made of {label, value} pairs, this method returns
    * the value.
    *
    * @return  The BaseDataType object representing the current value of
    *          the selection-list.
    */
   @Override
   public BaseDataType getValue()
   {
      if (selected )
         return getScreenValue();
      
      return initValue;
   }

   /**
    * Sets the BaseDataType instance representing the current value.
    * 
    * @param   value
    *          The value representing the current widget value.
    */
   @Override
   public void setValue(BaseDataType value)
   {  
      selected  = false;
      initValue = new character();
      
      boolean doRepaint = false;
      
      if (value.isUnknown())
         return;
      
      clear();
      
      if (value instanceof character)
      {
         String itemList = ((character) value).getValue();
         
         char[] delim = new char[1];
         delim[0] = config.delimiter;
         String d = new String(delim);
         
         String[] elem = itemList.split(d);
         
         for (String element : elem)
         {
            BaseDataType val = 
               DisplayFormat.instanceOfType(config.dataType, element);
            
            if (setSingleValue(val))
               doRepaint = true;
         }
      }
      else
         doRepaint = setSingleValue(value);

      if (doRepaint)
      {
         repaint();
         initValue = value; 
      }
      else
         if (CompareOps.isEqual(value, "").booleanValue())
            initValue = value;
   }

   /**
    * Checks if we need to post VALUE-CHANGED event.  UI specific part.
    *
    * @return  <code>TRUE</code> if single selection mode and changed value,
    *          <code>FALSE</code> otherwise.
    */
   protected boolean needValueChanged()
   {
      return selectionMode() == ListSelectionModel.SINGLE_SELECTION && !selected(currentRow());
   }
   
   /**
    * Process change selected row.
    * 
    * @param delta
    *          offset to next row
    */
   private void changeRow(int delta) 
   {
      if (delta != 0)
      {
         shiftCurrentRow(delta, true);
      }
      
      // some UI specific work
      if (needValueChanged())
      {
         selectItem();
         EventManager.postEvent(new KeyInput(this,
               EventType.KEY_PRESSED,
               Keyboard.SE_VALUE_CHANGED,
               true));
      }
   }
   
   /**
    * Set single value in the list as selected.
    * 
    * @param   value
    *          Value to set.
    * @return  <code>true</code> if value is set and repaint is required.
    */
   @SuppressWarnings("unchecked")
   private boolean setSingleValue(BaseDataType value)
   {
      ControlSetItem[] items = config.items;
      
      if (items == null)
      {
         return false;
      }
      
      boolean doRepaint = false;
      
      for (int i = 0; i < items.length; i++)
      {
         if (items[i].getValue().compareTo(value) == 0)
         {
            select(i,i);
            
            if (first() >= 0)
               ensureRowVisible(first());
            
            doRepaint = true;
         }
      }
      
      return doRepaint;
   }

   /**
    * The implementation of the ListSelectionListener interface. Marks the
    * widget as modified.
    */
   @Override
   public void changed()
   {
      ConfigHelper.setModified(config, true);
      selected = true;
   }
   
   /**
    * Getting the modification state of the widget.
    * 
    * @return  <code>true</code> if the widget value was modified,
    *          <code>false</code> otherwise.
    */
   @Override
   public boolean isModified()
   {
      return config.modified;
   }
   
   /**
    * This is called when the selection-list takes the input focus. When the help string is
    * defined for the selection-list it should be displayed at the window status line.
    *
    * @param   focusEvent
    *          The focus event to be processed.
    */
   @Override
   public void onFocusGained(FocusEvent focusEvent)
   {
      if (focusEvent == null)
      {
         return;
      }
      
      if (!widgetFocused)
      {
         widgetFocused = true;
         cursorVisible = screen().setCursorStatus(false);
      }
      // Draw the help string for focused button
      
      ThinClient.setHelp(config.help, this);
      
      repaint();
      requestSync();
   }

   /**
    * This is called when the selection-list looses the input focus.
    *
    * @param   focusEvent
    *          The focus event to be processed.
    */
   @Override
   public void onFocusLost(FocusEvent focusEvent)
   {
      if (focusEvent == null)
         return;
      
      widgetFocused = false;
      screen().setCursorStatus(cursorVisible);

      repaint();
      requestSync();
   }

   /**
    * Update list of items from configuration.
    */
   public void refreshItems()
   {
      int[] selected = selection();
      Set<Integer> selectedIdx = new HashSet<>();

      for (int i = 0; i < selected.length; i++)
      {
         int is = selected[i];
         if (0 <= is && is < items.length)
         {
            selectedIdx.add(Integer.valueOf(items[is].getItemId()));
         }
      }

      ControlSetItem[] citems = config.items;
      if (citems == null)
      {
         citems = new ControlSetItem[0];
      }

      model().clear();
      clear();

      for (int i = 0; i < citems.length; i++)
      {
         ControlSetItem element = citems[i];
         model().add(element.getLabel().getValue());
         if (selectedIdx.contains(Integer.valueOf(element.getItemId())))
         {
            select(i, i);  
         }
      }
      
      items = citems;
   }

   /**
    * Get state of the HORIZONTAL-SCROLL-BAR attribute.
    * 
    * @return  <code>true</code> if scroll bar is enabled.
    */
   public boolean isHorizontalScrollBar()
   {
      return config.scrollbarHorizontal;
   }
 
   /**
    * Get state of the VERTICAL-SCROLL-BAR attribute.
    * 
    * @return  <code>true</code> if scroll bar is enabled.
    */
   public boolean isVerticalScrollBar()
   {
      return config.scrollbarVertical;
   }

   /**
    * Get the text representation of the currently selected value. This will be
    * rendered as the labels that are displayed (which are the same as the
    * value in LIST-ITEMS mode, but are different from the value in 
    * LIST-ITEM-PAIRS mode).
    * <p>
    * This will be a single text value if this is a single-select list and
    * will be a delimited list if this is multiple-select.
    * 
    * @return   The screen representation of the value or empty string if there
    *           is no selection.
    */
   @Override
   public String getText()
   {
      String result = null;

      for (int i = 0; i < model().size(); i++)
      {
         if (isSelected(i))
         {
            if (result == null)
               result = entry(i);
            else
               result += Character.toString(config.delimiter) + entry(i);
         }
      }

      return (result == null) ? "" : result;
   }
   
   /**
    * Get widget screen value (suitable for assigning back to any associated
    * variable or field). This will be rendered as the values (which are the
    * same as the labels in LIST-ITEMS mode, but are different from the labels 
    * in LIST-ITEM-PAIRS mode).
    * <p>
    * This will be a single text value if this is a single-select list and
    * will be a delimited list if this is multiple-select.
    * 
    * @return  Current widget screen value.
    */
   public BaseDataType getScreenValue()
   {
      ControlSetItem[] items = config.items;
      
      int[]  sel   = selection();
      String delim = Character.toString(config.delimiter);
      
      StringBuilder sb = new StringBuilder();
      
      for (int i = 0; i < sel.length; i++)
      {
         if (sel[i] >= 0 && sel[i] < items.length)
         {
            character val = (character) items[sel[i]].getValue();
            
            if (i > 0)
               sb.append(delim);
            
            sb.append(val.toStringMessage());
         }
      }
      
      return sb.length() == 0 ? new character() : new character(sb.toString());
   }

   /**
    * Provide access to the attributes stored in the widget.
    *
    * @return  Reference to attribute container.
    */
   @SuppressWarnings("unchecked")
   @Override
   public SelectionListConfig config()
   {
      return config;
   }

   /**
    * The method is called after the configuration associated with 
    * the implementor changes. The parameter points to a reference
    * representing the original configuration state. This reference
    * can be used to detect what configuration fields changed and so
    * optimize any processing related to the configuration change.
    * 
    * @param    beforeUpdate
    */
   @Override
   public void afterConfigUpdate(SelectionListConfig beforeUpdate)
   {
      super.afterConfigUpdateBase(beforeUpdate);
   }

   /**
    * Check if the current widget can manage non-global system actions on its own, and not via the
    * enclosing event loop.
    * 
    * @return   <code>true</code> as the widget has its own event management.
    */
   @Override
   public boolean manageSystemActions()
   {
      return true;
   }
   
   /**
    * Get widget location relative to container.
    * 
    * @return  relative widget location.
    */
   @Override
   public Point location()
   {
      // List body shares its config with selection list.
      // Thus prevent the super method to be called which would also
      // affect the selection-list-set's position.
      return new Point(location);
   }

   /**
    * Get widget physical location relative to container, expressed in units native to UI type.
    * 
    * @return  relative widget location.
    */
   @Override
   public NativePoint physicalLocation()
   {
      // List body shares its config with selection list.
      // Thus prevent the super method to be called which would also
      // affect the selection-list-set's position.
      return new NativePoint(physicalLocation);
   }

   /**
    * Set widget relative location.
    * 
    * The x value corresponds to COLUMN while
    * y value corresponds to ROW 4GL attribute.
    * Note however that the values are expected to be
    * 0-based.
    * 
    * @param   x
    *          Widget X.
    * @param   y
    *          Widget Y.
    */
   @Override
   public void setLocation(double x, double y)
   {
      // List body shares its config with selection list.
      // Thus prevent the super method to be called which would also
      // affect the selection-list-set's position.
      
      if (isVisible())
         repaint(); 
            
      location.x = Coordinate.scale(x);
      location.y = Coordinate.scale(y);
      
      if (isVisible())
         repaint(); 
   }

   /**
    * Set widget relative physical location.
    * 
    * @param   x
    *          Widget physical X.
    * @param   y
    *          Widget physical Y.
    */
   @Override
   public void setPhysicalLocation(int x, int y)
   {
      // List body shares its config with selection list.
      // Thus prevent the super method to be called which would also
      // affect the selection-list-set's position.
      physicalLocation.x = x;
      physicalLocation.y = y;
   }
   
   /**
    * Directs the widget to scroll to the specified X and Y positions.
    * It is the widget's responsibility to update its visuals accordingly.
    * Note that the position values are in the same units as the returned 
    * dimensions from {@link #getScrollDimension()}, {@link #getVisibleDimension()}
    * and {@link #getScrollStep()}. 
    * 
    * @param    newXPosition
    *           New X scroll position.
    * @param    newYPosition
    *           New Y scroll position.
    */
   @Override
   public void scroll(Integer newXPosition, Integer newYPosition)
   {
      if (scrollWorker(newXPosition, newYPosition))
      {
         // and repaint the list
         ThinClient.getInstance().eventDrawingBracket(this, false, true, new Runnable()
         {
            @Override
            public void run()
            {
               repaint();
            }
         });
      }

   }
   
   /**
    * Calculate the scroll Y position based on the current state. 
    * 
    * @return   the new scroll Y position
    */
   @Override
   protected int calculateScrollYPosition()
   {
      int curr = currentRow();
      
      if (curr < 0)
      {
         return 0;
      }
      
      // check whether currently selected row is higher than the first visible
      if (curr < rowTop)
      {
         return curr;
      }
      
      // check whether currently selected row is lower than the last visible
      int distance = curr - rowTop;
      int visible = visibleRows() - 1;
      
      if (distance > visible)
      {
         return rowTop + (distance - visible);
      }
      
      // the selected row is in the visible range, the Y position doesn't change
      return rowTop;
   }
   
   /**
    * Directs the widget to scroll to the specified X and Y positions.
    * It is the widget's responsibility to update its visuals accordingly.
    * Note that the position values are in the same units as the returned 
    * dimensions from {@link #getScrollDimension()}, {@link #getVisibleDimension()}
    * and {@link #getScrollStep()}. 
    * 
    * @param    newXPosition
    *           New X scroll position.
    * @param    newYPosition
    *           New Y scroll position.
    *           
    * @return   <code>true</code> to indicate that the scroll operation actually 
    *           invalidated this list. 
    */
   protected boolean scrollWorker(Integer newXPosition, Integer newYPosition)
   {
      boolean dirty = false;
      
      if (newYPosition != null && newYPosition != rowTop)
      {
         rowTop = newYPosition;
         dirty = true;
      }
      
      return dirty;
   }
}