/*
 ** Module   : SysColor.java
 ** Abstract : define system color indexes.
 **
 ** Copyright (c) 2014-2017, Golden Code Development Corporation.
 **
 ** -#- -I- --Date-- ---------------------------------Description----------------------------------
 ** 001 MAG 20140827 Created initial version
 ** 002 EVL 20141128 Adding tooltip background color.
 ** 003 CA  20141221 Added constants for FILL-IN; needs to be reviewed if other sys colors can't
 **                  be used instead.
 ** 004 GES 20150917 Reset all default colors from captured values on a "Windows Classic" system.
 ** 005 OM  20170321 Moved the default system color to each Theme.
 */
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui.client;

import java.util.*;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.client.gui.theme.*;

/**
 * This class defines system colors constants.
 */
public class SysColor
{
   /** System colors indexes */
   public static final int COLOR_SCROLLBAR = 0;
   public static final int COLOR_BACKGROUND = 1;
   public static final int COLOR_ACTIVECAPTION = 2;
   public static final int COLOR_INACTIVECAPTION = 3;
   public static final int COLOR_MENU = 4;
   public static final int COLOR_WINDOW = 5;
   public static final int COLOR_WINDOWFRAME = 6;
   public static final int COLOR_MENUTEXT = 7;
   public static final int COLOR_WINDOWTEXT = 8;
   public static final int COLOR_CAPTIONTEXT = 9;
   public static final int COLOR_ACTIVEBORDER = 10;
   public static final int COLOR_INACTIVEBORDER = 11;
   public static final int COLOR_APPWORKSPACE = 12;
   public static final int COLOR_HIGHLIGHT = 13;
   public static final int COLOR_HIGHLIGHTTEXT = 14;
   public static final int COLOR_3DFACE = 15;
   public static final int COLOR_3DSHADOW = 16;
   public static final int COLOR_GRAYTEXT = 17;
   public static final int COLOR_BTNTEXT = 18;
   public static final int COLOR_INACTIVECAPTIONTEXT = 19;
   public static final int COLOR_BTNHIGHLIGHT = 20;
   public static final int COLOR_3DDKSHADOW = 21;
   public static final int COLOR_3DLIGHT = 22;
   public static final int COLOR_INFOTEXT = 23;
   public static final int COLOR_INFOBK = 24;
   public static final int COLOR_BUTTONALTERNATEFACE = 25;
   public static final int COLOR_HOTLIGHT = 26;
   public static final int COLOR_GRADIENTACTIVECAPTION = 27;
   public static final int COLOR_GRADIENTINACTIVECAPTION = 28;
   public static final int COLOR_MENUHILIGHT = 29;
   public static final int COLOR_MENUBAR = 30;
   public static final int COLOR_TOOLTIPBK = 31;
   public static final int COLOR_FILLIN_BG = 32;
   public static final int COLOR_FILLIN_FG = 33;
   public static final int COLOR_CARET = 34;

   /** System color P4GL names */
   public static final Map<String, Integer> systemColorNames;

   static
   {
      systemColorNames = new HashMap<>();
      systemColorNames.put("COLOR-SCROLLBAR", COLOR_SCROLLBAR);
      systemColorNames.put("COLOR-BACKGROUND", COLOR_BACKGROUND);
      systemColorNames.put("COLOR-ACTIVECAPTION", COLOR_ACTIVECAPTION);
      systemColorNames.put("COLOR-INACTIVECAPTION", COLOR_INACTIVECAPTION);
      systemColorNames.put("COLOR-MENU", COLOR_MENU);
      systemColorNames.put("COLOR-WINDOW", COLOR_WINDOW);
      systemColorNames.put("COLOR-WINDOWFRAME", COLOR_WINDOWFRAME);
      systemColorNames.put("COLOR-MENUTEXT", COLOR_MENUTEXT);
      systemColorNames.put("COLOR-WINDOWTEXT", COLOR_WINDOWTEXT);
      systemColorNames.put("COLOR-CAPTIONTEXT", COLOR_CAPTIONTEXT);
      systemColorNames.put("COLOR-ACTIVEBORDER", COLOR_ACTIVEBORDER);
      systemColorNames.put("COLOR-INACTIVEBORDER", COLOR_INACTIVEBORDER);
      systemColorNames.put("COLOR-APPWORKSPACE", COLOR_APPWORKSPACE);
      systemColorNames.put("COLOR-HIGHLIGHT", COLOR_HIGHLIGHT);
      systemColorNames.put("COLOR-HIGHLIGHTTEXT", COLOR_HIGHLIGHTTEXT);
      systemColorNames.put("COLOR-3DFACE", COLOR_3DFACE);
      systemColorNames.put("COLOR-3DSHADOW",COLOR_3DSHADOW);
      systemColorNames.put("COLOR-GRAYTEXT", COLOR_GRAYTEXT);
      systemColorNames.put("COLOR-BTNTEXT", COLOR_BTNTEXT);
      systemColorNames.put("COLOR-INACTIVECAPTIONTEXT", COLOR_INACTIVECAPTIONTEXT);
      systemColorNames.put("COLOR-BTNHIGHLIGHT", COLOR_BTNHIGHLIGHT);
      systemColorNames.put("COLOR-3DDKSHADOW", COLOR_3DDKSHADOW);
      systemColorNames.put("COLOR-3DLIGHT", COLOR_3DLIGHT);
      systemColorNames.put("COLOR-INFOTEXT", COLOR_INFOTEXT);
      systemColorNames.put("COLOR-INFOBK", COLOR_INFOBK);
      systemColorNames.put("COLOR-BUTTONALTERNATEFACE", COLOR_BUTTONALTERNATEFACE);
      systemColorNames.put("COLOR-HOTLIGHT", COLOR_HOTLIGHT);
      systemColorNames.put("COLOR-GRADIENTACTIVECAPTION", COLOR_GRADIENTACTIVECAPTION);
      systemColorNames.put("COLOR-GRADIENTINACTIVECAPTION", COLOR_GRADIENTINACTIVECAPTION);
      systemColorNames.put("COLOR-MENUHILIGHT", COLOR_MENUHILIGHT);
      systemColorNames.put("COLOR-MENUBAR", COLOR_MENUBAR);
      systemColorNames.put("COLOR-TOOLTIPBK", COLOR_TOOLTIPBK);
      // TODO: get the correct names for these colors
      systemColorNames.put("COLOR-FILLIN-BG", COLOR_FILLIN_BG); // confirm name
      systemColorNames.put("COLOR-FILLIN-FG", COLOR_FILLIN_FG); // confirm name
      systemColorNames.put("COLOR-CARET", COLOR_CARET);         // confirm name
   }
   
   /**
    * Private constructor.
    */
   private SysColor()
   {
   }
}