/*
** Module   : VT100RendererImpl.java
** Abstract : VT100 renderer implementation
**
** Copyright (c) 2014-2017, Golden Code Development Corporation.

**
** -#- -I- --Date-- --------------------------------Description---------------------------------
** 001 MAG 20140318 First version.
** 002 MAG 20140507 Fix cursor bug for web clients.
** 003 GES 20150414 Reworked to remove Swing-specific implementation details.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui.client.chui.driver;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.client.*;
import com.goldencode.terminal.*;

/**
 * Implements most behavior for the VT-100 renderer in a UI independent manner.
 */
public abstract class VT100RendererImpl 
extends VT100AbstractRenderer<Cell>
{      
   /**
    * Constructor.
    * 
    * @param    data
    *           Video buffer.
    * @param    rows
    *           Number of rows on the screen.
    * @param    cols
    *           Number of columns on the screen.
    */
   public VT100RendererImpl(Cell[][]data, int rows, int cols)   
   {
      super(data, rows, cols, 0, rows - 1);
   }
   
   /**
    * Get attribute mask.
    * 
    * @param    id
    *           Attribute id.
    * <p>       
    *           0 - NORMAL 
    *           1 - BLINK 
    *           2 - BOLD 
    *           3 - UNDERLINE 
    *           4 - REVERSE
    *           
    * @return   attribute mask.
    */
   @Override
   public int attributeMask(int id)
   {
      switch (id)
      {
         case 0:
            return ColorAttribute.NORMAL.getValue();
         case 1:
            return ColorAttribute.BLINK.getValue();
         case 2:
            return ColorAttribute.BOLD.getValue();
         case 3:
            return ColorAttribute.UNDERLINE.getValue();
         case 4:
            return ColorAttribute.REVERSE.getValue();
         default:
            return ColorAttribute.NORMAL.getValue();
      }      
   }   

   /**
    * Create an empty cell.
    * 
    * @return   An empty video memory cell.
    */
   @Override
   protected Cell emptyCell()
   {
      return Cell.EMPTY_CELL;
   }

   /**
    * Build a video memory cell.
    * 
    * @param    text
    *           Cell character.
    * @param    attribute
    *           Color attributes
    * @param    color
    *           Color pair FG/BG
    *           
    * @return   A new video memory cell.
    */   
   @Override
   protected Cell makeCell(char text, int attribute, int color)
   {
      // box characters are not properly rendered in SWING having BOLD attribute 
      if ((text >= '\u2500') && (text <= '\u25ff'))
      {
         attribute &= 0xFD;
      }
      
      // return a new Cell instance
      return new Cell(text, new Color(attribute, color));
   }
}