/*
** Module   : ImageXfer.java
** Abstract : helper to pass an image to the clipboard
**
** Copyright (c) 2011-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- -----------------------Description------------------------
** 001 GES 20110921 Created initial version.
** 002 GES 20150506 Moved to a driver package.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui.client.driver.swing;

import java.awt.*;
import java.awt.datatransfer.*;
import java.awt.image.*;
import java.io.*;

/**
 * Provides a convenience class to make it easy to pass an image as a
 * <code>Transferrable</code>.  This is needed to copy data to the
 * clipboard or other data transfer mechanisms such as drag and drop.
 */
public class ImageXfer
implements Transferable
{
   /** The image being transferred. */
   private Image image = null;

   /**
    * Construct an instance with the given image as the data to transfer.
    *
    * @param    image
    *           The image to transfer.
    */
   public ImageXfer(Image image)
   {
      this.image = image;
   }

   /**
    * Provides a list of supported data flavors.
    *
    * @return   An array of data flavors containing a single element that
    *           is <code>DataFlavor.imageFlavor</code>.
    */
   public DataFlavor[] getTransferDataFlavors()
   {
      return new DataFlavor[] { DataFlavor.imageFlavor };
   }

   /**
    * Reports if the given data flavor is supported.
    *
    * @param    flavor
    *           The flavor to check.
    *
    * @return   <code>true</code> if the given flavor equals
    *           <code>DataFlavor.imageFlavor</code>.
    */
   public boolean isDataFlavorSupported(DataFlavor flavor)
   {
      return DataFlavor.imageFlavor.equals(flavor);
   }

   /**
    * Obtains the contained data if it matches a supported data flavor.
    *
    * @param    flavor
    *           The flavor that is needed.
    *
    * @return   The image data if the given flavor equals
    *           <code>DataFlavor.imageFlavor</code>.  Otherwise an
    *           <code>UnsupportedFlavorException</code> is thrown.
    */
   public Object getTransferData(DataFlavor flavor)
   throws UnsupportedFlavorException
   {
      if (!DataFlavor.imageFlavor.equals(flavor))
      {
         throw new UnsupportedFlavorException(flavor);
      }
      
      return image;
   }
}