/*
** Module   : WindowBorder.java
** Abstract : Window border implementation.
**
** Copyright (c) 2016-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 HC  20160428 Created initial version
** 002 OM  20170320 Added support for Themes.
** 003 OM  20170502 Additional support for Windows8Theme added.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui.client.gui;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.border.*;
import com.goldencode.p2j.ui.client.gui.driver.*;
import com.goldencode.p2j.ui.client.gui.theme.*;
import com.goldencode.p2j.ui.client.widget.*;

/**
 * The class implements border for main and modal top-level windows.
 */
public class WindowBorder
implements Border<GuiOutputManager>
{
   /** The border size on each side. */
   private final NativeInsets ninsets;
   
   /** Cached driver. */
   private GuiDriver<?, ?> gd;
   
   /** Cached coordinates conversion support. */
   private CoordinatesConversion cc;
   
   /**
    * Creates main window border.
    * 
    * @return   New border instance.
    */
   public static Border<GuiOutputManager> createWindowBorder()
   {
      return new WindowBorder(ThemeManager.getCurrentTheme().getWindowBorderInsets(false));
   }
   
   /**
    * Creates modal window border.
    * 
    * @return   New border instance.
    */
   public static Border<GuiOutputManager> createModalWindowBorder()
   {
      return new WindowBorder(ThemeManager.getCurrentTheme().getWindowBorderInsets(true));
   }
   
   /**
    * Default constructor.
    *
    * @param    ins
    *           Border sizes.
    */
   private WindowBorder(NativeInsets ins)
   {
      ninsets = new NativeInsets(ins.top, ins.left, ins.bottom, ins.right);
      
      OutputManager<?> screen = OutputManager.instance();
      gd = (GuiDriver<?,?>) screen.getInstanceDriver();
      cc = screen.coordinates();
   }

   /**
    * Get insets introduced by the border.
    * 
    * @return border insets.
    */
   @Override
   public Insets insets()
   {
      return cc.insetsFromNative(ninsets);
   }

   /**
    * Draw border.
    * 
    * @param   widget
    *          Base widget. 
    * @param   size
    *          Border dimensions.
    */
   @Override
   public void drawBorder(Widget<GuiOutputManager> widget, Dimension size)
   {
      Theme theme = ThemeManager.getCurrentTheme();
      NativeDimension d = cc.dimensionToNative(size);
      boolean active = WindowManager.isWindowActive(widget.topLevelWindow());
      theme.drawWindowBorder(gd, 0, 0, d.width, d.height, ninsets, active);
   }
}
