/*
 ** Module   : FwdEnhancedTheme.java
 ** Abstract : Enhancements for Windows 10 theme manager.
 **
 ** Copyright (c) 2018, Golden Code Development Corporation.
 **
 ** -#- -I- --Date-- ---------------------------------Description---------------------------------
 ** 001 OM  20180221 First version.
 ** 002 SVL 20180529 Support for enhanced browse coloring.
 */

/*
 ** This program is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU Affero General Public License as
 ** published by the Free Software Foundation, either version 3 of the
 ** License, or (at your option) any later version.
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU Affero General Public License for more details.
 **
 ** You may find a copy of the GNU Affero GPL version 3 at the following
 ** location: https://www.gnu.org/licenses/agpl-3.0.en.html
 **
 ** Additional terms under GNU Affero GPL version 3 section 7:
 **
 **   Under Section 7 of the GNU Affero GPL version 3, the following additional
 **   terms apply to the works covered under the License.  These additional terms
 **   are non-permissive additional terms allowed under Section 7 of the GNU
 **   Affero GPL version 3 and may not be removed by you.
 **
 **   0. Attribution Requirement.
 **
 **     You must preserve all legal notices or author attributions in the covered
 **     work or Appropriate Legal Notices displayed by works containing the covered
 **     work.  You may not remove from the covered work any author or developer
 **     credit already included within the covered work.
 **
 **   1. No License To Use Trademarks.
 **
 **     This license does not grant any license or rights to use the trademarks
 **     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
 **     of Golden Code Development Corporation. You are not authorized to use the
 **     name Golden Code, FWD, or the names of any author or contributor, for
 **     publicity purposes without written authorization.
 **
 **   2. No Misrepresentation of Affiliation.
 **
 **     You may not represent yourself as Golden Code Development Corporation or FWD.
 **
 **     You may not represent yourself for publicity purposes as associated with
 **     Golden Code Development Corporation, FWD, or any author or contributor to
 **     the covered work, without written authorization.
 **
 **   3. No Misrepresentation of Source or Origin.
 **
 **     You may not represent the covered work as solely your work.  All modified
 **     versions of the covered work must be marked in a reasonable way to make it
 **     clear that the modified work is not originating from Golden Code Development
 **     Corporation or FWD.  All modified versions must contain the notices of
 **     attribution required in this license.
 */
package com.goldencode.p2j.ui.client.gui.theme;

import com.goldencode.p2j.ui.ColorRgb;
import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.gui.*;
import com.goldencode.p2j.ui.client.gui.driver.*;

import java.awt.*;

/**
 * This class enhances the standard Windows 10 GUI with some customization requested by customers
 * and other visual improvements over the standard theme. Reuses all graphic images from Windows10
 * theme.
 */
public class FwdEnhancedTheme
extends Windows10Theme
{
   /**
    * Get the name of this {@code Theme}.
    *
    * @return  the name of this theme.
    */
   @Override
   public String getName()
   {
      return "FwdEnhancedTheme";
   }
   
   /**
    * Draw label-style rectangle around the browse header cell. Uses a darker color for
    * separators for a better contrast with cell background.
    *
    * @param   column
    *          The {@code BrowseColumnGuiImpl} widget.
    * @param   gd
    *          The {@code GuiDriver} used for drawing operations.
    * @param   xOffset
    *          Left offset, in pixels.
    * @param   yOffset
    *          Top offset, in pixels.
    * @param   w
    *          The border width in pixels.
    * @param   h
    *          The border height in pixels.
    */
   protected void drawBrowseColumnLabelBorder(BrowseColumnGuiImpl column,
                                              GuiDriver gd,
                                              int xOffset,
                                              int yOffset,
                                              int w,
                                              int h)
   {
      // set the border color
      gd.setColor(column.isDrawnAsPressed() ? new Color(0x21A2DE)
                                            : new Color(0x737173));
      // and draw the border itself depending on pressed flag
      if (isBrowseLabelBorderDrawQuirk(column))
      {
         NativePoint clipPoint = new NativePoint(xOffset, yOffset);
         NativeRectangle clipRect =
               new NativeRectangle(clipPoint, new NativeDimension(w, h + 1));
         gd.draw(clipPoint, clipRect, () -> gd.drawRect(0, 0, w, h));
      }
      else
      {
         gd.drawRect(xOffset, yOffset, w, h);
      }
   }

   /**
    * Get the cell separator color. By default uses a darker color for separators for a better
    * contrast with cell background.
    *
    * @param   browse
    *          The browse widget.
    *
    * @return  cell separator color.
    */
   @Override
   public ColorRgb getBrowseCellSeparatorColor(BrowseGuiImpl browse)
   {
      return browse.config().ehSeparatorsColor != null ?
                browse.config().ehSeparatorsColor :
                new ColorRgb(new Color(0x737173));
   }
}
