/*
** Module   : ListModel.java
** Abstract : List data model.
**
** Copyright (c) 2010-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ----------------Description-----------------
** 001 SIY 20100629          Created initial version
** 002 SIY 20100804          Changes in imports, some refactorings.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui.client.model;

import java.util.*;

/**
 * List data model.
 * 
 * @param   <T>
 *          Type of the item stored in the model. 
 */
public class ListModel<T>
implements Iterable<T>
{
   /** Item container. */
   private ArrayList<T> list = new ArrayList<T>();

   /**
    * @see java.lang.Iterable#iterator()
    */
   @Override
   public Iterator<T> iterator()
   {
      return list.iterator();
   }
   
   /**
    * Add item into model.
    * 
    * @param   item
    *          Item to add.
    */
   public void add(T item)
   {
      list.add(item);
   }

   /**
    * Add item into model at specified position.
    * 
    * @param   index
    *          Position in model.
    * @param   item
    *          Item to add.
    */
   public void add(int index, T item)
   {
      list.add(index, item);
   }
   
   /**
    * Remove all items.
    */
   public void clear()
   {
      list.clear();
   }

   /**
    * Get index of item with specified value.
    * 
    * @param   item
    *          Item value to locate.
    *          
    * @return  index of the item of -1 if no such value present in model.
    */
   public int indexOf(T item)
   {
      return list.indexOf(item);
   }

   /**
    * Retrieve item at specified index.
    *  
    * @param   index
    *          Item index.
    *          
    * @return  retrieved item or <code>null</code> if index is out of range.
    */
   public T get(int index)
   {
      return list.get(index);
   }

   /**
    * Get count of items in the model.
    * 
    * @return  model items count.
    */
   public int size()
   {
      return list.size();
   }
   
   /**
    * Remove item at specified position.
    * 
    * @param   index
    *          Item index.
    */
   public void removeAt(int index)
   {
      list.remove(index);
   }
}