/*
** Module   : AsyncRequest.java
** Abstract : defines the async request resource, created during async procedure calls. 
**
** Copyright (c) 2013-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 CA  20130813 Created initial version.
** 002 CA  20130927 Added LegacyResource annotation, to determine resource type.
** 003 EVL 20131008 Adding legacy attributes and methods annotation.
** 004 EVL 20160224 Javadoc fixes to make compatible with Oracle Java 8 for Solaris 10.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.util;

/**
 * Define the async request resource, created during async procedure calls. Adds support for these
 * attributes and methods:
 * <table summary="">
 * <tr>
 *    <td>CANCELLED</td>
 *    <td>{@link #isCancelled()}</td>
 * </tr>
 * <tr>
 *    <td>COMPLETE</td>
 *    <td>{@link #isComplete()}</td>
 * </tr>
 * <tr>
 *    <td>EVENT-PROCEDURE</td>
 *    <td>{@link #getEventProcedure()}</td>
 * </tr>
 * <tr>
 *    <td>EVENT-PROCEDURE-CONTEXT</td>
 *    <td>{@link #getEventProcedureContext()}</td>
 * </tr>
 * <tr>
 *    <td>PERSISTENT-PROCEDURE</td>
 *    <td>{@link #getPersistentProcedure()}</td>
 * </tr>
 * <tr>
 *    <td>PROCEDURE-NAME</td>
 *    <td>{@link #getProcedureName()}</td>
 * </tr>
 * <tr>
 *    <td>QUIT</td>
 *    <td>{@link #isQuit()}</td>
 * </tr>
 * <tr>
 *    <td>STOP</td>
 *    <td>{@link #isStop()}</td>
 * </tr>
 * <tr>
 *    <td>SERVER</td>
 *    <td>{@link #getServerHandle()}</td>
 * </tr>
 * </table>
 */
@LegacyResource(resource = LegacyResource.ASYNC_REQUEST)
public interface AsyncRequest
extends CommonHandleChain,
        Remotable,
        Errorable
{
   /**
    * Get the <code>CANCELLED</code> attribute of this async request.
    * 
    * @return   See above.
    */
   @LegacyAttribute(name = "CANCELLED")
   public logical isCancelled();

   /**
    * Get the <code>COMPLETE</code> attribute of this async request.
    * 
    * @return   See above.
    */
   @LegacyAttribute(name = "COMPLETE")
   public logical isComplete();

   /**
    * Get the <code>EVENT-PROCEDURE</code> attribute of this async request.
    * 
    * @return   See above.
    */
   @LegacyAttribute(name = "EVENT-PROCEDURE")
   public character getEventProcedure();

   /**
    * Get the <code>EVENT-PROCEDURE-CONTEXT</code> attribute of this async request.
    * 
    * @return   See above.
    */
   @LegacyAttribute(name = "EVENT-PROCEDURE-CONTEXT")
   public handle getEventProcedureContext();

   /**
    * Get the <code>PERSISTENT-PROCEDURE</code> attribute of this async request.
    * 
    * @return   See above.
    */
   @LegacyAttribute(name = "PERSISTENT-PROCEDURE")
   public handle getPersistentProcedure();

   /**
    * Get the <code>PROCEDURE-NAME</code> attribute of this async request.
    * 
    * @return   See above.
    */
   @LegacyAttribute(name = "PROCEDURE-NAME")
   public character getProcedureName();

   /**
    * Get the <code>QUIT</code> attribute of this async request.
    * 
    * @return   See above.
    */
   @LegacyAttribute(name = "QUIT")
   public logical isQuit();

   /**
    * Get the <code>STOP</code> attribute of this async request.
    * 
    * @return   See above.
    */
   @LegacyAttribute(name = "STOP")
   public logical isStop();
}