/*
** Module   : CommonField.java
** Abstract : Define getters and setters for various widget and field common attributes.
**
** Copyright (c) 2013-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- -----------------------Description------------------------
** 001 CA  20130220 Created initial version.
** 002 EVL 20130911 Adding legacy attributes and methods annotation.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.util;

/**
 * Defines setters and getters for the FORMAT, COLUMN-LABEL, DATA-TYPE and LABEL attributes. 
 */
public interface CommonField
{
   /**
    * Getter of FORMAT attribute.
    *
    * @return   current value of FORMAT attribute.
    */
   @LegacyAttribute(name = "FORMAT")
   public character getFormat();

   /**
    * Setter of FORMAt attribute.
    *
    * @param   format
    *          The new value.
    */
   @LegacyAttribute(name = "FORMAT", setter = true)
   public void setFormat(character format);

   /**
    * Setter of FORMAt attribute.
    *
    * @param   format
    *          The new value.
    */
   @LegacyAttribute(name = "FORMAT", setter = true)
   public void setFormat(String format);

   /**
    * Getter of COLUMN-LABEL attribute.
    *
    * @return   current value of COLUMN-LABEL attribute.
    */
   @LegacyAttribute(name = "COLUMN-LABEL")
   public character getColumnLabel();

   /**
    * Setter of COLUMN-LABEL attribute.
    * 
    * @param    columnLabel
    *           New value of a attribute.
    */
   @LegacyAttribute(name = "COLUMN-LABEL", setter = true)
   public void setColumnLabel(character columnLabel);

   /**
    * Setter of COLUMN-LABEL attribute.
    * 
    * @param    columnLabel
    *           New value of a attribute.
    */
   @LegacyAttribute(name = "COLUMN-LABEL", setter = true)
   public void setColumnLabel(String columnLabel);
 
   /**
    * Returns the value of the DATA-TYPE attribute
    *
    * @return   Current value of the DATA-TYPE attribute.
    */
   @LegacyAttribute(name = "DATA-TYPE")
   public character getDataType();
 
   /**
    * Set the <code>DATA-TYPE</code> attribute..
    *
    * @param    dataType
    *           The DATA-TYPE to be set.
    */
   @LegacyAttribute(name = "DATA-TYPE", setter = true)
   public void setDataType(String dataType);

   /**
    * Set the <code>DATA-TYPE</code> attribute..
    *
    * @param    dataType
    *           The DATA-TYPE to be set.
    */
   @LegacyAttribute(name = "DATA-TYPE", setter = true)
   public void setDataType(character dataType); 

   /**
    * Returns the value of the LABEL attribute
    *
    * @return    Current value of the LABEL attribute.
    */
   @LegacyAttribute(name = "LABEL")
   public character getLabel();

   /**
    * Set the <code>LABEL</code> attribute of handle.
    *
    * @param    label
    *           The LABEL to be set.
    */
   @LegacyAttribute(name = "LABEL", setter = true)
   public void setLabel(String label);

   /**
    * Set the <code>LABEL</code> attribute of handle.
    *
    * @param    label
    *           The LABEL to be set.
    */
   @LegacyAttribute(name = "LABEL", setter = true)
   public void setLabel(character label);
}