/*
** Module   : PrintOptions.java
** Abstract : Container class for print options.
**
** Copyright (c) 2017-2018, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 HC  20171024 Initial version.
** 002 EVL 20180626 Changing default page size to LETTER.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
**
** Additional terms under GNU Affero GPL version 3 section 7:
**
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
**
**   0. Attribution Requirement.
**
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
**
**   1. No License To Use Trademarks.
**
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
**
**   2. No Misrepresentation of Affiliation.
**
**     You may not represent yourself as Golden Code Development Corporation or FWD.
**
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
**
**   3. No Misrepresentation of Source or Origin.
**
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/
package com.goldencode.p2j.util;

import java.io.*;

/**
 * Container class for print options.
 */
public class PrintOptions
implements Serializable
{
   /** Page size */
   public PageSize pageSize = PageSize.LETTER;

   /** Page orientation */
   public PageOrientation pageOrientation = PageOrientation.PORTRAIT;

   /** Top page margin in millimeters */
   public float marginTop = 0;

   /** Left page margin in millimeters */
   public float marginLeft = 0;

   /** Bottom page margin in millimeters */
   public float marginBottom = 0;

   /** Right page margin in millimeters */
   public float marginRight = 0;

   /**
    * Default ctor.
    */
   public PrintOptions()
   {
   }

   /**
    * Copy ctor.
    *
    * @param   other
    *          The other instance.
    */
   public PrintOptions(PrintOptions other)
   {
      apply(other);
   }

   /**
    * Assigns all fields from other instance to this instance.
    *
    * @param   other
    *          The other instance.
    */
   public void apply(PrintOptions other)
   {
      pageSize = other.pageSize;
      pageOrientation = other.pageOrientation;
      marginTop = other.marginTop;
      marginLeft = other.marginLeft;
      marginBottom = other.marginBottom;
      marginRight = other.marginRight;
   }

   /**
    * Available page sizes.
    */
   public enum PageSize
   {
      LETTER,
      LEGAL,
      A0,
      A1,
      A2,
      A3,
      A4,
      A5,
      A6
   }

   /**
    * Page orientations.
    */
   public enum PageOrientation
   {
      PORTRAIT,
      LANDSCAPE
   }
}
