/*
** Module   : ServerPropertiesInspector.java
** Abstract : Progress 4GL compatible System properties checking methods interface.
**
** Copyright (c) 2014-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 EVL 20140327 Created initial version.
** 002 CA  20141029 Added API to interrogate the server-side about the client's OS compatibility 
*                   mode.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.util;

/**
 * Interface describing the methods that gets different system variables and characteristics.
 * <p>
 * The accessor classes implementing this interface provide the way to access environment worker
 * classes on the server side.
 *
 * @author    EVL
 */
public interface ServerPropertiesInspector
{
   /**
    * Gets the current version string of the Progress compatible environment.
    * <p>
    * If no value is found via this lookup, then the default value of
    * "10.2B" will be returned.
    *
    * @return   The current version string.
    */
   public String getVersion();

   /**
    * Reports if the current server context is running in batch mode.
    * <p>
    * If no value is found via this lookup, then the default value of
    * <code>false</code> will be returned.
    *
    * @return   <code>true</code> if the current context is in batch mode.
    */
   public boolean isBatchMode();
   
   /**
    * Check the server-side configuration of this client - if it is running in windows 
    * compatibility mode or not.
    * 
    * @return   <code>true</code> if the client is running in windows compatibility mode.
    */
   public boolean isUnderWindowsFamily();
}