/*
** Module   : WrappedResource.java
** Abstract : interface for objects that can be checked for validity
**
** Copyright (c) 2005-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- --------------------------Description---------------------------
** 001 GES 20051125   @23506 Provides an interface for objects that can
**                           be checked for validity.
** 002 CA  20090227   @41381 Added isUnknown method - returns true if the
**                           wrapped resource is unknown.
** 003 OM  20130304          Refactored isValid and isUnknown of WrappedResource to valid 
**                           and unknown.
** 004 CA  20150502          The resource ID needs to be kept at the resource, so it can be 
**                           accessed after the resource is deleted.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.util;

/**
 * An API for objects that can be checked for validity.
 */
public interface WrappedResource
{
   /**
    * Reports if this object is valid for use.  
    *
    * @return   <code>true</code> if we are valid (can be used).
    */
   public boolean valid();
   
   /**
    * Reports if this object is unknown.  
    *
    * @return   <code>true</code> if object is unknown.
    */
   public boolean unknown();
   
   /**
    * Get this resource's ID, if is already set.
    * <p>
    * This is for internal usage only.  If you need the resource ID, use {@link handle#resourceId},
    * which will compute it and save it at the resource, too.
    * 
    * @return   The resource's ID or <code>null</code> if not set.
    */
   public Long id();
   
   /**
    * Set this resource's ID.
    * 
    * @param    id
    *           The resource's ID.
    */
   public void id(long id);
}