/*
** Module   : XCommon
** Abstract : XML related generic common interface.
**
** Copyright (c) 2013-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 EVL 20130214 Created initial version.
** 002 EVL 20131002 Adding legacy attributes and methods annotation.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.xml;

import com.goldencode.p2j.util.*;

/**
 * Interface decribing the common features for XML related objects.
 */
public interface XCommon
extends WrappedResource
{
   // Attributes
   /**
    * Gets the value of the XML schema file for elements with no namespaces.  
    *
    * @return   The value of the nonamespace-schema-location attribute.
    */
   @LegacyAttribute(name = "NONAMESPACE-SCHEMA-LOCATION")
   public character getNonamespaceSchemaLocation();
   
   /**
    * Setting the new value for the nonamespace-schema-location attribute.
    * 
    * @param    location
    *           New value of the nonamespace-schema-location attribute to set.
    */
   @LegacyAttribute(name = "NONAMESPACE-SCHEMA-LOCATION", setter = true)
   public void setNonamespaceSchemaLocation(character location);
   
   /**
    * Setting the new value for the nonamespace-schema-location attribute.
    * 
    * @param    location
    *           New value of the nonamespace-schema-location attribute to set.
    */
   @LegacyAttribute(name = "NONAMESPACE-SCHEMA-LOCATION", setter = true)
   public void setNonamespaceSchemaLocation(String location);
   
   // Methods
   /**
    * Adding new match pair for namespace schema and physical file location.
    * 
    * @param    namespace
    *           Target namespace of the schema or am empty string or Unknown value.
    * @param    location
    *           Location of the XML schema file.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "ADD-SCHEMA-LOCATION")
   public logical addSchemaLocation(character namespace, character location);

   /**
    * Adding new match pair for namespace schema and physical file location.
    * 
    * @param    namespace
    *           Target namespace of the schema or am empty string or Unknown value.
    * @param    location
    *           Location of the XML schema file.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "ADD-SCHEMA-LOCATION")
   public logical addSchemaLocation(String namespace, character location);

   /**
    * Adding new match pair for namespace schema and physical file location.
    * 
    * @param    namespace
    *           Target namespace of the schema or am empty string or Unknown value.
    * @param    location
    *           Location of the XML schema file.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "ADD-SCHEMA-LOCATION")
   public logical addSchemaLocation(character namespace, String location);

   /**
    * Adding new match pair for namespace schema and physical file location.
    * 
    * @param    namespace
    *           Target namespace of the schema or am empty string or Unknown value.
    * @param    location
    *           Location of the XML schema file.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "ADD-SCHEMA-LOCATION")
   public logical addSchemaLocation(String namespace, String location);
   
   /**
    * Removes the specified attribute.
    * 
    * @param    name
    *           The name of the attribute to remove.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "REMOVE-ATTRIBUTE")
   public logical removeAttribute(character name);
   
   /**
    * Removes the specified attribute.
    * 
    * @param    name
    *           The name of the attribute to remove.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "REMOVE-ATTRIBUTE")
   public logical removeAttribute(String name);
   
   /**
    * Removes the specified attribute.
    * 
    * @param    index
    *           The 1-based index of the attribute to remove.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "REMOVE-ATTRIBUTE")
   public logical removeAttribute(NumberType index);
   
   /**
    * Removes the specified attribute.
    * 
    * @param    index
    *           The 1-based index of the attribute to remove.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "REMOVE-ATTRIBUTE")
   public logical removeAttribute(double index);

   /**
    * Removes the specified attribute.
    * 
    * @param    name
    *           The name of the attribute to remove.
    * @param    uri
    *           The URI of the attribute to remove.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "REMOVE-ATTRIBUTE")
   public logical removeAttribute(character name, character uri);
   
   /**
    * Removes the specified attribute.
    * 
    * @param    name
    *           The name of the attribute to remove.
    * @param    uri
    *           The URI of the attribute to remove.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "REMOVE-ATTRIBUTE")
   public logical removeAttribute(String name, character uri);
   
   /**
    * Removes the specified attribute.
    * 
    * @param    index
    *           The 1-based index of the attribute to remove.
    * @param    uri
    *           The URI of the attribute to remove.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "REMOVE-ATTRIBUTE")
   public logical removeAttribute(NumberType index, character uri);
   
   /**
    * Removes the specified attribute.
    * 
    * @param    index
    *           The 1-based index of the attribute to remove.
    * @param    uri
    *           The URI of the attribute to remove.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "REMOVE-ATTRIBUTE")
   public logical removeAttribute(double index, character uri);

   /**
    * Removes the specified attribute.
    * 
    * @param    name
    *           The name of the attribute to remove.
    * @param    uri
    *           The URI of the attribute to remove.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "REMOVE-ATTRIBUTE")
   public logical removeAttribute(character name, String uri);
   
   /**
    * Removes the specified attribute.
    * 
    * @param    name
    *           The name of the attribute to remove.
    * @param    uri
    *           The URI of the attribute to remove.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "REMOVE-ATTRIBUTE")
   public logical removeAttribute(String name, String uri);
   
   /**
    * Removes the specified attribute.
    * 
    * @param    index
    *           The 1-based index of the attribute to remove.
    * @param    uri
    *           The URI of the attribute to remove.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "REMOVE-ATTRIBUTE")
   public logical removeAttribute(NumberType index, String uri);
   
   /**
    * Removes the specified attribute.
    * 
    * @param    index
    *           The 1-based index of the attribute to remove.
    * @param    uri
    *           The URI of the attribute to remove.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "REMOVE-ATTRIBUTE")
   public logical removeAttribute(double index, String uri);
}