/*
** Module   : filesys.h
** Abstract : Header file for file system functions.
**            This is used by filesys.c, filesys_(linix|win).c
**
** Copyright (c) 2013-2018, Golden Code Development Corporation.
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 EVL 20130329 Created initial version for file system declarations.
** 002 EVL 20180206 Fix file separator platform dependency.
** 003 EVL 20180530 Adding get/set current directory support.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/
#define _GNU_SOURCE
#include <stdio.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/types.h>
#include <errno.h> 
#include <string.h> 
#include <stdlib.h> 
#include <fcntl.h> 
#include <jni.h>
#include <time.h>

// status bitfield for unknown type
#define UNKNOWN 0

// status bitfield for directory type
#define DIR (1<<0)

// status bitfield for file type
#define FILE (1<<1)

// status bitfield for symbolic link type
#define SYMLINK (1<<2)

// status bitfield for device type
#define DEVICE  (1<<3)

// status bitfield for pipe type
#define PIPE (1<<4)

//status bitfield for hidden type
#define HIDDEN (1<<5)

// status bitfield for UNIX OS
#define UNIX (1<<6)

// status bitfield for WINDOWS OS
#define WINDOWS (1<<7)

// Getting the symbolic link status (supports from Windows Vista)
int getSymLinkStatus(JNIEnv *, const char *, int);

// Platform dependent open file routine
int open_native(const char *, int);

// Platform dependent path separator constant
char getFileSeparator();